Package: enmpa
Title: Ecological Niche Modeling using Presence-Absence Data
Version: 0.2.4
Authors@R: c(
    person("Luis F.", "Arias-Giraldo", email = "lfarias.giraldo@gmail.com",
    role = c("aut", "cre"), comment = c(ORCID = "0000-0003-4861-8064")),
    person("Marlon E.", "Cobos", email = "manubio13@gmail.com", 
    role = "aut", comment = c(ORCID = "0000-0002-2611-1767")),
    person("A. Townsend", "Peterson", email = "town@ku.edu", role = "ctb",
    comment = c(ORCID = "0000-0003-0243-2379"))
    )
Maintainer: Luis F. Arias-Giraldo <lfarias.giraldo@gmail.com>
Date: 2026-05-28
Description: A set of tools to perform Ecological Niche Modeling with
    presence-absence data. It includes algorithms for data partitioning, 
    model fitting, calibration, evaluation, selection, and prediction. 
    Other functions help to explore signals of ecological niche using univariate
    and multivariate analyses, and model features such as variable response 
    curves and variable importance. Unique characteristics of this package are
    the ability to exclude models with concave quadratic responses, and the
    option to clamp model predictions to specific variables. These tools are
    implemented following principles proposed in
    Cobos et al., (2022) <doi:10.17161/bi.v17i.15985>, 
    Cobos et al., (2019) <doi:10.7717/peerj.6281>, 
    and Peterson et al., (2008) <doi:10.1016/j.ecolmodel.2007.11.008>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 8.0.0
URL: https://github.com/Luisagi/enmpa
BugReports: https://github.com/Luisagi/enmpa/issues
Imports: doSNOW, ellipse, foreach, graphics, methods, mgcv, parallel,
        Rcpp, snow, stats, terra, utils, vegan
LazyData: true
Depends: R (>= 3.5.0)
LinkingTo: Rcpp
NeedsCompilation: yes
Packaged: 2026-05-28 07:59:54 UTC; luisagi
Author: Luis F. Arias-Giraldo [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4861-8064>),
  Marlon E. Cobos [aut] (ORCID: <https://orcid.org/0000-0002-2611-1767>),
  A. Townsend Peterson [ctb] (ORCID:
    <https://orcid.org/0000-0003-0243-2379>)
Repository: CRAN
Date/Publication: 2026-05-28 08:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-28 11:55:24 UTC; unix
Archs: enmpa.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(print,enmpa_calibration)
S3method(print,enmpa_fitted_models)
S3method(summary,enmpa_calibration)
S3method(summary,enmpa_fitted_models)
export(aux_string_comb)
export(aux_var_comb)
export(calibration_glm)
export(evaluation_stats)
export(fit_glms)
export(fit_selected)
export(get_formulas)
export(get_formulas_main)
export(independent_eval01)
export(independent_eval1)
export(jackknife)
export(kfold_partition)
export(model_selection)
export(model_validation)
export(new_enmpa_calibration)
export(new_enmpa_fitted_models)
export(niche_signal)
export(niche_signal_permanova)
export(niche_signal_univariate)
export(optimize_metrics)
export(plot_importance)
export(plot_jk)
export(plot_niche_signal)
export(plot_niche_signal_permanova)
export(plot_niche_signal_univariate)
export(predict_glm)
export(predict_selected)
export(proc_enm)
export(resp2var)
export(response_curve)
export(var_importance)
importFrom(Rcpp,sourceCpp)
importFrom(doSNOW,registerDoSNOW)
importFrom(ellipse,ellipse)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(grDevices,hcl.colors)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(mgcv,gam)
importFrom(parallel,clusterExport)
importFrom(snow,makeSOCKcluster)
importFrom(snow,stopCluster)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cov)
importFrom(stats,deviance)
importFrom(stats,ecdf)
importFrom(stats,glm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,predict.glm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,update)
importFrom(stats,var)
importFrom(terra,as.data.frame)
importFrom(terra,extract)
importFrom(terra,minmax)
importFrom(terra,predict)
importFrom(terra,rast)
importFrom(terra,setValues)
importFrom(utils,combn)
importFrom(utils,txtProgressBar)
importFrom(utils,write.table)
importFrom(vegan,adonis2)
useDynLib(enmpa, .registration = TRUE)

enmpa: Ecological Niche Modeling for Presence-absence Data
================
Luis F. Arias-Giraldo, Marlon E. Cobos, A. Townsend Peterson

- [Package description](#package-description)
- [Installation](#installation)
- [Packages required](#packages-required)
- [Example data](#example-data)
- [Example](#example)
  - [Detecting signals of ecological
    niche](#detecting-signals-of-ecological-niche)
  - [Model formulas](#model-formulas)
  - [Model calibration and selection](#model-calibration-and-selection)
  - [Fitting and predictions for selected
    models](#fitting-and-predictions-for-selected-models)
  - [Consensus models](#consensus-models)
  - [Response Curves](#response-curves)
  - [Two-way interactions](#two-way-interactions)
  - [Variable importance](#variable-importance)
  - [Final model evaluation with independent
    data](#final-model-evaluation-with-independent-data)
  - [Literature](#literature)

<!-- README.md is generated from README.Rmd. Please edit that file -->
<!-- badges: start -->

[![R-CMD-check](https://github.com/Luisagi/enmpa/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Luisagi/enmpa/actions/workflows/R-CMD-check.yaml)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/enmpa)](https://cran.r-project.org/package=enmpa)
[![](https://cranlogs.r-pkg.org/badges/grand-total/enmpa)](https://cran.r-project.org/package=enmpa)
<!-- badges: end -->

<img src="man/figures/enmpa_logo_100dpi.png" align="right" width="166"/></a>

<hr>

## Package description

`enmpa` is an R package that contains a set of tools to perform
Ecological Niche Modeling using presence-absence data. Some of the main
functions help perform data partitioning, model calibration, model
selection, variable response exploration, and model projection.

##### Citation

If you use `enmpa` in your research, please cite the following paper:

Arias-Giraldo, Luis F., and Marlon E. Cobos. 2024. “enmpa: An R Package
for Ecological Niche Modeling Using Presence-Absence Data and
Generalized Linear Models”. Biodiversity Informatics 18.
<https://doi.org/10.17161/bi.v18i.21742>.

<br>

<hr>

## Installation

To install the stable version of `enmpa` from CRAN use:

``` r
install.packages("enmpa")
```

You can install the development version of `enmpa` from
[GitHub](https://github.com/Luisagi/enmpa) with:

``` r
# install.packages("remotes")
remotes::install_github("Luisagi/enmpa")
```

<br>

## Packages required

The package `terra` is used to handle spatial data, and `enmpa` is used
to perform ENM.

``` r
library(enmpa)
library(terra)
```

<br>

## Example data

The data used in this example is included in `enmpa`.

``` r
# Species presence absence data associated with environmental variables
data("enm_data", package = "enmpa")

# Data for final model evaluation 
data("test", package = "enmpa")

# Environmental data as raster layers for projections
env_vars <- rast(system.file("extdata", "vars.tif", package = "enmpa"))

# Check the example data
head(enm_data)
#>   Sp     bio_1 bio_12
#> 1  0  4.222687    403
#> 2  0  6.006802    738
#> 3  0  4.079385    786
#> 4  1  8.418489    453
#> 5  0  8.573750    553
#> 6  1 16.934618    319
```

The raster layers for projections were obtained from
[WorldClim](https://worldclim.org/):

- bio_1 = Annual Mean Temperature
- bio_12 = Annual Precipitation

``` r
plot(env_vars, mar = c(1, 1, 2, 4))
```

<img src="man/figures/README-figures-raster_layers-1.png" width="100%" />

<br>

## Example

### Detecting signals of ecological niche

To explore the relevance of the variables to be used in niche models, we
implemented in the methods developed by ([Cobos and Peterson
2022](#ref-cobos2022)). These methods help to identify signals of
ecological niche considering distinct variables and the presence-absence
data. By characterizing the sampling universe, this approach can
determine whether presences and absences can be separated better than
randomly considering distinct environmental factors.

``` r
sn_bio1  <- niche_signal(data = enm_data, variables = "bio_1", 
                         condition = "Sp", method = "univariate")

sn_bio12 <- niche_signal(data = enm_data, variables = "bio_12",
                         condition = "Sp", method = "univariate")
```

``` r
plot_niche_signal(sn_bio1, variables = "bio_1")
plot_niche_signal(sn_bio12, variables = "bio_12")
```

<img src="man/figures/README-figures-niche_signal-1.png" width="50%" /><img src="man/figures/README-figures-niche_signal-2.png" width="50%" />

Based on the univariate test results, the variables bio_1 and bio_12
help to detect signals of ecological niche in our data. In our example,
the species tends to occur in areas with higher annual mean temperatures
(bio_1); whereas, considering annual precipitation (bio_12), the species
seems to be present in areas with lower values. See the function’s
documentation for more information.

<br>

### Model formulas

With `enmpa` you have the possibility to explore multiple model formulas
derived from combinations of variables considering linear (l), quadratic
(q), and product (p) responses. Product refers to pair interactions of
variables.

The function includes the flag `mode` to determine what strategy to use
to combine predictors based on the responses defined in . The options of
mode are:

- **light**.– returns simple iterations of complex formulas.
- **moderate**.– returns a comprehensive number of iterations.
- **intensive**.– returns all possible combination. Very time-consuming
  for 6 or more independent variables.
- **complex**.– returns only the most complex formula.

An example using linear + quadratic responses:

``` r
get_formulas(dependent = "Sp", independent = c("bio_1", "bio_12"),  
             type = "lq", mode = "intensive")
#>  [1] "Sp ~ bio_1"                                    
#>  [2] "Sp ~ bio_12"                                   
#>  [3] "Sp ~ I(bio_1^2)"                               
#>  [4] "Sp ~ I(bio_12^2)"                              
#>  [5] "Sp ~ bio_1 + bio_12"                           
#>  [6] "Sp ~ bio_1 + I(bio_1^2)"                       
#>  [7] "Sp ~ bio_1 + I(bio_12^2)"                      
#>  [8] "Sp ~ bio_12 + I(bio_1^2)"                      
#>  [9] "Sp ~ bio_12 + I(bio_12^2)"                     
#> [10] "Sp ~ I(bio_1^2) + I(bio_12^2)"                 
#> [11] "Sp ~ bio_1 + bio_12 + I(bio_1^2)"              
#> [12] "Sp ~ bio_1 + bio_12 + I(bio_12^2)"             
#> [13] "Sp ~ bio_1 + I(bio_1^2) + I(bio_12^2)"         
#> [14] "Sp ~ bio_12 + I(bio_1^2) + I(bio_12^2)"        
#> [15] "Sp ~ bio_1 + bio_12 + I(bio_1^2) + I(bio_12^2)"
```

<br>

### Model calibration and selection

The function `calibration_glm()` is a general function that allows to
perform the following steps:

- Partition data in k-folds
- Create formulas for candidate models
- Fit and evaluate candidate models
- Select best performing models

Model selection consists of three steps:

1.  A first filter to keep the models with ROC AUC \>= 0.5
    (statistically significant models).
2.  A second filter to maintain only models that meet a
    `selection_criterion` (“TSS”: TSS \>= 0.4; or “ESS”: maximum
    Accuracy - tolerance).
3.  From those, pick the ones with delta AIC \<= 2.

<br>

The results are returned as a list containing:

- Selected models (`*$selected`)
- A summary of statistics for all models (`*$summary`)
- Results obtained from cross-validation for all models
  (`*$calibration_results`)
- Input data used (`*$data`)
- The weights used (`*$weights`)
- The list of partition indices (`*$kfold_index_partition`)

<br>

Now lets run an example of model calibration and selection:

``` r
# Linear + quadratic + products responses
calibration <- calibration_glm(data = enm_data, dependent = "Sp",
                               independent = c("bio_1", "bio_12"),
                               response_type = "lpq",
                               formula_mode = "intensive", 
                               exclude_bimodal = TRUE, 
                               selection_criterion = "TSS",
                               cv_kfolds = 5, verbose = FALSE)
calibration
#> enmpa-class `enmpa_calibration`:
#> $selected             : Selected models (N = 2)
#> $summary              : A summary of statistics for all models. 
#> $calibration_results  : Results obtained from cross-validation for all models. 
#> $data                 : Data used for calibration. 
#> $partitioned_data     : k-fold indexes (k = 5)
#> $weights              : Use of weights (FALSE)
```

Process results:

``` r
## Summary of the calibration
summary(calibration)
#> 
#>                      Summary of enmpa_calibration                  
#> -------------------------------------------------------------------
#> 
#> Number of selected models: 2
#> Number of partitions: (k = 5)
#> Weights used: No
#> Summary of selected models (threshold criteria =  maxTSS ):
#>      ModelID ROC_AUC_mean Accuracy_mean Specificity_mean Sensitivity_mean
#> 1 ModelID_29       0.9003        0.8274           0.8245            0.858
#> 2 ModelID_31       0.9002        0.8305           0.8280            0.856
#>   TSS_mean AIC_weight
#> 1   0.6825  0.3751818
#> 2   0.6840  0.6248182
```

<br>

### Fitting and predictions for selected models

After one or more models are selected, the next steps are the fitting
and projection of these models. In this case we are projecting the
models to the whole area of interest. Models can be transferred with
three options: free extrapolation (‘E’), extrapolation with clamping
(‘EC’), and no extrapolation (‘NE’).

``` r
# Fitting selected models
fits <- fit_selected(calibration)

# Prediction for the two selected models and their consensus
preds_E  <- predict_selected(fits, newdata = env_vars, extrapolation_type = "E",
                             consensus = TRUE)
preds_NE <- predict_selected(fits, newdata = env_vars,extrapolation_type = "NE",
                             consensus = TRUE)

# Visualization
plot(c(preds_E$predictions, preds_NE$predictions),
     main = c("Model ID 29 (E)", "Model ID 31 (E)",
              "Model ID 29 (NE)", "Model ID 31 (NE)"),
     mar = c(1, 1, 2, 5))
```

<img src="man/figures/README-figures-prediction_selected-1.png" width="100%" />

<br>

### Consensus models

An alternative to strict selection of a single model is to use a
consensus model. The main idea is to avoid selecting the best model and
instead rely on multiple models with similar performance.

During the prediction of selected models, we calculated model consensus
using the mean, median, and a weighted average (using Akaike weights)
([Akaike 1973](#ref-akaike1973); [Wagenmakers and Farrell
2004](#ref-wagenmakers2004)).

``` r
# Consensus projections
plot(preds_E$consensus, mar = c(1, 1, 2, 5))
```

<img src="man/figures/README-figures-consensus-1.png" width="100%" />

<br>

### Response Curves

An important step in understanding the ecological niches with these
models is to explore variable response curves. The following lines of
code help to do so:

``` r
# Response Curves for Bio_1 and Bio_2, first selected model 
response_curve(fitted = fits, modelID = "ModelID_29", variable = "bio_1")
response_curve(fitted = fits, modelID = "ModelID_29", variable = "bio_12")
```

<img src="man/figures/README-figures-rcurve_model_ID_1-1.png" width="50%" /><img src="man/figures/README-figures-rcurve_model_ID_1-2.png" width="50%" />

``` r
# Consensus Response Curves for Bio_1 and Bio_2, for both models 
response_curve(fits, variable = "bio_1")
response_curve(fits, variable = "bio_12")
```

<img src="man/figures/README-figures-rcurve_consensus-1.png" width="50%" /><img src="man/figures/README-figures-rcurve_consensus-2.png" width="50%" />

### Two-way interactions

It is useful to examine whether the effect of one variable depends on
the level of other variables. If it does, then we have what is called an
‘interaction’. According to the calibration results from this example,
in both models, the predictor `bio_1:bio_12` was selected. To explore
the interaction of these two variables, the function `resp2var` can help
us to visualize this interaction.

``` r
# Consensus Response Curves for Bio_1 and Bio_2, for both models
resp2var(model = fits,
         modelID = "ModelID_29",
         main = "ModelID 29",
         variable1 = "bio_1",
         variable2 = "bio_12",
         extrapolate = TRUE,
         add_limits = TRUE)

resp2var(model = fits,
         modelID = "ModelID_31",
         main = "ModelID 31",
         variable1 = "bio_1",
         variable2 = "bio_12",
         extrapolate = TRUE,
         add_limits = TRUE)
```

<img src="man/figures/README-figures-rcurve_two-1.png" width="50%" /><img src="man/figures/README-figures-rcurve_two-2.png" width="50%" />

<br>

### Variable importance

Variable importance or contribution to models can be calculated as a
function of the relative deviance explained by each predictor.

Analysis of Deviance for the first selected model:

``` r
anova(fits$glms_fitted$ModelID_29, test = "Chi")
#> Analysis of Deviance Table
#> 
#> Model: binomial, link: logit
#> 
#> Response: Sp
#> 
#> Terms added sequentially (first to last)
#> 
#> 
#>              Df Deviance Resid. Df Resid. Dev  Pr(>Chi)    
#> NULL                          5626     3374.9              
#> bio_1         1   869.35      5625     2505.6 < 2.2e-16 ***
#> I(bio_1^2)    1    46.77      5624     2458.8 7.972e-12 ***
#> I(bio_12^2)   1   239.69      5623     2219.1 < 2.2e-16 ***
#> bio_1:bio_12  1    42.40      5622     2176.7 7.450e-11 ***
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
```

Using a function from `enmpa` you can also explore variable importance
in terms of contribution.

``` r
# Relative contribution of the deviance explained for the first model
var_importance(fitted = fits, modelID = "ModelID_29")
#>      predictor contribution cum_contribution
#> 3  I(bio_12^2)    0.3572673        0.3572673
#> 1        bio_1    0.2919587        0.6492259
#> 2   I(bio_1^2)    0.2031750        0.8524009
#> 4 bio_1:bio_12    0.1475991        1.0000000
```

The function also allows to plot the contributions of the variables for
the two models together which can help with the interpretations:

``` r
# Relative contribution of the deviance explained
vi_both_models <- var_importance(fits)
```

``` r
# Plot
plot_importance(vi_both_models, extra_info = TRUE)
```

<img src="man/figures/README-figures-var_importance-1.png" width="70%" />

The Jackknife function providing a detailed reflection of the impact of
each variable on the overall model, considering four difference
measures: ROC-AUC, TSS, AICc, and Deviance.

``` r
# Jackknife test
jk <- jackknife(data = enm_data,
          dependent = "Sp",
          independent = c("bio_1", "bio_12"),
          response_type = "lpq")

jk
#> $Full_model_stats
#>     ROC_AUC       TSS     AIC Residual_deviance
#> 1 0.9023279 0.6732916 2185.68           2173.68
#> 
#> $Formula
#> [1] "Sp ~ bio_1 + bio_12 + I(bio_1^2) + I(bio_12^2) + bio_1:bio_12"
#> 
#> $Without
#>                ROC_AUC       TSS      AIC Residual_deviance
#> bio_1        0.9015319 0.6707435 2212.307          2202.307
#> bio_12       0.8991734 0.6684712 2226.488          2216.488
#> I(bio_1^2)   0.8978504 0.6651925 2237.051          2227.051
#> I(bio_12^2)  0.9016647 0.6744525 2186.700          2176.700
#> bio_1:bio_12 0.8938496 0.6600718 2243.562          2233.562
#> 
#> $With_only
#>                ROC_AUC       TSS      AIC Residual_deviance
#> bio_1        0.6549700 0.2970911 3373.978          3369.978
#> bio_12       0.6902666 0.2871473 3191.205          3187.205
#> I(bio_1^2)   0.8641785 0.5786819 2599.638          2595.638
#> I(bio_12^2)  0.6916871 0.2894878 3164.568          3160.568
#> bio_1:bio_12 0.8646620 0.5868336 2509.565          2505.565
```

``` r
# Jackknife plots
plot_jk(jk, metric = "TSS")
plot_jk(jk, metric = "AIC")
plot_jk(jk, metric = "ROC_AUC")
plot_jk(jk, metric = "Residual_deviance")
```

<img src="man/figures/README-figures-jackk-1.png" width="50%" /><img src="man/figures/README-figures-jackk-2.png" width="50%" /><img src="man/figures/README-figures-jackk-3.png" width="50%" /><img src="man/figures/README-figures-jackk-4.png" width="50%" />

### Final model evaluation with independent data

Finally, we will evaluate the final models using the “independent_eval”
functions. Ideally, the model should be evaluated with an independent
data set (i.e., data that was not used during model calibration or for
final model fitting).

``` r
# Loading an independent dataset
data("test", package = "enmpa")

# The independent evaluation data are divided into two groups: 
# presences-absences (test_01) and presences-only (test_1).
test_1  <- test[test$Sp == 1,]
test_01 <- test

head(test_1)
#>    Sp       lon      lat
#> 4   1 -117.5543 33.62975
#> 8   1 -100.9363 38.68424
#> 19  1 -110.4748 37.78367
#> 53  1 -106.2348 31.70118
#> 54  1 -112.3623 39.35173
#> 60  1 -109.3399 38.26670
head(test_01)
#>   Sp       lon      lat
#> 1  0 -105.6639 35.81247
#> 2  0 -107.9354 33.37200
#> 3  0 -100.3134 48.96018
#> 4  1 -117.5543 33.62975
#> 5  0 -120.6168 36.59670
#> 6  0 -105.3379 40.08928
```

#### Using presence-absence data.

Using independent data for which presence and absence is known can give
the most robust results. Here an example:

``` r
projections <- list(
  ModelID_29 = preds_E$predictions$ModelID_29,
  ModelID_31 = preds_E$predictions$ModelID_31,
  Consensus_WA = preds_E$consensus$Weighted_average
)

ie_01 <- lapply(projections, function(x){
  independent_eval01(prediction = x,
                     observation = test_01$Sp,
                     lon_lat = test_01[, c("lon", "lat")])
})

ie_01
#> $ModelID_29
#>     Threshold_criteria Threshold   ROC_AUC False_positive_rate Accuracy
#> 204                ESS 0.1365298 0.9570991          0.08988764     0.91
#> 175             maxTSS 0.1170256 0.9570991          0.10112360     0.91
#> 196              SEN90 0.1311493 0.9570991          0.10112360     0.90
#>     Sensitivity Specificity       TSS
#> 204   0.9090909   0.9101124 0.8192033
#> 175   1.0000000   0.8988764 0.8988764
#> 196   0.9090909   0.8988764 0.8079673
#> 
#> $ModelID_31
#>     Threshold_criteria Threshold   ROC_AUC False_positive_rate Accuracy
#> 239                ESS 0.1579818 0.9570991          0.08988764     0.91
#> 205             maxTSS 0.1354130 0.9570991          0.10112360     0.91
#> 218              SEN90 0.1440422 0.9570991          0.10112360     0.90
#>     Sensitivity Specificity       TSS
#> 239   0.9090909   0.9101124 0.8192033
#> 205   1.0000000   0.8988764 0.8988764
#> 218   0.9090909   0.8988764 0.8079673
#> 
#> $Consensus_WA
#>     Threshold_criteria Threshold   ROC_AUC False_positive_rate Accuracy
#> 226                ESS 0.1500930 0.9570991          0.08988764     0.91
#> 192             maxTSS 0.1274123 0.9570991          0.10112360     0.91
#> 210              SEN90 0.1394197 0.9570991          0.10112360     0.90
#>     Sensitivity Specificity       TSS
#> 226   0.9090909   0.9101124 0.8192033
#> 192   1.0000000   0.8988764 0.8988764
#> 210   0.9090909   0.8988764 0.8079673
```

#### Evaluation using presence-only data.

When only presence data is available, the evaluation of performance is
based on the omission error and the partial ROC analysis.

To do this in our example, we will need to define a threshold value. We
can use any of the three threshold values obtained above: ESS, maxTSS or
SEN90.

``` r
# In this example, we will use the weighted average of the two selected models.

# Consensus_WA
Consensus_WA_T1 <- ie_01[["Consensus_WA"]][1, "Threshold"] # ESS
Consensus_WA_T2 <- ie_01[["Consensus_WA"]][2, "Threshold"] # maxTSS
Consensus_WA_T3 <- ie_01[["Consensus_WA"]][3, "Threshold"] # SEN90

aux_list <- list(ESS = Consensus_WA_T1, maxTSS = Consensus_WA_T2,
                  SEN90 = Consensus_WA_T3)

lapply(aux_list, function(th){
  independent_eval1(prediction = projections[["Consensus_WA"]],
                    threshold = th,
                    lon_lat = test_1[, c("lon", "lat")])
  
})
#> $ESS
#>   omission_error threshold Mean_AUC_ratio pval_pROC
#> 1     0.09090909  0.150093       1.634595         0
#> 
#> $maxTSS
#>   omission_error threshold Mean_AUC_ratio pval_pROC
#> 1              0 0.1274123        1.63588         0
#> 
#> $SEN90
#>   omission_error threshold Mean_AUC_ratio pval_pROC
#> 1     0.09090909 0.1394197       1.632866         0
```

### Literature

<div id="refs" class="references csl-bib-body hanging-indent"
entry-spacing="0">

<div id="ref-akaike1973" class="csl-entry">

Akaike, H. 1973. “Information Theory and an Extension of Maximum
Likelihood Principle.” In *Proc. 2nd Int. Symp. On Information Theory*,
267–81.

</div>

<div id="ref-cobos2022" class="csl-entry">

Cobos, Marlon E., and A. Townsend Peterson. 2022. “Detecting Signals of
Species’ Ecological Niches in Results of Studies with Defined Sampling
Protocols: Example Application to Pathogen Niches.” *Biodiversity
Informatics* 17 (May): 50–58. <https://doi.org/10.17161/bi.v17i.15985>.

</div>

<div id="ref-wagenmakers2004" class="csl-entry">

Wagenmakers, Eric-Jan, and Simon Farrell. 2004. “AIC Model Selection
Using Akaike Weights.” *Psychonomic Bulletin & Review* 11 (1): 192–96.
<https://doi.org/10.3758/BF03206482>.

</div>

</div>

#' GLM calibration with presence-absence data
#'
#' @description
#' Creates candidate models based on distinct parameter settings, evaluates
#' models, and selects the ones that perform the best.
#'
#' @usage
#' calibration_glm(data, dependent, independent, weights = NULL,
#'                 response_type = "l", formula_mode = "moderate",
#'                 minvar = 1, maxvar = NULL, user_formulas = NULL,
#'                 cv_kfolds = 5, partition_index = NULL, seed = 1,
#'                 n_threshold = 100, selection_criterion = "TSS",
#'                 exclude_bimodal = FALSE, tolerance = 0.01,
#'                 out_dir = NULL, parallel = FALSE,
#'                 n_cores = NULL, verbose = TRUE)
#'
#' @param data data.frame or matrix of data to be used in model calibration.
#' Columns represent dependent and independent variables.
#' @param dependent (character) name of dependent variable.
#' @param independent (character) vector of name(s) of independent variable(s).
#' @param weights (numeric) a vector with the weights for observations.
#' @param response_type (character) a character string that must contain "l",
#' "p", "q" or a combination of them. l = lineal, q = quadratic,
#' p = interaction between two variables. Default = "l".
#' @param formula_mode (character) a character string to indicate the strategy to
#' create the formulas for candidate models. Options are: "light", "moderate",
#' "intensive", or "complex". Default = "moderate". "complex" returns only the
#' most complex formula defined in `response_type`.
#' @param minvar (numeric) minimum number of independent variables in formulas.
#' @param maxvar (numeric) maximum number of independent variables in formulas.
#' @param user_formulas (character) vector  with formula(s) to test.
#' Default = NULL.
#' @param partition_index list of indices for cross-validation in k-fold. The
#' default, NULL, uses the function \code{\link{kfold_partition}}.
#' @param cv_kfolds (numeric) number of folds to use for k-fold
#' cross-validation exercises. Default = 5. Ignored if `partition_index`
#' is defined.
#' @param seed (numeric) a seed for k-fold partitioning.
#' @param n_threshold (logical) number of threshold values to produce
#' evaluation metrics. Default = 100.
#' @param selection_criterion (character) criterion used to select best models,
#' options are "TSS" and "ESS". Default = "TSS".
#' @param exclude_bimodal (logical) whether to filter out models with one or
#' more variables presenting concave responses. Default = FALSE.
#' @param tolerance (numeric) value to modify the limit value of the metric
#' used to filter models during model selection if none of the models meet
#' initial considerations. Default = 0.01
#' @param out_dir (character) output directory name to save the main calibration
#' results. Default = NULL.
#' @param parallel (logical) whether to run on parallel or sequential.
#' Default = FALSE.
#' @param n_cores (numeric) number of cores to use. Default = number of free
#' processors - 1.
#' @param verbose (logical) whether to print messages and show progress bar.
#' Default = TRUE
#'
#' @return
#' An object of the class enmpa_calibration containing: selected models,
#' a summary of statistics for all models, results obtained in cross-validation
#' for all models, original data used, weights, and data-partition indices used.
#'
#' @details
#' Model evaluation is done considering the ability to predict presences and
#' absences,as well as model fitting and complexity. Model selection consists
#' of three steps: 1) a first filter to keep the models with ROC AUC >= 0.5
#' (statistically significant models), 2) a second filter to maintain only
#' models that meet the `selection_criterion` ("TSS": TSS >= 0.4; or "ESS":
#' maximum Accuracy - `tolerance`), and 3) from those, pick the ones with
#' delta AIC <= 2.
#'
#'
#' `formula_mode` options determine what strategy to iterate the predictors
#' defined in \code{type} for creating models:
#' - **light**.-- returns simple iterations of complex formulas.
#' - **moderate**.-- returns a comprehensive number of iterations.
#' - **intensive**.-- returns all possible combination. Very time-consuming for
#' 6 or more independent variables.
#' - **complex**.-- returns only the most complex formula.
#'
#' @examples
#' # Load species occurrences and environmental data.
#' data("enm_data", package = "enmpa")
#' head(enm_data)
#'
#' # Calibration using linear (l), quadratic (q), products(p) responses.
#' cal_res <- calibration_glm(data = enm_data, dependent = "Sp",
#'                            independent = c("bio_1", "bio_12"),
#'                            response_type = "lpq", formula_mode = "moderate",
#'                            selection_criterion = "TSS", cv_kfolds = 3,
#'                            exclude_bimodal = TRUE, verbose = FALSE)
#' print(cal_res)
#' summary(cal_res)
#'
#' head(cal_res$calibration_results)
#' head(cal_res$summary)
#' head(cal_res$selected)
#' head(cal_res$data)
#'
#' @export
#'
#' @importFrom utils txtProgressBar write.table
#' @importFrom stats aggregate sd
#' @importFrom snow makeSOCKcluster stopCluster
#' @importFrom doSNOW registerDoSNOW
#' @importFrom parallel clusterExport
#' @importFrom foreach foreach %dopar%

calibration_glm <- function(data, dependent, independent, weights = NULL,
                            response_type = "l", formula_mode = "moderate",
                            minvar=1, maxvar = NULL,  user_formulas = NULL,
                            cv_kfolds = 5, partition_index = NULL, seed = 1,
                            n_threshold = 100, selection_criterion = "TSS",
                            exclude_bimodal = FALSE,  tolerance = 0.01,
                            out_dir = NULL, parallel = FALSE,
                            n_cores = NULL, verbose = TRUE) {

  # initial tests
  if (missing(data) | missing(dependent) | missing(dependent)) {
    stop("Arguments 'data', 'dependent', and 'independent' must be defined.")
  }

  if (is.null(user_formulas) & missing(dependent)) {
    stop("Argument 'respuesta' must be defined if 'user_formulas' in NULL.")
  }

  ## 1. Data partitioning: k-Fold Cross-Validation
  if (is.null(partition_index)){
    k <- cv_kfolds
    data_partition <- kfold_partition(data, dependent = dependent, k = k,
                                      seed = seed)
  } else {
    k <- length(partition_index)
    data_partition <- partition_index
  }

  ## 2. Formula combination
  if (is.null(user_formulas)) {
    if (verbose == TRUE) {
      message("\nEstimating formulas combinations for evaluation.")
    }
    user_formulas <- get_formulas(dependent = dependent,
                                  independent = independent,
                                  type = response_type, mode = formula_mode,
                                  minvar=minvar, maxvar = maxvar)

  } else {
    if (verbose == TRUE) {
    message("\nUsing user-defined formulas.")
    }
  }

  if (verbose == TRUE) {
    message("Evaluating a total of ", length(user_formulas), " models.\n")
  }


  # 3. MAIN FUNCTION ___________________________________________________________
  # number of models to test
  iterations <- length(user_formulas)

  if (parallel == FALSE) {
    ## SEQUENTIAL RUNNING
    if (verbose == TRUE) {
      message("Running in Sequential.")
    }

    # time start
    start <- Sys.time()

    ## progress bar
    if (verbose == TRUE) {
      pb <- utils::txtProgressBar(min = 1, max = iterations, style = 3)
    }

    glm_res <- data.frame() # empty df to store results for each iteration.

    for (i in 1:iterations) {
      if (verbose == TRUE) {
        utils::setTxtProgressBar(pb, i)
      }

      res <- model_validation(formula = user_formulas[i],
                              data = data, weights = weights, cv = TRUE,
                              partition_index = data_partition,
                              n_threshold = n_threshold,
                              keep_coefficients = exclude_bimodal)

      # Concatenate by rows each loop iteration
      glm_res <- rbind(glm_res, res)
    }

    # Final time
    if (verbose == TRUE) {
      elapsed <- difftime(Sys.time(), start, units = "secs")
      format_time(elapsed)
    }

  } else {
    ## PARALLEL RUNNING
    if (is.null(n_cores)) {
      n_cores <- parallel::detectCores() - 1
    }

    if (verbose == TRUE) {
      message(paste0("Running in Parallel using ", n_cores, " threads."))
    }

    # time start
    start <- Sys.time()

    ## progress bar
    if (verbose == TRUE) {
      pb <- utils::txtProgressBar(min = 1, max = iterations, style = 3)
      progress <- function(n) {
        utils::setTxtProgressBar(pb, n)
      }
      opts <- list(progress = progress)
    } else {
      opts <- NULL
    }


    ## Make cluster
    cl <- snow::makeSOCKcluster(n_cores)
    # export local function
    #parallel::clusterExport(cl, c("optimize_metrics", "model_validation"))
    doSNOW::registerDoSNOW(cl)

    glm_res <- foreach::foreach(
      i = 1:iterations, .combine = "rbind", .inorder = FALSE,
      .options.snow = opts
    ) %dopar% {
      res <- model_validation(formula = user_formulas[i],
                              data = data, weights = weights, cv = TRUE,
                              partition_index = data_partition,
                              n_threshold = n_threshold,
                              keep_coefficients = exclude_bimodal)
      return(res)
    }

    # Closing cluster
    snow::stopCluster(cl)

    # Final time
    if (verbose == TRUE) {
      elapsed <- difftime(Sys.time(), start, units = "secs")
      format_time(elapsed)
    }
  }

  # 4. data output rearrangement
  if (verbose == TRUE) {
    message("\nPreparing results...\n")   ### Final steps
  }

  # Summary stats: mean and SD calculation for each model based on folds
  stats <- evaluation_stats(evaluation_results = glm_res,
                            bimodal_toexclude = exclude_bimodal)

  # selected models
  sel <- model_selection(evaluation_stats = stats,
                         criterion = selection_criterion,
                         exclude_bimodal = exclude_bimodal,
                         tolerance = 0.01)



  # add data_partition in the input data.
  aux_f <- lapply(
    names(data_partition),
    function(x){data.frame(obs = data_partition[[x]],kfold_ID = x)}
  )

  folds <- do.call(rbind, aux_f)
  folds <- folds[order(folds$obs),]
  data_final <- data.frame(data, kfold_ID = folds$kfold_ID)

  # Add ModelID tag
  fx  <- user_formulas
  mid <- data.frame(ModelID = paste0("ModelID_", 1:length(fx)), Formulas = fx)

  glm_res <- merge(glm_res, mid, by = "Formulas", sort = F)
  glm_res <- glm_res[, c("ModelID", names(glm_res)[names(glm_res) != "ModelID"])]

  stats <- merge(stats, mid, by = "Formulas", sort = F)
  stats <- stats[, c("ModelID", names(stats)[names(stats) != "ModelID"])]

  sel <- merge(sel, mid, by = "Formulas", sort = F)
  sel <- sel[, c("ModelID", names(sel)[names(sel) != "ModelID"])]


  # Final output
  output <- new_enmpa_calibration(
    selected = sel,
    summary = stats,
    calibration_results = glm_res,
    data = data_final,
    weights = weights,
    partitioned_data = data_partition)

  # Save calibration tables
  if (!is.null(out_dir) && is.character(out_dir)){
    save_cal(x = output, out_dir = out_dir)
    }

  return(output)
}

#' K-fold data partitioning
#'
#' @description
#' Creates indices to partition available data into k equal-sized
#' subsets or folds, maintaining the global proportion of presence-absences
#' in each fold.
#'
#' @usage
#' kfold_partition(data, dependent, k = 2, seed = 1)
#'
#' @param data data.frame or matrix containing at least two columns.
#' @param dependent (character) name of column that contains the
#' presence-absence records (1-0).
#' @param k (numeric) the number of groups that the given data is to be split
#' into.
#' @param seed (numeric) integer value to specify an initial seed. Default = 1.
#'
#' @return
#' A list of vectors with the indices of rows corresponding to each fold.
#'
#' @export
#'
#' @examples
#' # example data
#' data <- data.frame(species = c(rep(0, 80), rep (1,20)),
#'                    variable1 = rnorm(100),
#'                    variable2 = rpois(100, 2))
#'
#' # create partition indices
#' kfolds <- kfold_partition(data, dependent = "species", k = 2)
#'
#' # data for partition 1
#' data[kfolds$Fold_1, ]


kfold_partition <- function(data, dependent, k = 2,  seed = 1){

  # initial tests
  if(missing(data) | missing(dependent)) {
    stop("Argument 'data' or 'dependent' must be defined.")
  }

  if (k < 2) {
    stop("Number of k-folds must be equal or greater than 2.")
  }

  # We extract the positions for presences and absences then to keep the original
  # ratio presences/absences we do it for each set separately
  pre <- which(data[, dependent] == 1)
  aus <- which(data[, dependent] == 0)

  # To each original position we assign a value from 1:k and the is randomly shuffled
  set.seed(seed)
  foldp <- sample(cut(seq(1, length(pre)), breaks = k, labels = FALSE))
  folda <- sample(cut(seq(1, length(aus)), breaks = k, labels = FALSE))

  # We name the "fold" vector with the positions of the original dataset
  names(foldp) <- pre
  names(folda) <- aus

  # Combination of P/A
  all <- c(foldp, folda)

  out <- list() # empty list when are stored the index of each FOLD

  for (i in 1:k) {
    out[[paste0("Fold_", i)]] <- as.numeric(names(all[all == i]))
  }

  return(out)
}

#' Partial ROC calculation
#'
#' @description proc applies partial ROC tests to model predictions.
#'
#' @usage
#' proc_enm(test_prediction, prediction, threshold = 5, sample_percentage = 50,
#'          iterations = 500)
#'
#' @param prediction `SpatRaster` or numeric vector of model predictions to be
#' evaluated.
#' @param test_prediction (numeric) vector of model predictions for testing
#' data.
#' @param threshold (numeric) value from 0 to 100 to represent the percentage of
#' potential error (E) that the data could have due to any source of uncertainty.
#' Default = 5.
#' @param iterations (numeric) number of bootstrap iterations to be performed;
#' default = 500.
#' @param sample_percentage (numeric) percentage of testing data to be used in each
#' bootstrapped process for calculating the partial ROC. Default = 50.
#'
#' @return A list with the summary of the results and a data.frame containing
#' the AUC values and AUC ratios calculated for all iterations.
#'
#' @details Partial ROC is calculated following Peterson et al. (2008)
#' <doi:10.1016/j.ecolmodel.2007.11.008>.
#'
#' @importFrom terra as.data.frame minmax
#' @importFrom stats na.omit
#'
#' @useDynLib enmpa, .registration = TRUE
#'
#' @export
#'
#' @examples
#' # Loading a model prediction
#' pred <- terra::rast(system.file("extdata", "proj_out_wmean.tif",
#'                                 package = "enmpa"))
#'
#' # Simulated data
#' test <- runif(100, min = 0.3, max = 0.8)
#'
#' # partial ROC calculation
#' pr <- proc_enm(test, pred, threshold = 5, sample_percentage = 50,
#'                iterations = 500)

proc_enm <- function(test_prediction, prediction, threshold = 5,
                     sample_percentage = 50, iterations = 500) {

  # detecting potential errors, other potential problems tested in code
  if (missing(prediction)) {
    stop("Argument 'prediction' is necessary to perform the analysis.")
  }
  if (missing(test_prediction)) {
    stop("Argument 'test_prediction' is necessary to perform the analysis.")
  }
  c_pred <- class(prediction)[1]
  if (!c_pred %in% c("SpatRaster", "numeric")) {
    stop("'prediction' must be of class 'SpatRaster' or 'numeric'.")
  }
  c_tdat <- class(test_prediction)[1]
  if (c_tdat != "numeric") {
    stop("'test_prediction' must be of class 'numeric'.")
  }

  # preparing data
  ## user data
  if (c_pred == "SpatRaster") {
    mmx <- c(terra::minmax(prediction))
    prediction <- unlist(terra::as.data.frame(prediction))
  } else {
    mmx <- range(prediction, na.rm = TRUE)
    prediction <- stats::na.omit(prediction)
  }

  test_prediction <- stats::na.omit(test_prediction)

  # analysis
  if(mmx[1] == mmx[2]) {
    warning("\nprediction has no variability, pROC will return NA.\n")

    # returning NA if model is too bad
    p_roc <- rep(NA, 2)
    names(p_roc) <- c(paste0("Mean_AUC_ratio_at_", threshold, "%"), "pval_pROC")

    auc_ratios <- rep(NA, 3)
    names(auc_ratios) <- c("Model_partial_AUC", "Random_curve_partial_AUC",
                           "AUC_ratio")

    p_roc_res <- list(pROC_summary = p_roc, pROC_results = auc_ratios)

  } else {
    # preparing inputs for proc
    nprediction <- length(prediction)
    prediction <- c(prediction, test_prediction)
    prediction <- as.numeric(cut(prediction, 500))
    test_prediction <- prediction[(nprediction + 1):length(prediction)]
    prediction <- prediction[1:nprediction]

    ## counting pixels in distinct classes
    classpixels <- as.data.frame(table(prediction), stringsAsFactors = FALSE)
    colnames(classpixels) <- c("value", "count")
    classpixels$value <- as.numeric(classpixels$value)

    classpixels <- classpixels[order(classpixels$value, classpixels$count,
                                      decreasing = TRUE), ]
    csum <- cumsum(classpixels$count)
    classpixels <- data.frame(classpixels,
                               totpixperclass = csum,
                               percentpixels = csum / sum(classpixels$count))
    classpixels <- classpixels[order(classpixels$value), ]

    ## proc parameters
    error_sens <- 1 - (threshold / 100)
    n_data <- length(test_prediction)
    n_samp <- ceiling((sample_percentage / 100) * n_data)

    ## matrix with values according to needs
    big_classpixels <- matrix(rep(classpixels$value, each = n_samp),
                              ncol = length(classpixels$value))

    # P ROC calculation
    partial_AUC <- lapply(1:iterations, function(x) {
      calc_aucDF(big_classpixels, classpixels$percentpixels, test_prediction,
                 n_data, n_samp, error_sens)
    })

    partial_AUC <- do.call(rbind, partial_AUC)

    # filtering
    nona_valproc <- partial_AUC$auc_ratio[!is.na(partial_AUC$auc_ratio)]
    mauc <- mean(nona_valproc)
    proc <- sum(nona_valproc <= 1) / length(nona_valproc)

    p_roc <- c(mauc, proc)
    names(p_roc) <- c(paste0("Mean_AUC_ratio_at_", threshold, "%"), "pval_pROC")

    auc_ratios <- partial_AUC
    names(auc_ratios) <- c("Model_partial_AUC", "Random_curve_partial_AUC",
                           "AUC_ratio")

    p_roc_res <- list(pROC_summary = p_roc, pROC_results = auc_ratios)
  }

  return(p_roc_res)
}



# helper to calculate AUC for partial ROC
calc_aucDF <- function(big_classpixels, fractional_area, test_prediction,
                       n_data, n_samp, error_sens) {

  rowsID <- sample(x = n_data, size = n_samp, replace = TRUE)
  omssion_matrix <- big_classpixels > test_prediction[rowsID]
  sensibility <- 1 - colSums(omssion_matrix) / n_samp

  xyTable <- data.frame(fractional_area, sensibility)
  xyTable <- xyTable[-which(xyTable$sensibility <= error_sens), ]
  xyTable <- xyTable[order(xyTable$fractional_area, decreasing = FALSE), ]

  auc_pmodel <- trap_roc(xyTable$fractional_area, xyTable$sensibility)
  auc_prand <- trap_roc(xyTable$fractional_area, xyTable$fractional_area)
  auc_table <- data.frame(auc_pmodel, auc_prand,
                          auc_ratio = auc_pmodel / auc_prand )
  return(auc_table)
}

#' Selection of best candidate models considering various criteria
#'
#' @description
#' Applies a series of criteria to select best candidate models.
#'
#' @usage
#' model_selection(evaluation_stats, criterion = "TSS", exclude_bimodal = FALSE,
#'                 tolerance = 0.01)
#'
#' @param evaluation_stats data.frame with the statistics of model evaluation
#' results. These results are the output of the function
#' \code{\link{evaluation_stats}}.
#' @param criterion (character) metric used as the predictive criterion for
#' model selection.
#' @param exclude_bimodal (logical) whether to exclude models in which binomial
#' variable response curves were detected.
#' @param tolerance (numeric)
#'
#' @return
#' A data.frame with one or more selected models.
#'
#' @export
#'
#' @examples
#' # data
#' data("cal_res", package = "enmpa")
#' eval_stats <- cal_res$summary[, -1]
#'
#' # selecting best model
#' selected_mod <- model_selection(eval_stats, exclude_bimodal = TRUE)

model_selection <- function(evaluation_stats, criterion = "TSS",
                            exclude_bimodal = FALSE, tolerance = 0.01) {
  if (missing(evaluation_stats)) {
    stop("Argument 'evaluation_stats' must be defined.")
  }
  if (!criterion %in% c("TSS", "ESS")) {
    stop("'criterion' must be 'TSS' or 'ESS'.")
  }
  if (!is.numeric(tolerance)) {
    stop("Argument 'tolerance' must be of class numeric.")
  }

  # selection based on significance
  if (exclude_bimodal) {
    sel <- evaluation_stats[evaluation_stats$Concave_responses == "", ]

    if (nrow(sel) == 0) {
      message("All models have at least one predictor with concave responses.",
              "\nReturning selected_models = NULL.")
      return(NULL)
    }

    sel <- sel[sel$ROC_AUC_mean > 0.5, ]

  } else {
    sel <- evaluation_stats[evaluation_stats$ROC_AUC_mean > 0.5, ]
  }

  if (nrow(sel) == 0) {
    warning("No candidate model met the 'AUC > 0.5' criterion.",
            "\nModels with maximum AUC values will be used.")
    sel <- evaluation_stats[evaluation_stats$ROC_AUC_mean ==
                              max(evaluation_stats$ROC_AUC_mean), ]
  }

  # intermediate selection based on TSS or Accuracy (filter)
  if (criterion == "TSS") {
    sel <- sel[sel$Threshold_criteria == "maxTSS", ]
    sel1 <- sel[sel$TSS_mean >= 0.4, ]
    if (nrow(sel1) == 0) {
      warning("No candidate model met the 'TSS >= 0.4' criterion.",
              "\nModels with 'TSS values >= (maximum TSS - ",
              tolerance, ")' will be used.")
      sel1 <- sel[sel$TSS_mean >= (max(sel$TSS_mean) -
                                     tolerance), ]
    }
  } else {
    sel <- sel[sel$Threshold_criteria == "ESS", ]
    sel1 <- sel[sel$Accuracy >= (max(sel$Accuracy) - tolerance),
    ]
  }

  tryCatch({
    # delta AIC for filtered models
    sel1$Delta_AIC <- sel1$AIC - min(sel1$AIC, na.rm = TRUE)
    sel1 <- sel1[sel1$Delta_AIC <= 2, ]

    # weight of AIC selected models
    sel1$AIC_weight <- exp(-0.5 * sel1$Delta_AIC)
    sel1$AIC_weight <- sel1$AIC_weight/sum(sel1$AIC_weight,
                                           na.rm = TRUE)
    rownames(sel1) <- 1:nrow(sel1)
  }, error = function(e) {
    message_error <- paste0("No model passed selection criteria,",
                            "try increasing 'tolerance'.", "\nCurrent 'tolerance' = ",
                            tolerance)
    message(message_error)
    sel1 <- NULL
  })

  return(sel1)
}

#' Niche Signal detection using one or multiple variables
#'
#' @description
#' Identifies whether a signal of niche can be detected using one or multiple
#' variables. This is an implementation of the methods developed by
#' Cobos & Peterson (2022) <doi:10.17161/bi.v17i.15985> that
#' focuses on identifying niche signals in presence-absence data.
#'
#' @usage
#' niche_signal(data, condition, variables, method = "univariate",
#'              permanova_method = "mahalanobis", iterations = 1000,
#'              set_seed = 1, verbose = TRUE, ...)
#'
#' @param data matrix or data.frame containing at least the following
#' information: a column representing \code{condition} (positive = 1 or
#' negative = 0), and one or more columns representing environmental variables.
#' @param condition (character) name of the column with numeric information
#' about detection (positive = 1 or negative = 0).
#' @param variables (character) vector of one or more names of columns to be
#' used as environmental variables. If \code{method} = "univariate", only one
#' variable is used; for \code{method} = "permanova", multiple variables can
#' be used.
#' @param method (character) name of the method to be used for niche comparison.
#' Default = "univariate".
#' @param permanova_method (character) name of the dissimilarity index to be
#' used as \code{method} in \code{\link[vegan]{adonis2}}. See all options in
#' \code{\link[vegan]{vegdist}}. Default = "mahalanobis".
#' @param iterations (numeric) number of iterations to be used in analysis.
#' Default = 1000. If \code{method} = "permanova", permutations = iterations - 1.
#' @param set_seed (numeric) integer value to specify a initial seed.
#' Default = 1.
#' @param verbose (logical) whether or not to print messages about the process.
#' Default = TRUE.
#' @param ... other arguments to be passed to \code{\link[vegan]{adonis2}}.
#'
#' @return
#' A list with results from analysis depending on `method`.
#'
#' @export
#'
#' @importFrom vegan adonis2
#' @importFrom stats median sd ecdf
#'
#' @rdname niche_signal
#'
#' @examples
#' # Load species occurrences and environmental data.
#' data("enm_data", package = "enmpa")
#' head(enm_data)
#'
#' # Detection of niche signal using an univariate non-parametric test
#' sn_bio1 <- niche_signal(data = enm_data, variables = "bio_1",
#'                         condition = "Sp", method = "univariate")
#' sn_bio1
#'
#' sn_bio12 <- niche_signal(data = enm_data, variables = "bio_12",
#'                          condition = "Sp", method = "univariate")
#' sn_bio12


niche_signal <- function(data, condition, variables, method = "univariate",
                         permanova_method = "mahalanobis", iterations = 1000,
                         set_seed = 1, verbose = TRUE, ...) {
  # initial tests
  if (!method %in% c("univariate", "permanova")) {
    stop("Argument 'method' not valid, options are 'univariate' and 'permanova'")
  }
  if (missing(variables)) {
    stop("Argument 'variables' must be defined")
  }

  # number of positives
  n_pos <- nrow(data[data[, condition] == 1, ])

  # running analysis
  if (method == "univariate") {
    if (length(variables) > 1) {
      if (verbose) {
        message("More than one element detected in 'variables', analysis",
                "\nwill run with the first variable with method 'univariate'")
      }

      variables <- variables[1]
    }

    res <- niche_signal_univariate(data, condition, variables, iterations,
                                   set_seed, verbose)

  } else {
    if (verbose) {
      message("Number of permutations for permanova = interations - 1")
    }

    res <- niche_signal_permanova(data, condition, variables,
                                  permutations = (iterations - 1),
                                  permanova_method, verbose, ...)
  }

  # preparing final result
  res <- list(
    data = data, modified_data = res$modified_data,
    summary = list(variables = variables, condition = condition,
                   method = method, n_complete = nrow(data),
                   n_positive = n_pos),
    univariate_results = res$analysis_results,
    permanova_results = res$permanova_results
  )

  # returning result
  return(res)
}


#' @rdname niche_signal
#' @param variable (character) name of the column containing data to be used
#' as environmental variable.
#' @export
#' @usage
#' niche_signal_univariate(data, condition, variable, iterations = 1000,
#'                         set_seed = 1, verbose = TRUE)

niche_signal_univariate <- function(data, condition, variable, iterations = 1000,
                                    set_seed = 1, verbose = TRUE) {
  # initial tests
  if (missing(data)) {
    stop("Argument 'data' must be defined")
  }
  if (missing(condition)) {
    stop("Argument 'condition' must be defined")
  }
  if (missing(variable)) {
    stop("Argument 'variable' must be defined")
  }

  # number of infected hosts
  if (verbose) {
    message("Preparing data")
  }
  n_inf <- nrow(data[data[, condition] == 1, ])

  # representation of niches via sd and range
  posit <- data[, condition] == 1
  pa_mean <- mean(data[posit, variable])
  pa_med <- median(data[posit, variable])
  pa_sd <- sd(data[posit, variable])
  pa_range <- diff(range(data[posit, variable]))

  h_mean <- mean(data[, variable])
  h_med <- median(data[, variable])
  h_sd <- sd(data[, variable])
  h_range <- diff(range(data[, variable]))

  # positive cases null distribution of niches
  if (verbose) {
    message("Running analysis...")
  }
  ## mean
  set.seed(set_seed)
  n_mean <- vapply(1:iterations, FUN.VALUE = numeric(1), function(x) {
    mean(sample(data[, variable], n_inf))
  })

  ## median
  set.seed(set_seed)
  n_med <- vapply(1:iterations, FUN.VALUE = numeric(1), function(x) {
    median(sample(data[, variable], n_inf))
  })

  ## sd
  set.seed(set_seed)
  n_sd <- vapply(1:iterations, FUN.VALUE = numeric(1), function(x) {
    sd(sample(data[, variable], n_inf))
  })

  ## range
  set.seed(set_seed)
  n_range <- vapply(1:iterations, FUN.VALUE = numeric(1), function(x) {
    diff(range(sample(data[, variable], n_inf)))
  })

  # helper to calculated p value according to position of observed in null
  p_calc <- function(observed, null_distribution) {
    ef <- ecdf(null_distribution)
    ef(observed)
  }

  # pseudo p values
  pml <- p_calc(pa_mean, n_mean)
  pmdl <- p_calc(pa_med, n_med)
  psl <- p_calc(pa_sd, n_sd)
  prl <- p_calc(pa_range, n_range)

  # hypothesis test
  tm <- ifelse(pml <= 0.025 | pml >= 0.975, "rejected", "accepted")
  tmd <- ifelse(pmdl <= 0.025 | pmdl >= 0.975, "rejected", "accepted")
  ts <- ifelse(psl <= 0.025 | psl >= 0.975, "rejected", "accepted")
  tr <- ifelse(prl <= 0.025 | prl >= 0.975, "rejected", "accepted")

  # position of observed
  qm <- ifelse(tm == "accepted", "indistinct",
               ifelse(pml <= 0.025, "lower", "higher"))
  qmd <- ifelse(tmd == "accepted", "indistinct",
                ifelse(pmdl <= 0.025, "lower", "higher"))
  qs <- ifelse(ts == "accepted", "indistinct",
               ifelse(psl <= 0.025, "lower", "higher"))
  qr <- ifelse(tr == "accepted", "indistinct",
               ifelse(prl <= 0.025, "lower", "higher"))


  # return results
  results <- list(
    data = data,
    summary = list(variable = variable, condition = condition,
                   n_complete = nrow(data), n_positive = n_inf),
    analysis_results = list(
      hypothesis_test = c(h0_mean = tm, h0_median = tmd, h0_SD = ts,
                          h0_range = tr, positive_mean_vs_null = qm,
                          positive_median_vs_null = qmd,
                          positive_SD_vs_null = qs,
                          positive_range_vs_null = qr),
      complete_niche = c(mean = h_mean, median = h_med, SD = h_sd,
                         range = h_range),
      positive_niche = c(mean = pa_mean, median = pa_med, SD = pa_sd,
                         range = pa_range),
      null_results = list(mean = n_mean, median = n_med, SD = n_sd,
                          range = n_range)
    )
  )

  return(results)
}




#' @rdname niche_signal
#' @param permutations number of permutations to be performed.
#' @export
#' @usage
#' niche_signal_permanova(data, condition, variables, permutations = 999,
#'                        permanova_method = "mahalanobis", verbose = TRUE, ...)

niche_signal_permanova <- function(data, condition, variables, permutations = 999,
                                   permanova_method = "mahalanobis",
                                   verbose = TRUE, ...) {
  # initial tests
  if (missing(data)) {
    stop("Argument 'data' must be defined")
  }
  if (missing(condition)) {
    stop("Argument 'condition' must be defined")
  }
  if (missing(variables)) {
    stop("Argument 'variables' must be defined")
  }

  # preparing data
  if (verbose) {
    message("Preparing data")
  }
  n_com <- nrow(data)
  dataihost <- data[data[, condition] == 1, ]
  n_inf <- nrow(dataihost)
  dataihost[, condition] <- 0
  data <- rbind(data, dataihost)
  form <- paste("as.matrix(data[, variables]) ~", condition)

  # running analysis
  if (verbose) {
    message("Running analysis...")
  }
  perm <- vegan::adonis2(as.formula(form), data = data,
                         permutations = permutations,
                         method = permanova_method, ...)

  # results preparation
  results <- list(modified_data = data,
                  summary = list(variables = variables, condition = condition,
                                 n_complete = n_com, n_positive = n_inf),
                  permanova_results = perm)

  return(results)
}

#' Two-Way interaction response plot
#'
#' @description
#' A view of the species probability into a two-dimensional
#' environmental space.
#'
#' @usage
#' resp2var(model, variable1 , variable2, modelID = NULL, data = NULL, n = 1000,
#'          new_data = NULL, extrapolate = FALSE, add_bar = TRUE,
#'          add_limits = FALSE, color.palette	= NULL, xlab = NULL, ylab = NULL,
#'          ...)
#
#' @param model an object of class `glm` or `enmpa_fitted_models`.
#' @param variable1 (character) name of the variable to be plotted in x axis.
#' @param variable2 (character) name of the variable to be plotted in y axis.
#' @param modelID (character) name of the ModelID if inputed `model` is  in the
#' `enmpa_fitted_models` object.
#' Default = NULL.
#' @param data data.frame or matrix of data to be used in model calibration.
#' Default = NULL.
#' @param n (numeric) an integer guiding the number of breaks. Default = 100
#' @param new_data a `SpatRaster`, data.frame, or  matrix of variables
#' representing the range of variable values in an area of interest.
#' Default = NULL.
#' @param extrapolate (logical) whether to allow extrapolation to study the
#' behavior of the response outside the calibration limits. Ignored if
#' `new_data` is defined. Default = TRUE.
#' @param add_bar (logical) whether to add bar legend. Default = TRUE.
#' @param add_limits (logical) whether to add calibration limits if
#' `extrapolate = TRUE`. Default = FALSE.
#' @param color.palette (function) a color palette function to be used to assign
#' colors in the plot. Default = function(n) rev(hcl.colors(n, "terrain")).
#' @param xlab (character) a label for the x axis. The default, NULL, uses the
#' name defined in `variable1`.
#' @param ylab (character) a label for the y axis. The default, NULL, uses the
#' name defined in `variable2`.
#' @param ... additional arguments passed to
#' \code{\link[graphics]{image}}.
#'
#' @details
#' The function calculates probabilities by focusing on each combination of the
#' two supplied environmental variable while keeping all other variables constant
#' at their mean values.
#'
#'
#' @return
#' A plot with the response interaction of two environmental dimensions for
#' `variable1` and `variable2`, and don't return anything.
#'
#' @export
#' @rdname resp2var
#'
#' @importFrom stats predict coef
#' @importFrom graphics par title axis image layout mtext plot.new plot.window rect
#' @importFrom terra minmax
#' @importFrom grDevices hcl.colors
#'
#' @examples
#' # Load a fitted selected model
#' data(sel_fit, package = "enmpa")
#'
#' # Two-Way interaction response plot in the calibration limits
#'
#' resp2var(sel_fit, variable1 = "bio_1", variable2 = "bio_12", xlab = "BIO-1",
#' ylab = "BIO-12", modelID = "ModelID_7")
#'
#' # Two-Way interaction response plot allowing extrapolation
#' resp2var(sel_fit, variable1 = "bio_1", variable2 = "bio_12", xlab = "BIO-1",
#' ylab = "BIO-12", modelID = "ModelID_7", extrapolate = TRUE)

resp2var <- function(model, variable1 , variable2, modelID = NULL, data = NULL,
                     n = 1000, new_data = NULL, extrapolate = FALSE,
                     add_bar = TRUE, add_limits = FALSE, color.palette	= NULL,
                     xlab = NULL, ylab = NULL, ...) {

  # initial tests
  if (missing(model) | missing(variable1) | missing(variable2)) {
    stop("Argument 'model' or 'variables' must be defined.")
  }

  if (!is.null(new_data)) {
    if (!class(new_data)[1] %in% c("matrix", "data.frame", "SpatRaster")) {
      stop("'new_data' must be of class 'matrix', 'data.frame', 'SpatRaster'")
    }
  }

  if (class(model)[1] == "glm" && is.null(try(model$data)) && is.null(data)) {
    stop(paste("The argument 'data' must be defined in case the model entered",
               "does not explicitly include a data component."))
  }

  if (class(model)[1] == "glm" && !is.null(try(model$data))) {
    data <- model$data
  }

  #  using de 'enmpa fitted object'
  if (class(model)[1] == "enmpa_fitted_models" && !is.null(modelID)) {
    data  <- model$data
    model <-  model$glms_fitted[modelID][[1]]
  } else {
    if (class(model)[1] == "enmpa_fitted_models" && is.null(modelID)) {
      stop(paste("The argument 'modelID' must be defined if you are entering",
                 "an object 'enmpa_fitted_models'."))
    }
  }


  if (is.null(xlab)) xlab <- variable1
  if (is.null(ylab)) ylab <- variable2
  if (is.null(color.palette)) color.palette = function(n) rev(hcl.colors(n, "terrain"))
  # if (is.null(color.palette)) color.palette = function(n) hcl.colors(n)


  variables <- c(variable1, variable2)

  # It gets only the variable names used in the fitted model
  vnames <- colSums(
    sapply(colnames(data), grepl, names(coef(model)[-1]))
  ) > 0

  if (any(!variables %in% names(vnames))) {
    stop("The name of the 'variables' was not defined correctly.")
  }

  # Extract calibration data from the model object
  cal_data <- data[, vnames]

  # Extract the limits of the calibration data
  cal_maxs <-  apply(cal_data, 2, FUN = max)
  cal_mins <-  apply(cal_data, 2, FUN = min)

  # Get the average of all variabless
  means <- apply(cal_data, 2, FUN = mean)


  if (is.null(new_data)) {

    if (extrapolate){

      rr1 <- range(cal_data[, variables[1]]) # range of the calibration data
      rr2 <- range(cal_data[, variables[2]]) # range of the calibration data

      l_limit1 <- rr1[1] - 0.11 * diff(rr1)
      u_limit1 <- rr1[2] + 0.11 * diff(rr1)

      l_limit2 <- rr2[1] - 0.11 * diff(rr2)
      u_limit2 <- rr2[2] + 0.11 * diff(rr2)


      rangev1 <- c(l_limit1, u_limit1)
      rangev2 <- c(l_limit2, u_limit2)

    } else {

      rangev1 <- range(cal_data[, variables[1]])
      rangev2 <- range(cal_data[, variables[2]])
    }

  } else {

    if (class(new_data)[1] == "SpatRaster") {
      rangev1 <- terra::minmax(new_data[[variables[1]]])
      rangev2 <- terra::minmax(new_data[[variables[2]]])

    } else {
      rangev1 <- range(new_data[, variables[1]])
      rangev2 <- range(new_data[, variables[2]])
    }

  }

  newvar <- expand.grid(x = seq(rangev1[1], rangev1[2], length = n),
                        y = seq(rangev2[1], rangev2[2], length = n))


  m <- data.frame(matrix(means, nrow(newvar), length(means), byrow = T))
  colnames(m) <- names(means)

  m[, variables[1]] <- newvar[, 1]
  m[, variables[2]] <- newvar[, 2]

  # Response of the variables
  predicted <- stats::predict(model, m, type = "response")

  # Arguments for filled.contour
  # "x,y" locations of grid lines at which the values in z are measured
  # "z" a numeric matrix containing the values to be plotted
  x = unique(m[, variables[1]])
  y = unique(m[, variables[2]])
  z = matrix(data = predicted, nrow = n, ncol = n)


  # Set colors
  colors <- color.palette(10)

  # Save the original par settings
  original_par <- par(no.readonly = TRUE)

  if (add_bar == FALSE){
    # Plot the main image
    image(x,y,z,
          zlim = c(0,1),
          col = colors,
          xlab = xlab, ylab = ylab, useRaster = F, ...)

    if (extrapolate & add_limits ){

      abline(v = c(cal_mins[variables[1]], cal_maxs[variables[1]]),
             h = c(cal_mins[variables[2]], cal_maxs[variables[2]]),
             lty = 2)
    }
  } else{

    layout(matrix(1:2, ncol = 2), widths = c(4, 1))  # Adjust widths to allocate space for the legend
    par(mar = c(5, 4, 4, 2) + 0.1)  # Set margins for the main plot

    # Plot the main image
    image(x,y,z,
          zlim = c(0,1),
          col = colors,
          xlab = xlab, ylab = ylab, useRaster = T, ...)

    if (extrapolate & add_limits ){

      abline(v = c(cal_mins[variables[1]], cal_maxs[variables[1]]),
             h = c(cal_mins[variables[2]], cal_maxs[variables[2]]),
             lty = 2)
    }

    # Add the color bar legend
    par(mar = c(5, 0, 4, 2) + 0.1)  # Set margins for the legend
    color_levels <- seq(0, 1, length.out = length(colors) + 1)
    legend_y <- seq(min(y), max(y), length.out = length(colors) + 1)

    # Plot the legend
    plot.new()
    plot.window(xlim = c(0, 1), ylim = range(legend_y))

    # Draw rectangles for the legend
    for (i in seq_along(colors)) {
      rect(0.25, legend_y[i], 0.5, legend_y[i + 1], col = colors[i],
           border = "transparent")
    }
    rect(0.25, min(legend_y), 0.5, max(legend_y), col = NA,
         border = "black")

    # Add text labels to the legend
    axis(4, at = legend_y, labels = round(color_levels, 2), las = 1,
         tick = FALSE, pos = 0.5  )
    mtext("Suitability", side = 3, line = 0, cex = 1.2)

    # Restore the original par settings
    par(original_par)
  }


  # filled.contour(x, y, z,
  #                nlevels = 10,
  #                color.palette = color.palette,
  #                key.title = {par(cex.main = 0.9); title(main = NULL)},
  #                xlab = xlab, ylab = ylab,
  # )

  # other alternative
  # image(x,y,z,
  #       zlim = c(0,1),
  #       col = color.palette(10),
  #       xlab = xlab, ylab = ylab, useRaster = T)
  # contour(x,y,z,
  #         levels = seq(0, 1, by = 0.1),
  #         add = TRUE, col = "black")

}

#' Jackkniffe test for variable contribution
#'
#' @description
#' The Jackknife function providing a detailed reflection of the  impact of each
#' variable on the overall model, considering four difference measures:
#' ROC-AUC, TSS, AICc, and Deviance.
#'
#' @usage
#' jackknife(data, dependent, independent, user_formula = NULL, cv = 3,
#'           response_type = "l", weights = NULL)
#'
#' @param dependent (character) name of dependent variable.
#' @param independent (character) vector of name(s) of independent variable(s).
#' @param user_formula (character) custom formula to test. Default = NULL.
#' @param data data.frame or matrix of data to be used in model calibration.
#' Columns represent dependent and independent variables.
#' @param cv (numeric) number of folds to use for k-fold
#' cross-validation exercises. Default = 3.
#' @param response_type (character) a character string that must contain "l",
#' "p", "q" or a combination of them. l = lineal, q = quadratic,
#' p = interaction between two variables. Default = "l".
#' @param weights (numeric) a vector with the weights for observations.
#'
#' @return list including model performance metrics (ROC-AUC, TSS, AICc, and
#' deviance) for the complete model, model performance when excluding a specific
#' predictor, and the independent contribution of that predictor to the model.
#'
#' @examples
#'
#' # Load data
#' data("enm_data", package = "enmpa")
#'
#' jk <- jackknife(data = enm_data,
#'                 dependent = "Sp",
#'                 independent = c("bio_1", "bio_12"),
#'                 user_formula = NULL,
#'                 cv = 3, response_type = "lpq")
#' jk
#'
#' # plot JK's results
#' plot_jk(jk, metric = "TSS")
#' plot_jk(jk, metric = "ROC_AUC")
#' plot_jk(jk, metric = "AIC")
#' plot_jk(jk, metric = "Residual_deviance")
#'
#' @export
#'
#' @importFrom utils combn
#'

jackknife <- function(data, dependent, independent, user_formula = NULL, cv = 3,
                      response_type = "l", weights = NULL){

  # initial tests
  if (missing(data) | missing(dependent) | missing(dependent)) {
    stop("Arguments 'data', 'dependent', and 'independent' must be defined.")
  }

  if (is.null(user_formula)) {
    # get full model formula using get_formulas
    ff <- get_formulas(dependent, independent, type = response_type,
                       mode = "complex")
  } else {
    # use user formula
    ff <- user_formula
  }

  aux <- gsub(paste0(dependent," ~ "), "", ff)
  features <- unlist(strsplit(gsub(" ", "", aux), split = "[+]"))

  comb <- utils::combn(features, length(features) - 1)

  withon <- sapply(1:length(features), function(x) {
    get_formulas(dependent, rev(features)[x], type = "l", mode = "complex")
  })

  without <- sapply(1:length(features), function(x) {
    get_formulas(dependent, comb[, x], type = "l", mode = "complex")
  })

  metrics <- c("ROC_AUC", "TSS", "AIC", "Residual_deviance")

  full <- model_validation(ff, data = data, weights = weights)[2, metrics]
  rownames(full) <- NULL

  withon_r <- lapply(withon, function(x) {
    model_validation(x, data = data, weights = weights)[2, metrics]
  })

  without_r <- lapply(without, function(x) {
    model_validation(x, data = data, weights = weights)[2, metrics]
  })

  withon_r <- do.call(rbind, withon_r)
  without_r <- do.call(rbind, without_r)

  rownames(withon_r) <- features
  rownames(without_r) <- features


  return(list(Full_model_stats = full,
              Formula = ff,
              Without = without_r,
              With_only = withon_r)
  )
}

#' Model validation options
#'
#' @description
#' Model evaluation using entire set of data and a k-fold cross validation
#' approach. Models are assessed based on discrimination power (ROC-AUC),
#' classification ability (accuracy, sensitivity, specificity, TSS, etc.),
#' and the balance between fitting and complexity (AIC).
#'
#' @usage
#' model_validation(formula, data, family = binomial(link = "logit"),
#'                  weights = NULL, cv = FALSE, partition_index = NULL,
#'                  k = NULL, dependent = NULL, n_threshold = 100,
#'                  keep_coefficients = FALSE, seed = 1)
#'
#' @param formula (character) `expression` to be used as a glm `formula`.
#' @param data data.frame with dependent and independent variables.
#' @param family a `family` object for models used by functions such as `glm`.
#' Default = binomial(link = "logit").
#' @param weights (numeric) vector with weights for observations. Default = NULL.
#' @param cv (logical) whether to use a k-fold cross validation for evaluation.
#' Default = FALSE.
#' @param partition_index list of indices for cross validation in k-fold.
#' Obtained with the function \code{\link{kfold_partition}}. Default = NULL.
#' @param k (numeric) number of folds for a new k-fold index preparation.
#' Ignored if `partition_index` is defined or if `cv` = FALSE. Default = NULL.
#' @param dependent (character) name of dependent variable. Ignore if
#' `cv` = FALSE. Default = NULL.
#' @param n_threshold (numeric) number of threshold values to be used for ROC.
#' Default = 100.
#' @param keep_coefficients (logical) whether to keep model coefficients.
#' Default = FALSE.
#' @param seed (numeric) a seed number. Default = 1.
#'
#' @return
#' A data.frame with results from evaluation.
#'
#' @export
#'
#' @importFrom stats as.formula binomial glm predict.glm
#'
#' @examples
#' # Load species occurrences and environmental data.
#' data("enm_data", package = "enmpa")
#' head(enm_data)
#'
#' # Custom formula
#' form <- c("Sp ~ bio_1 + I(bio_1^2) + I(bio_12^2)")
#'
#' # Model evaluation using the entire set of records
#' model_validation(form, data = enm_data)
#'
#' # Model evaluation using a k-fold cross-validation (k = 3)
#' model_validation(form, data = enm_data, cv = TRUE, k = 3, dependent = "Sp")

model_validation <- function(formula, data, family = binomial(link = "logit"),
                             weights = NULL, cv = FALSE, partition_index = NULL,
                             k = NULL, dependent = NULL, n_threshold = 100,
                             keep_coefficients = FALSE, seed = 1) {

  # initial test
  if (missing(formula) | missing(data)) {
    stop("Argument 'formula' or 'data' must be defined.")
  }
  if (is.logical(cv)) {
    if (cv) {
      if (is.null(partition_index) & (is.null(k) | is.null(dependent))) {
        stop(" If 'partition_index' = NULL, then 'k' and 'dependent' must be defined if 'cv' = TRUE.")
      }
    }
  } else {
    stop("'cv' must be logical.")
  }
  f <- as.formula(formula)

  gfit <- suppressWarnings(
    stats::glm(formula = f, family = family, data = data, weights = weights)
    )

  AIC <- gfit$aic
  Residual_deviance <- gfit$deviance
  nparameters <- length(gfit$coefficients) - 1

  if (cv) {
    ## Cross-validation
    if (is.null(partition_index) & !is.null(k) & !is.null(dependent)) {
      partition_index <- kfold_partition(data, dependent = dependent, k = k,
                                         seed = seed)
    }

    out <- data.frame()

    for (x in 1:length(partition_index)) {

      # Define the train and test data
      data_test <- data[partition_index[[x]], ]
      data_train <- data[-partition_index[[x]], ]

      # extract the corresponding weight values for the k-fold
      if (!is.null(weights)) {
        weights_p <- weights[-partition_index[[x]]]
      } else {
        weights_p <- NULL
      }

      # Fit using training data
      kfit <- suppressWarnings(
        stats::glm(formula = f, family = family, data = data_train,
                   weights = weights_p)
      )

      # Evaluation using Test Dependent data
      pred_k <- stats::predict.glm(kfit, data_test, type = "response")
      eval_k <- optimize_metrics(actual = data_test[, all.vars(f)[1]],
                                 predicted = pred_k,
                                 n_threshold = n_threshold)$optimized

      res <- data.frame(Formulas = formula,
                        Kfold = x,
                        eval_k,
                        Parameters = nparameters,
                        Residual_deviance = Residual_deviance,
                        AIC = AIC)
      out <- rbind(out, res)
    }

  } else {
    pred_global <- stats::predict.glm(gfit, data, type = "response")
    eval_global <- optimize_metrics(actual = data[, all.vars(f)[1]],
                                    predicted = pred_global,
                                    n_threshold = n_threshold)$optimized
    out <- data.frame(Formulas = formula,
                      eval_global,
                      Parameters = nparameters,
                      Residual_deviance = Residual_deviance,
                      AIC = AIC)
  }

  if (keep_coefficients) {
    out <- data.frame(out,
                      Concave_responses = detect_concave(gfit$coefficients))
  }

  return(out)
}

#' Predictions for the models selected after calibration
#'
#' @description
#' Wrapper function that facilitates the prediction of those models
#' selected as the most robust. In addition, it allows the calculation of
#' consensus models, when more than one model are selected.
#'
#' @usage
#' predict_selected(fitted, newdata, extrapolation_type = "E",
#'                  restricted_vars = NULL, type = "response", consensus = TRUE)
#'
#' @param fitted an enmpa-class`fitted models` object obtained using the
#' functions \code{\link{fit_selected}}.
#' @param newdata a `SpatRaster`, data.frame, or matrix with the new data on
#' which to predict.
#' @param extrapolation_type (character) to indicate extrapolation type of model.
#' Models can be transferred with three options: free extrapolation ('E'),
#' extrapolation with clamping ('EC'), and no extrapolation ('NE').
#' Default = 'E'.
#' @param restricted_vars (character) a vector containing the names of the
#' variables that will undergo clamping or no extrapolation. For clamping,
#' these variables are set to minimum and maximum values established for the
#' max and min values within calibration values. For no extrapolation, the
#' variables outside calibration limits became NA. If no specific names are
#' provided, the value is set to NULL by default, indicating that clamping (EC)
#' or no extrapolation (NE) will be applied to all variables. Ignore if
#' extrapolation_type = 'E'.
#' @param type (character) the type of prediction required. For a default
#' binomial model the default predictions are of log-odds (probabilities on
#' logit scale). The default, "response", returns predicted probabilities.
#' @param consensus (logical) valid if `newdata` is a `SpatRaster`, whether to
#' produce consensus results obtained by combining the predictions from the
#' collection of selected models. By default consensuses are calculated using
#' the mean, median, variance, and weighted average using the AIC weights.
#' Default = TRUE.
#'
#' @return
#' A list with predictions of selected models on the `newdata` and fitted
#' selected model(s). Consensus predictions are added if multiple selected
#' models exits and if `newdata` is a `SpatRaster` object.
#'
#' @export
#'
#' @importFrom terra rast
#' @importFrom stats var median
#'
#' @examples
#' # Load a fitted selected model
#' data(sel_fit, package = "enmpa")
#'
#' # Load raster layers to be projected
#' env_vars <- terra::rast(system.file("extdata", "vars.tif", package = "enmpa"))
#'
#' # Predictions (only one selected mode, no consensus required)
#' preds <- predict_selected(sel_fit, newdata = env_vars, consensus = FALSE)
#'
#' # Plot prediction
#' terra::plot(preds$predictions)

predict_selected <- function(fitted, newdata, extrapolation_type = "E",
                             restricted_vars = NULL, type = "response",
                             consensus = TRUE) {

  if (missing(fitted)) {
    stop("Arguments 'fitted' must be defined.")
  }
  if (missing(newdata)) {
    stop("Arguments 'newdata' must be defined.")
  }

  # Obtain the predicted values (p) for each selected model
  p <- lapply(fitted$glms_fitted, function(y) {
    predict_glm(y, newdata, data = fitted$data,
                extrapolation_type = extrapolation_type,
                restricted_vars = restricted_vars, type = type)
  })

  if (class(newdata)[1] == "SpatRaster") {
    p <- terra::rast(p)
  }

  names(p) <- names(fitted$glms_fitted)

  # Consensus obtained by combining the forecasts from
  # the selected models.
  if (consensus  && length(fitted$glms_fitted) > 1 &&
      class(newdata)[1] == "SpatRaster") {

    # Adjust AIC weights to sum 1
    aicw_dif <- 1 - sum(fitted$selected$AIC_weight)
    fitted$selected$AIC_weight[1] <- fitted$selected$AIC_weight[1] + aicw_dif

    cons_p <- consensus_p(predictions = p, weights = fitted$selected$AIC_weight)
    out <- list(predictions = p, consensus = cons_p)
    return(out)

  } else {
    out <- list(predictions = p)
    return(out)
  }
}

#' Get GLM formulas according to defined response types
#'
#' @description
#' Generate GLM formulas for independent variables predicting a dependent
#' variable, taking into account response types required. All possible
#' combinations of variables can be created using arguments of the function.
#'
#' @usage
#' get_formulas(dependent, independent, type = "l", mode = "moderate",
#'              minvar = 1, maxvar = NULL)
#'
#' @param dependent (character) name of dependent variable.
#' @param independent (character) vector of name(s) of independent variable(s).
#' @param type (character) a character string that must contain "l",
#' "p", "q" or a combination of them. l = lineal, q = quadratic,
#' p = interaction between two variables. Default = "l".
#' @param mode (character) (character) a character string to indicate the strategy to
#' create the formulas for candidate models. Options are: "light", "moderate",
#' "intensive", or "complex". Default = "moderate".
#' @param minvar (numeric) minimum number of independent variables in formulas.
#' @param maxvar (numeric) maximum number of independent variables in formulas.
#'
#' @details
#' `mode` options determine what strategy to iterate the predictors
#' defined in \code{type} for creating models:
#' - **light**.-- returns simple iterations of complex formulas.
#' - **moderate**.-- returns a comprehensive number of iterations.
#' - **intensive**.-- returns all possible combination. Very time-consuming for
#' 6 or more independent variables.
#' - **complex**.-- returns only the most complex formula.
#'
#' @return
#' A character vector containing the resulting formula(s).
#'
#' @importFrom utils combn
#'
#' @export
#' @rdname get_formulas
#'
#' @examples
#' # example variables
#' dep <- "sp"
#' ind <- c("temp", "rain", "slope")
#'
#' # The most complex formula according to "type"
#' get_formulas(dep, ind, type = "lqp", mode = "complex")
#'
#' # mode = 'light', combinations according to type
#' get_formulas(dep, ind, type = "lqp", mode = "light")
#'
#' # mode = 'light', combinations according to type
#' get_formulas(dep, ind, type = "lqp", mode = "intensive")

get_formulas <- function(dependent, independent, type = "l",
                         mode = "moderate", minvar = 1, maxvar = NULL) {

  # initial test
  if (!is.character(dependent) || length(dependent) != 1) {
    stop("'dependent' must be a unique response variable name.")
  }
  if (!is.character(dependent)) {
    stop("'independent' must be a character vector.")
  }
  if (is.character(type)) {
    if (!all(unlist(strsplit(type, "")) %in% c("l", "p", "q"))) {
      stop("'type' must be: 'l', 'p', 'q', or a combination of those three.")
    }
  } else {
    stop("'type' must be of class character.")
  }

  if (! mode %in% c("light", "moderate", "intensive", "complex")) {
    stop("'mode' must be defined as 'light', 'moderate', 'intensive' or 'complex'")
  }

  if (mode == "light") {

    red_var_comb <- aux_var_comb(independent, minvar = 2, maxvar = maxvar)
    reponse_comb <- aux_string_comb(type)

    output <-lapply(red_var_comb, function(y){
      get_formulas_main(dependent, independent = y, type = type, complex = TRUE)
    })

    return(unlist(output, use.names = F ))
  }

  if (mode == "moderate") {

    red_var_comb <- aux_var_comb(independent, minvar = 2, maxvar = maxvar)
    reponse_comb <- aux_string_comb(type)

    output <- lapply(reponse_comb, function(x){
      lapply(red_var_comb, function(y){

        get_formulas_main(dependent, independent = y, type = x, complex = TRUE)
      })
    })

    return(unlist(output, use.names = F ))
  }

  if (mode == "intensive"){

    output <- get_formulas_main(dependent, independent, type = type,
                                complex = FALSE, minvar = minvar,
                                maxvar = maxvar)
    return(output)
  }

  if (mode == "complex"){
    output <- get_formulas_main(dependent, independent, type = type,
                                complex = TRUE, minvar = minvar,
                                maxvar = maxvar)
    return(output)
  }
}

#' @export
#' @rdname get_formulas
#' @param complex (logical) whether to return the most complex formula.
#' @usage
#' get_formulas_main(dependent, independent, type = "l",
#'                   complex = FALSE, minvar = 1, maxvar = NULL)

get_formulas_main <- function(dependent, independent, type = "l",
                              complex = FALSE, minvar = 1 , maxvar = NULL) {

  # initial test
  if (!is.character(dependent) || length(dependent) != 1) {
    stop("'dependent' must be a unique response variable name.")
  }
  if (!is.character(dependent)) {
    stop("'independent' must be a character vector.")
  }
  if (is.character(type)) {
    if (!all(unlist(strsplit(type, "")) %in% c("l", "p", "q"))) {
      stop("'type' must be: 'l', 'p', 'q', or a combination of those three.")
    }
  } else {
    stop("'type' must be of class character.")
  }

  if (!is.logical(complex)) {
    stop("'complex' must be logical (TRUE or FALSE).")
  }

  predictors <- independent
  npred <- length(predictors)

  # remove the response variable if given in data
  if (dependent %in% predictors) {
    predictors <- predictors[predictors != dependent]
  }

  # produce formulas
  aux <- " "

  # Lineal response
  if (grepl("l", type)) {
    aux <- paste(aux, paste(predictors, collapse = " + "), sep = " + ")
  }

  # Quadratic response
  if (grepl("q", type)) {
    for (v in 1:length(predictors)) {
      aux <- paste(aux, paste0("I(", predictors[v], "^2)"), sep = " + ")
    }
  }

  # Interaction between two variables
  if (grepl("p", type)) {
    if (npred > 1) {
      inter_tab <- utils::combn(predictors, 2)
      aux_inter <- paste0(" + ",
                          paste(apply(inter_tab, 2, paste, collapse = ":"),
                                collapse = " + "))

      if (length(aux_inter) > 0) {
        aux <- paste0(aux, aux_inter)
      }
    } else {
      if (grepl("l", type) | grepl("q", type)) {
        message("'p' is is only possible with 2 or more independent variables.",
                "\nReturning other combinations.")
      } else {
        stop("'p' is is only possible with 2 or more independent variables.",
             "\nTry other combinations of type.")
      }
    }
  }

  # Create all possible combination
  if (!complex) {
    # unlist predictors in formula
    vec <- unlist(strsplit(gsub(" ", "", aux), split = "[+]"))[-1]

    if (!is.null(maxvar)){
      if (maxvar > length(vec)) {
        message("Maximum number of features: ", length(vec))
        maxvar = length(vec)
      }
    } else {
      maxvar = length(vec)
    }

    range_var <- minvar:maxvar

    # if (length(range_var) >= 20){
    #   l <- length(range_var)
    #   range_var <- range_var[-c(l %/% 2 - 1, l %/% 2, l %/% 2 + 1)]
    #
    # }

    ## Get all combinations
    all_comb <- lapply(range_var, utils::combn, x = vec, simplify = FALSE)
    all_comb <- unlist(all_comb, recursive = FALSE)

    output <- sapply(all_comb, function(x) {
      paste0(dependent, " ~ ", paste(x, collapse = " + "))
    })

  } else{
    output <- gsub("  \\+", paste(dependent, "~"), aux)
  }

  # Return formula(s)
  return(output)
}

#' @export
#' @rdname get_formulas
#' @param var_names sames as `independent`.

# Variable combination from KUENM
aux_var_comb <- function(var_names, minvar = 2, maxvar = NULL) {

  if(is.null(maxvar)) {maxvar <- length(var_names)}

  var_comb <- lapply(minvar:maxvar, function(x) {
    comb <- combn(var_names, m = x)
    comb_vs <- lapply(1:dim(comb)[2], function(y) {comb[, y]})
  })

  var_combs <- do.call(c, var_comb)
  names(var_combs) <- paste0("Set_", 1:length(var_combs))
  return(var_combs)
}

#' @export
#' @rdname get_formulas
#' @param string same as `type`.

# Function to generate all possible combinations of a string
aux_string_comb <- function(string) {

  bins <- strsplit(string, '')[[1]]
  n <- length(bins)

  all_comb <- lapply(1:n, function(i) {combn(bins, i, simplify = FALSE)})
  all_comb <- unlist(all_comb, recursive = FALSE)

  output <- lapply(all_comb, function(x){paste(x, collapse = "")})
  return(output)
}

#' Variable importance for GLMs
#'
#' @description
#' Calculates the relative importance of predictor variables based on the
#' concept of explained deviance. This is achieved by fitting a GLMs multiple
#' times, each time leaving out a different predictor variable to observe its
#' impact on the model's performance.
#'
#' @usage
#' var_importance(fitted, modelID = NULL, data = NULL)
#'
#' @param fitted an object of class `glm`, a list of GLMs obtained using the
#' function \code{\link{fit_glms}}, or an object `enmpa_fitted_models` from
#' \code{\link{fit_selected}} .
#' @param modelID (character) vector of ModelID(s) to be considered when the
#' `fitted` input is of the class `enmpa_fitted_models`. By default all models
#' are included. Default = NULL.
#' @param data data.frame or matrix of data used in the model calibration step.
#' It must be defined in case the model entered does not explicitly include a
#' data component. Default = NULL.
#'
#' @details
#' The process begins by fitting the full GLM model, which includes all predictor
#' variables. Subsequently, separate GLM models are fitted, excluding one
#' variable at a time to assess the influence of its absence on the model's
#' performance. By systematically evaluating the effect of removing each
#' predictor variable, the function provides valuable insights into their
#' individual contributions to the model's overall performance and explanatory
#' power.
#'
#' @return
#' A data.frame containing the relative contribution of each variable. An
#' identification for distinct models is added if `fitted` contains multiple
#' models.
#'
#' @export
#'
#' @importFrom stats update as.formula deviance coef
#'
#' @examples
#' # Load a fitted selected model
#' data(sel_fit, package = "enmpa")
#'
#' # Variable importance for single models
#' var_importance(sel_fit, modelID = "ModelID_7")
#'
#' # Variable importance for multiple models (only one model in this list)
#' var_importance(sel_fit)

var_importance <- function(fitted, modelID = NULL, data = NULL){
  # initial tests
  if (missing(fitted)) {
    stop("Argument 'model' must be defined.")
  }

  if (!class(fitted)[1] %in% c("glm", "enmpa_fitted_models", "list")){
    stop("'fitted' must be of class 'glm','list', or 'enmpa_fitted_models'")
  }

  #  for an individual GLM______________________________________________________
  if (class(fitted)[1] == "glm") {

    if (!is.null(fitted$data)){
    tab_contr <- var_importance_ind(fitted, data = fitted$data,
                                    weights = fitted$weights)

    } else {
      stop(paste0("The argument 'data' must be defined in case the model entered",
                  " does not explicitly include a data component."))
    }
  }

  #  List of GLMs_______________________________________________________________
  if (check_if_glm_list(fitted)){

    # if data argument is not empty
    if (!is.null(data) && length(fitted) > 1 ){
      aux <- lapply(fitted, function(y) {var_importance_ind(y, data, weights)})
      tab_contr <- do.call(rbind, aux)[, 1:2]
      tab_contr$Models <- rep(names(aux), times = sapply(aux, nrow))
      rownames(tab_contr) <- NULL

      # standardize product names
      tab_contr$predictor <- standardize_interaction_names(tab_contr$predictor)
    }

    if (!is.null(data) && length(fitted) == 1 ){
      tab_contr <- var_importance_ind(fitted[1][[1]], data = fitted$data,
                                      weights = fitted$weights)
    }

    # if data argument is empty
    if (is.null(fitted[1][[1]]$data)){
      stop("Calibration data must be defined.")

    } else {
      data <- fitted[1][[1]]$data

      if (length(fitted) > 1){
        aux <- lapply(fitted, function(y) {var_importance_ind(y, data, weights)})
        tab_contr <- do.call(rbind, aux)[, 1:2]
        tab_contr$Models <- rep(names(aux), times = sapply(aux, nrow))
        rownames(tab_contr) <- NULL

        # standardize product names
        tab_contr$predictor <- standardize_interaction_names(tab_contr$predictor)

      } else {
        tab_contr <- var_importance_ind(fitted[1][[1]], data = fitted$data,
                                        weights = fitted$weights)
      }
    }
  }


  #  using de 'enmpa fitted object'_____________________________________________
  if (class(fitted)[1] == "enmpa_fitted_models") {

    data  <- fitted$data
    weights <- fitted$weights
    list_glms <-  fitted$glms_fitted

    if (is.null(modelID)){

      aux <- lapply(list_glms, function(y) {var_importance_ind(y, data, weights)})
      tab_contr <- do.call(rbind, aux)[, 1:2]
      tab_contr$Models <- rep(names(aux), times = sapply(aux, nrow))
      rownames(tab_contr) <- NULL

      # standardize product names
      tab_contr$predictor <- standardize_interaction_names(tab_contr$predictor)

    } else {
      tab_contr <- var_importance_ind(list_glms[modelID][[1]], data, weights )
    }
  }
  return(tab_contr)

}

#' Print a short version of elements in 'calibration' and 'fitted models' objects
#'
#' @name print
#' @aliases print,enmpa_calibration-method print,enmpa_calibration-method
#' @aliases print,enmpa_fitted_models-method print,enmpa_fitted_models-method
#' @param x object of enmpa_fitted_models or enmpa_calibration
#' @param ... additional arguments affecting the summary produced. Ignored in
#' these functions.
#'
#' @export
#' @rdname print

print.enmpa_calibration <- function(x, ...) {
  cat("enmpa-class `enmpa_calibration`:\n")
  cat(paste0(
    "$selected             : Selected models (N = ", nrow(x$selected), ")\n"))
  cat(
    "$summary              : A summary of statistics for all models. \n")
  cat(
    "$calibration_results  : Results obtained from cross-validation for all models. \n")
  cat(
    "$data                 : Data used for calibration. \n")
  cat(paste0(
    "$partitioned_data     : k-fold indexes (k = ", length(x$partitioned_data), ")\n"))
  cat(paste0(
    "$weights              : Use of weights (", !is.null(x$weights), ")\n"))
  invisible(x)

}

#' @export
#' @rdname print

print.enmpa_fitted_models <- function(x, ...) {
  cat("enmpa-class `fitted models`:\n")
  cat(paste0(
    "$glms_fitted    : List of GLMs fitted (N = ", length(x$glms_fitted), ")\n"
  ))
  cat(paste0(
    "$selected       : Selected models (N = ", nrow(x$selected), ")\n"
  ))
  cat(
    "$data           : Data used for calibration. \n"
  )
  cat(paste0(
    "$weights        : Use of weights (", !is.null(x$weights), ")\n"
  ))
  invisible(x)
}


#' Summary of 'calibration' and 'fitted models'
#' @name summary
#' @aliases summary,enmpa_calibration-method summary,enmpa_calibration-method
#' @aliases summary,enmpa_fitted_models-method summary,enmpa_fitted_models-method
#' @param object of class enmpa_calibration or enmpa_fitted_models
#' @param ... additional arguments affecting the summary produced. Ignored in
#' these functions.
#' @export
#' @return
#' A printed summary.
#' @rdname summary

summary.enmpa_calibration <- function(object, ...) {
  cat("\n                     Summary of enmpa_calibration                  \n")
  cat("-------------------------------------------------------------------\n\n")
  cat(paste0(
    "Number of selected models: ", nrow(object$selected), "\n"))
  cat(paste0(
    "Number of partitions: (k = ", length(object$partitioned_data), ")\n"))
  cat(paste0(
    "Weights used: ", ifelse(is.null(object$weights), "No", "Yes"), "\n"))
  cat(
    "Summary of selected models (threshold criteria = ",
    object$selected$Threshold_criteria[1],
    "):\n")
  print(object$selected[,c("ModelID", "ROC_AUC_mean", "Accuracy_mean",
                           "Specificity_mean", "Sensitivity_mean" ,"TSS_mean",
                           "AIC", "Delta_AIC")
                        ]
        )
  invisible(object)
}

#' @export
#' @rdname summary

summary.enmpa_fitted_models <- function(object, ...) {
  cat("\n                     Summary of enmpa_fitted_models                \n")
  cat("-------------------------------------------------------------------\n\n")
  cat(paste0("Number of GLMs fitted: ", length(object$glms_fitted), "\n"))
  cat(paste0("Weights used: ", ifelse(is.null(object$weights), "No", "Yes"), "\n"))
  cat("Formulas of fitted models:\n")

  for (i in (names(object$glms_fitted))){
    y <- object$selected[object$selected$ModelID == i, "Formulas"]
    cat(paste0(i,":\t",y, "\n"))
  }
  invisible(object)
}

#' Evaluate final models using independent data
#'
#' @description
#' Final evaluation steps for model predictions using an independent dataset
#' (not used in model calibration).
#'
#' @usage
#' independent_eval1(prediction, threshold, test_prediction = NULL,
#'                   lon_lat = NULL)
#'
#' @param prediction (numeric) vector or `SpatRaster` object. If numeric,
#' predicted values in independent data (for \code{\link{independent_eval01}}),
#' or the entire area of prediction (for \code{\link{independent_eval1}}).
#' If `SpatRaster` prediction over the area of interest.
#' @param threshold (numeric) the lowest predicted probability value for an
#' occurrence point. This value must be defined for presences-only data.
#' Default = NULL.
#' @param test_prediction (numeric) vector of predictions for independent data.
#' Default = NULL.
#' @param lon_lat matrix or data.frame of coordinates (longitude and latitude,
#' in that order) of independent data. Points must be located within the valid
#' area of `prediction`. For \code{\link{independent_eval01}} they
#' must correspond with values in observation.
#'
#' @return
#' A data.frame or list containing evaluation results.
#'
#' @export
#'
#' @rdname independent_eval
#'
#' @importFrom terra extract
#'
#' @examples
#' # Independent test data based on coordinates (lon/lat WGS 84) from presence
#' # and absences records
#' data("test", package = "enmpa")
#' head(test)
#'
#' # Loading a model prediction
#' pred <- terra::rast(system.file("extdata", "proj_out_wmean.tif",
#'                                 package = "enmpa"))
#' terra::plot(pred)
#'
#' # Evaluation using presence-absence data
#' independent_eval01(prediction = pred, observation = test$Sp,
#'                    lon_lat = test[, 2:3])
#'
#' # Evaluation using presence-only data
#' test_p_only <- test[test$Sp == 1, ]
#' th_maxTSS   <- 0.1274123            # threshold based on the maxTSS
#'
#' independent_eval1(prediction = pred, threshold = th_maxTSS,
#'                   lon_lat = test_p_only[, 2:3])

independent_eval1 <- function(prediction, threshold, test_prediction = NULL,
                              lon_lat = NULL) {

  # initial test
  if (missing(prediction) | missing(threshold)) {
    stop("Arguments 'prediction' and 'threshold' must be defined.")
  }
  if (threshold <= 0 | threshold >= 1) {
    stop("Threshold value must be within the range 0-1.")
  }
  if (class(prediction)[1] == "SpatRaster" & is.null(lon_lat)) {
    stop("Argument 'lonlat' is required if 'prediction' is a 'SpatRaster'.")
  }
  if (class(prediction)[1] == "numeric" & is.null(test_prediction)) {
    stop("Argument 'test_prediction' is required if 'prediction' is a 'numeric' vector.")
  }

  if (class(prediction)[1] == "SpatRaster") {
    if (!class(lon_lat)[1] %in% c("matrix", "data.frame")) {
      stop("'lonlat' must be a 'matrix' or a 'data.frame'.")
    }
    # transform to a Spatvector object and extract predicted values
    prd <- terra::extract(prediction, as.matrix(lon_lat))[, 1]
  } else {
    prd <- test_prediction
  }

  # Omission error (false negative rate)
  oe <- 1 - sum(prd >= threshold) / length(prd)

  # pROC
  proc <- proc_enm(test_prediction = prd, prediction = prediction,
                   threshold = round(threshold * 100, 2))

  # results
  oe_out <- data.frame(omission_error = oe,
                       threshold = threshold,
                       Mean_AUC_ratio = proc$pROC_summary[[1]],
                       pval_pROC = proc$pROC_summary[[2]], row.names = NULL)

  return(oe_out)
}

#'
#' @rdname independent_eval
#' @param observation (numeric) vector of observed (known) values of presence
#' or absence to test against `prediction` (if numeric) or values of prediction
#' in `lon_lat`.
#' @export
#'

independent_eval01 <- function(prediction, observation, lon_lat = NULL) {

  # initial test
  if (missing(prediction) | missing(observation)) {
    stop("Arguments 'prediction' and 'observation' must be defined.")
  }
  if (!all(observation %in% c(0, 1))) {
    stop("'observation' contain non-valid values.")
  }
  if (class(prediction)[1] == "SpatRaster" & is.null(lon_lat)) {
    stop("Argument 'lonlat' is required if 'prediction' is a 'SpatRaster'.")
  }

  if (class(prediction)[1] == "SpatRaster") {
    if (!class(lon_lat)[1] %in% c("matrix", "data.frame")) {
      stop("'lonlat' must be a 'matrix' or a 'data.frame'.")
    }
    # transform to a Spatvector object and extract predicted values
    prediction <- terra::extract(prediction, as.matrix(lon_lat))[, 1]
  }

  # evaluation
  metrics <- optimize_metrics(actual = observation, predicted = prediction,
                              n_threshold = 1000)

  return(metrics$optimized)
}

#' Jackkniffe plot for variable contribution
#'
#' @description
#' The jackknife figure shows the impact of each variable on the full model,
#' providing detailed information about the function and significance of each
#' variable. Light blue indicates the impact on the model if the variable is not
#' included, while dark blue indicates the independent contribution of the
#' variable to the model.
#'
#' @usage
#' plot_jk(x, metric = "ROC_AUC", legend = TRUE,
#'         colors = c("cyan", "blue", "red"),
#'         xlab = NULL, main = NULL)
#'
#' @param x list output from \code{\link{jackknife}}.
#' @param metric (character) model metric to plot. Default = "ROC_AUC".
#' @param legend (logical) whether to add legend. Default = TRUE.
#' @param colors (character) vector of colors.
#' Default = c("cyan", "blue", "red").
#' @param xlab (character) a label for the x axis.
#' @param main (character) main title for the plot.
#'
#' @export
#' @importFrom graphics barplot par abline legend
#'

plot_jk <- function(x, metric = "ROC_AUC", legend = TRUE,
                    colors = c("cyan", "blue", "red"),
                    xlab = NULL, main = NULL){

  if (missing(x)) {
    stop("Arguments 'x' must be defined.")
  }

  if (!is.character(colors) && is.vector(colors) && length(colors) == 3){
    colors = c("cyan", "blue", "red")
  }

  # get matrix
  m <- as.matrix(cbind(x$Without[metric], x$With_only[metric]))
  colnames(m) <- c("without", "with_only")

  fm <- x[["Full_model_stats"]][[metric]]

  if(is.null(xlab)){xlab = metric}
  if(is.null(main)){main = paste("Jackknife of", metric)}

  if (metric %in% c("ROC_AUC", "TSS")){
    # xlim = c(0,1)}
    xlim = c(0, max(c(max(m), fm)) + 0.1)
  } else {
    xlim = c(0, max(c(max(m), fm)) + 0.10*max(c(max(m), fm)))
  }

  # Adjust margins to accommodate variable names and legend
  original_par <- par(no.readonly = TRUE)  # Save the original par settings

  if (legend){
    par(mar = c(5, 8, 4, 8) + 0.1)
  } else {
    par(mar = c(5, 8, 4, 6) + 0.1)
  }

  barplot(t(m),
          beside = TRUE,
          horiz = TRUE,
          xlim = xlim,
          las=1,
          col =  colors[1:2],
          xlab = xlab,
          cex.names = 1,
          main = main)
  abline(v = fm, col = colors[3], lty = "dotted", lwd = 2)

  if (legend){
    legend("topright",
           inset=c(-0.285, -0.075), xpd = TRUE, horiz = FALSE, bty = "n",
           cex = 0.95,
           legend=c("Without variable", "With only variable", "Full model"),
           fill = colors)
  }
  # Restore the original par settings
  par(original_par)
}

#' Fitting selected GLMs models
#'
#' @description
#' Functions to facilitate fitting multiple GLMs.
#'
#' @param glm_calibration a list resulting from \code{\link{calibration_glm}}.
#' Models fitted are those in the slot "selected".
#' @param formulas (character) a vector containing the formula(s) for GLM(s).
#' @param data data.frame with the dependent and independent variables.
#' @param weights (numeric) a vector with the weights for observations.
#' Default = NULL.
#' @param id (character) id code for models fitted. Default = NULL.
#'
#' @return
#' A list of fitted GLMs.
#'
#' For `fit_selected`, an enmpa `fitted models` object.
#'
#' @export
#' @importFrom stats glm as.formula binomial
#' @rdname fit_glms
#'
#' @examples
#' # GLM calibration results
#' data(cal_res, package = "enmpa")
#'
#' # Fitting selected models
#' sel_fit <- fit_selected(cal_res)
#'
#' sel_fit
#'
#' # Custom formulas
#' forms <- c("Sp ~ bio_1 + I(bio_1^2) + I(bio_12^2)",
#'            "Sp ~ bio_12 + I(bio_1^2) + I(bio_12^2)")
#'
#' # Fitting models
#' fits <- fit_glms(forms, data = cal_res$data)
#'
#' fits$ModelID_1

fit_selected <- function(glm_calibration) {
  if (missing(glm_calibration)) {
    stop("Arguments 'glm_calibration' must be defined.")
  }

  glms_fitted_list_full <- fit_glms(formulas = glm_calibration$selected$Formulas,
                                    data = glm_calibration$data,
                                    weights = glm_calibration$weights,
                                    id = glm_calibration$selected$ModelID)

  # deleted redundant data to save memory
  glms_fitted_list_full <- lapply(glms_fitted_list_full, function(x){
    x$data <- NULL
    x$weights <- NULL
    return(x)
  })

  output <- new_enmpa_fitted_models(data = glm_calibration$data,
                                    selected = glm_calibration$selected,
                                    weights = glm_calibration$weights,
                                    glms_fitted = glms_fitted_list_full)
  return(output)
}


#' @rdname fit_glms
#' @usage fit_glms(formulas, data, weights = NULL, id = NULL)
#' @export

fit_glms <- function(formulas, data, weights = NULL, id = NULL) {
  if (missing(formulas)) {
    stop("Arguments 'formulas' must be defined.")
  }
  if (missing(data)) {
    stop("Arguments 'data' must be defined.")
  }

  # Model fitting
  fits <- lapply(formulas, function(y) {
    suppressWarnings(
      glm(formula = as.formula(y), family = binomial(link = "logit"),
          data = data, weights = weights)
    )
  })

  if (is.null(id)) {
    names(fits) <- paste0("ModelID_", 1:length(formulas))
  } else {
    names(fits) <- id
  }
  return(fits)
}

#' Extension of glm predict to generate predictions of different types
#'
#' @description
#' Obtains predictions from a fitted generalized linear model objects. It also
#' allows the clamping option to restrict extrapolation in areas outside the
#' calibration area.
#'
#' @param model a `glm` object.
#' @param newdata a data.frame or matrix with the new data to project the
#' predictions.
#' @param data data.frame or matrix of data used in the model calibration step.
#' Default = NULL.
#' @param extrapolation_type (character) to indicate extrapolation type of model.
#' Models can be transferred with three options: free extrapolation ('E'),
#' extrapolation with clamping ('EC'), and no extrapolation ('NE').
#' Default = 'E'.
#' @param restricted_vars (character) a vector containing the names of the
#' variables that will undergo clamping or no extrapolation. For clamping,
#' these variables are set to minimum and maximum values established for the
#' max and min values within calibration values. For no extrapolation, the
#' variables outside calibration limits became NA. If no specific names are
#' provided, the value is set to NULL by default, indicating that clamping (EC)
#' or no extrapolation (NE) will be applied to all variables. Ignore if
#' extrapolation_type = 'E'.
#' @param type (character) the type of prediction required. For a default
#' binomial model the default predictions are of log-odds (probabilities on
#' logit scale). The default, "response", returns predicted probabilities.
#'
#' @return
#' A `SpatRaster` object or a vector with predictions.
#'
#' @export
#' @importFrom terra as.data.frame predict setValues
#' @importFrom stats coef predict.glm
#'
#' @examples
#' # Load fitted model
#' data("sel_fit", package = "enmpa")
#'
#' # Load raster layers to be projected
#' env_vars <- terra::rast(system.file("extdata", "vars.tif", package = "enmpa"))
#'
#' # Prediction
#' pred <- predict_glm(sel_fit$glms_fitted$ModelID_7, newdata = env_vars,
#'                     data = sel_fit$data)
#' terra::plot(pred)


predict_glm <- function(model, newdata, data = NULL, extrapolation_type = "E",
                        restricted_vars = NULL, type = "response") {

  # initial test
  if(missing(model) | missing(newdata)){
    stop("'model' or 'newdata' must be defined.")
  }

  if (!is.null(newdata)) {
    if (!class(newdata)[1] %in% c("matrix", "data.frame", "SpatRaster")) {
      stop("'newdata' must be of class 'matrix', 'data.frame' or 'SpatRaster'")
    }
  }

  if (!extrapolation_type %in% c("E", "NE", "EC")){
    stop("'extrapolation_type' must be of class 'E','NE', or 'EC'")
  }

  # check calibration data availability
  if (is.null(data) && is.null(model$data)){
    stop("Calibration data must be defined.")
  } else {
    if (!is.null(model$data)){
      data  <- model$data
    }
  }

  ## Extrapolation type:

  # E = "Free extrapolation"____________________________________________________
  if (extrapolation_type == "E"){
    # Prediction
    if (class(newdata)[1] == "SpatRaster") {
      out <- terra::predict(newdata, model,  type = type)
    } else {
      out <- predict.glm(model, newdata, type = type)
    }
  }

  # NE = "No extrapolation"_____________________________________________________
  if (extrapolation_type == "NE") {

    if (class(newdata)[1] == "SpatRaster") {
      newdata_aux <- terra::as.data.frame(newdata, na.rm = FALSE)
    } else {
      newdata_aux <- newdata
    }

    # It gets only the variable names used in the fitted model
    modelp <- sapply(colnames(data), grepl, names(coef(model)[-1]))
    vnames <- colSums(modelp) > 0

    # min and max limit of the calibration data
    cal_data <- data[, vnames]
    varmin <- apply(cal_data, 2, FUN = min)
    varmax <- apply(cal_data, 2, FUN = max)

    if (!is.null(restricted_vars)){

      if (!is.vector(restricted_vars) | any(!restricted_vars %in% names(newdata_aux))){
        stop("'restricted_vars' must provide a vector with the variables names to be restricted.")
      } else {
        # no extrapolation for custom variables
        nep <- sapply(restricted_vars, function(v){
          which(newdata_aux[, v] < varmin[v] | newdata_aux[, v] > varmax[v])
        })
        nep <- unique(unname(do.call("c", nep)))
      }
    } else {
      # Clamping for all variables
      nep <- sapply(intersect(names(varmax), names(newdata_aux)), function(v){
        which(newdata_aux[, v] < varmin[v] | newdata_aux[, v] > varmax[v])
      })
      nep <- unique(unname(do.call("c", nep)))
    }

    # Prediction with NE
    if (class(newdata)[1] == "SpatRaster") {
      out <- terra::predict(newdata, model,  type = type)
      out[nep] <- 0
    } else {
      out <- predict.glm(model, newdata, type = type)
      out[nep] <- 0
    }
  }

  # EC = "Extrapolation with clamping"_________________________________________
  if (extrapolation_type == "EC") {
    if (class(newdata)[1] == "SpatRaster") {
      newdata_aux <- terra::as.data.frame(newdata, na.rm = FALSE)
    } else {
      newdata_aux <- newdata
    }

    # It gets only the variable names used in the fitted model
    modelp <- sapply(colnames(data), grepl, names(coef(model)[-1]))
    vnames <- colSums(modelp) > 0

    # min and max limit of the calibration data
    cal_data <- data[, vnames]
    varmin <- apply(cal_data, 2, FUN = min)
    varmax <- apply(cal_data, 2, FUN = max)

    if (!is.null(restricted_vars)){

      if (!is.vector(restricted_vars) | any(!restricted_vars %in% names(newdata_aux))){
        stop("'restricted_vars' must provide a vector with the variables names to be restricted.")
      } else {
        # Clamping for  custom variables
        for (v in restricted_vars) {
          newdata_aux[,v] <- pmin(pmax(newdata_aux[,v], varmin[v]), varmax[v])
        }
      }
    } else {
      # Clamping for all variables
      for (v in intersect(names(varmax), names(newdata_aux))) {
        newdata_aux[,v] <- pmin(pmax(newdata_aux[,v], varmin[v]), varmax[v])
      }
    }
    if (class(newdata)[1] == "SpatRaster") {
      newdata <- setValues(newdata, newdata_aux)
    } else{
      newdata <- newdata_aux
    }
    # Prediction
    if (class(newdata)[1] == "SpatRaster") {
      out <- terra::predict(newdata, model,  type = type)
    } else {
      out <- predict.glm(model, newdata, type = type)
    }
  }
  return(out)
}

#' Summary of evaluation statistics for candidate models
#'
#' @description
#' Calculate median and standard deviation of evaluation results for all
#' candidate models considering cross-validation kfolds.
#'
#' @usage
#' evaluation_stats(evaluation_results, bimodal_toexclude = FALSE)
#'
#' @param evaluation_results data.frame model evaluation results. These results
#' are the output of the function \code{\link{model_validation}}.
#' @param bimodal_toexclude (logical) whether models in which binomial variable
#' response curves were detected will be excluded during selection processes.
#'
#' @return
#' A data.frame with the mean and standard deviation for all metrics considering
#' cross-validation kfolds.
#'
#' @export
#'
#' @examples
#' # data
#' data("cal_res", package = "enmpa")
#' all_res <- cal_res$calibration_results[, -1]
#'
#' # statistics for all evaluation results
#' evaluation_stats(all_res, bimodal_toexclude = TRUE)

evaluation_stats <- function(evaluation_results, bimodal_toexclude = FALSE) {

  if (missing(evaluation_results)) {
    stop("Argumet 'evaluation_results' must be defined.")
  }

  cv_kfolds <- max(evaluation_results$Kfold)

  if (bimodal_toexclude) {
    toagg <- colnames(evaluation_results)[4:(ncol(evaluation_results) - 1)]
    bim <- evaluation_results[seq(1, nrow(evaluation_results), (cv_kfolds * 3)),
                              "Concave_responses"]

    agg_formula <- "~ Formulas + Threshold_criteria + Concave_responses"
    msd <- 4:5

  } else {
    toagg <- colnames(evaluation_results)[4:ncol(evaluation_results)]

    agg_formula <- "~ Formulas + Threshold_criteria"
    msd <- 3:4
  }

  # summary stats
  xy <- lapply(toagg, function(x) {
    do.call(
      data.frame,
      aggregate(
        as.formula(paste(x, agg_formula)),
        data = evaluation_results,
        FUN = function(y) c(mean = round(mean(y), 4), sd = round(sd(y), 4))
      )
    )
  })

  # put summary together
  stats <- do.call(data.frame, lapply(xy, function(y) {y[, msd]}))
  colnames(stats) <- unlist(lapply(xy, function(y) {colnames(y[, msd])}))

  # remove sd for Devaince, AIC and parameters
  stats <- cbind(xy[[1]][, 1:2] , stats[, -c(16, 18, 20)])
  colnames(stats)[c(17, 18, 19)] <- c("Parameters", "Deviance", "AIC")
  colnames(stats) <- gsub(".", "_", colnames(stats), fixed = TRUE)


  if (bimodal_toexclude) {
    stats$Concave_responses <- xy[[1]][, 3]
  }

  # sort data by formula (more simple to complex)
  ii <- order(factor(stats$Formulas, levels = unique(evaluation_results$Formulas)))
  stats <- stats[ii,]
  rownames(stats) <- 1:nrow(stats)

  # delta and weight of AIC for the aggregated data
  AICs <- stats[!duplicated(stats$Formulas), "AIC"] # per individual model
  Delta_AIC <- AICs - min(AICs, na.rm = TRUE)
  AIC_weight <- exp(-0.5 * Delta_AIC)
  AIC_weight <- AIC_weight / sum(AIC_weight, na.rm = TRUE)

  stats$Delta_AIC <- rep(Delta_AIC, each = length(unique(stats$Threshold_criteria)))
  stats$AIC_weight <- rep(AIC_weight, each = length(unique(stats$Threshold_criteria)))

  if (bimodal_toexclude) {stats <- data.frame(stats[, c(1:19, 21, 22, 20)])}

  return(stats)
}

#' enmpa: Ecological Niche Modeling using Presence-Absence Data
#'
#' @description
#' `enmpa` contains a set of tools to perform detailed Ecological Niche Modeling
#' using presence-absence data.
#'
#' @details
#' It includes algorithms for data partitioning, model fitting, calibration,
#' evaluation, selection, and prediction. Other functions help to explore model
#' features as such variable response curves and variable importance.
#'
#' @section Main functions in `enmpa`:
#' \code{\link{calibration_glm}}, \code{\link{evaluation_stats}},
#' \code{\link{fit_glms}}, \code{\link{fit_selected}},
#' \code{\link{get_formulas}}, \code{\link{independent_eval1}},
#' \code{\link{kfold_partition}}, \code{\link{model_selection}}
#' \code{\link{model_validation}}, \code{\link{niche_signal}},
#' \code{\link{optimize_metrics}}, \code{\link{predict_glm}},
#' \code{\link{predict_selected}}, \code{\link{response_curve}},
#' \code{\link{resp2var}}, \code{\link{var_importance}},
#' \code{\link{jackknife}}, \code{\link{plot_jk}}
#'
#' @name enmpa

## usethis namespace: start
#' @importFrom Rcpp sourceCpp
## usethis namespace: end
"_PACKAGE"

#' Variable response curves for GLMs
#'
#' @description
#' A view of variable responses in models. Responses based on single or multiple
#' models can be provided.
#'
#' @usage
#' response_curve(fitted, variable, data = NULL, modelID = NULL,  n = 100,
#'                new_data = NULL,  extrapolate = TRUE, show_lines = TRUE,
#'                xlab = NULL, ylab = "Probability", col = "red", ...)
#
#' @param fitted an object of class `glm`, a list of GLMs obtained using the
#' function \code{\link{fit_glms}},  or an object `enmpa_fitted_models` from
#' \code{\link{fit_selected}} .
#' @param variable (character) name of the variables to be plotted.
#' @param data data.frame or matrix of data used in the model calibration step.
#' Default = NULL.
#' @param modelID (character) vector of ModelID(s) to be considered when the
#' fitted models is an `enmpa_fitted_object`. By default all models are included.
#' Default = NULL.
#' @param n (numeric) an integer guiding the number of breaks. Default = 100
#' @param new_data a `SpatRaster`, data.frame, or  matrix of variables
#' representing the range of variable values in an area of interest.
#' Default = NULL. It must be defined in case the model entered does not
#' explicitly include a data component.
#' @param extrapolate (logical) whether to allow extrapolation to study the
#' behavior of the response outside the calibration limits. Ignored if
#' `new_data` is defined. Default = TRUE.
#' @param show_lines (logical) whether to show variable responses of distinct
#' models as different lines. Default = TRUE. If
#' @param xlab (character) a label for the x axis. The default, NULL, uses the
#' name defined in `variable`.
#' @param ylab (character) a label for the y axis. Default = "Probability".
#' @param col (character) color for lines. Default = "red".
#' @param ... additional arguments passed to \code{\link[graphics]{plot}}.
#'
#' @details
#' The function calculates these probabilities by focusing on a single
#' environmental variable while keeping all other variables constant at their
#' mean values.
#'
#' When responses for multiple models are to be plotted, and `show_lines` =
#' FALSE, the mean and confidence intervals for the set of responses are
#' calculated using a GAM.
#'
#' @return
#' A plot with the response curve for a `variable`.
#'
#' @export
#'
#' @importFrom stats predict coef
#' @importFrom graphics abline polygon
#' @importFrom terra minmax
#' @importFrom mgcv gam
#'
#' @examples
#' # Load a fitted selected model
#' data(sel_fit, package = "enmpa")
#'
#' # Response curve for single models
#' response_curve(sel_fit, modelID = "ModelID_7", variable = "bio_1")
#'
#' # Response curve when model(s) are in a list (only one model in this one)
#' response_curve(sel_fit, variable = "bio_12")

response_curve <- function(fitted, variable, data = NULL, modelID = NULL, n = 100,
                           new_data = NULL, extrapolate = TRUE, show_lines = TRUE,
                           xlab = NULL, ylab = "Probability", col = "red", ...) {

  # initial tests
  if (missing(fitted) | missing(variable)) {
    stop("Argument 'fitted' or 'variable' must be defined.")
  }

  if (!is.null(new_data)) {
    if (!class(new_data)[1] %in% c("matrix", "data.frame", "SpatRaster")) {
      stop("'new_data' must be of class 'matrix', 'data.frame', 'SpatRaster'")
    }
  }

  if (!is.null(data)) {
    if (!class(data)[1] %in% c("matrix", "data.frame")) {
      stop("'data' must be of class 'matrix', 'data.frame'")
    }
  }


  if (is.null(xlab)) {xlab <- variable}

  #  individual GLMs____________________________________________________________
  if (class(fitted)[1] == "glm") {
    response_curve_ind(fitted, variable, data = data, n = n,
                       new_data = new_data, extrapolate = extrapolate,
                       xlab = xlab, ylab = ylab, col = col, ...)
  }

  #  List of GLMs_______________________________________________________________
  if (check_if_glm_list(fitted)){

    # check availability
    if (is.null(data) && is.null(fitted[1][[1]]$data)){
      stop("Calibration data must be defined.")
    } else {
      if (!is.null(fitted[1][[1]]$data)){
        data  <- fitted[1][[1]]$data
      }
    }

    if (is.null(modelID)){

      response_curve_cons(fitted, variable, data = data, n = n, new_data = new_data,
                          extrapolate = extrapolate, show_lines = show_lines,
                          xlab = xlab, ylab = ylab, col = col, ...)
    } else {

      if (!modelID %in% names(fitted)){
        stop(paste0("The 'ModelID' is not correct, check the following:\n[",
                    paste(names(fitted), collapse = ", ")),
             "]"
        )
      }

      if (length(modelID) > 1){
        response_curve_cons(fitted[modelID], variable, data = data, n = n,
                            new_data = new_data, extrapolate = extrapolate,
                            show_lines = show_lines, xlab = xlab, ylab = ylab,
                            col = col, ...)
      } else {
        response_curve_ind(fitted[modelID][[1]], variable, data = data, n = n,
                           new_data = new_data, extrapolate = extrapolate,
                           xlab = xlab, ylab = ylab, col = col, ...)
      }
    }
  }

  #   # if data argument is not empty
  #   if (!is.null(data) && length(fitted) > 1 ){
  #     response_curve_cons(fitted, variable, data = data, n = n,
  #                         new_data = new_data, extrapolate = extrapolate,
  #                         xlab = xlab, ylab = ylab, col = col, ...)
  #   }
  #
  #   if (!is.null(data) && length(fitted) == 1 ){
  #     response_curve_ind(fitted[1][[1]], variable, data = data, n = n,
  #                         new_data = new_data, extrapolate = extrapolate,
  #                         xlab = xlab, ylab = ylab, col = col, ...)
  #   }
  #
  #   # if data argument is empty
  #   if (is.null(fitted[1][[1]]$data)){
  #     stop("Calibration data must be defined.")
  #
  #   } else {
  #     data <- fitted[1][[1]]$data
  #
  #     if (length(fitted) > 1){
  #       response_curve_cons(fitted, variable, data = data, n = n,
  #                           new_data = new_data, extrapolate = extrapolate,
  #                           xlab = xlab, ylab = ylab, col = col, ...)
  #     } else {
  #       response_curve_ind(fitted[1][[1]], variable, data = data, n = n,
  #                          new_data = new_data, extrapolate = extrapolate,
  #                          xlab = xlab, ylab = ylab, col = col, ...)
  #     }
  #   }
  # }

  #  using de 'enmpa fitted object'_____________________________________________
  if (class(fitted)[1] == "enmpa_fitted_models") {
    data  <- fitted$data
    list_glms <-  fitted$glms_fitted

    if (is.null(modelID)){

      response_curve_cons(list_glms, variable, data = data, n = n, new_data = new_data,
                          extrapolate = extrapolate, show_lines = show_lines,
                          xlab = xlab, ylab = ylab, col = col, ...)
    } else {

      if (!modelID %in% names(list_glms)){
        stop(paste0("The ModelID is not correct, check the following:\n[",
                    paste(names(list_glms), collapse = ", ")),
             "]"
        )
      }

      if (length(modelID) > 1){
        response_curve_cons(list_glms[modelID], variable, data = data, n = n,
                            new_data = new_data,extrapolate = extrapolate,
                            show_lines = show_lines, xlab = xlab, ylab = ylab,
                            col = col, ...)
      } else {
        response_curve_ind(list_glms[modelID][[1]], variable, data = data, n = n,
                           new_data = new_data, extrapolate = extrapolate,
                           xlab = xlab, ylab = ylab, col = col, ...)
      }
    }
  }
}

#' Plot variable importance
#'
#' @description
#' Visualization of the results obtained with the function
#' \code{\link{var_importance}}.
#'
#' @usage
#' plot_importance(x, xlab = NULL, ylab = "Relative contribution",
#'                 main = "Variable importance", extra_info = TRUE, ...)
#'
#'
#' @param x data.frame output from \code{\link{var_importance}}.
#' @param xlab (character) a label for the x axis.
#' @param ylab (character) a label for the y axis.
#' @param main (character) main title for the plot.
#' @param extra_info (logical) when results are from more than one model, it
#' adds information about the number of models using each predictor and the mean
#' contribution found.
#' @param ... additional arguments passed to \code{\link[graphics]{barplot}} or
#' \code{\link[graphics]{boxplot}}.
#'
#' @return
#' A plot
#'
#' @export
#'
#' @importFrom graphics barplot boxplot text
#'
#'
#' @examples
#'
#' # Load species occurrences and environmental data.
#' data("enm_data", package = "enmpa")
#'
#' # Custom formulas
#' forms <- c("Sp ~ bio_1 + I(bio_1^2) + I(bio_12^2)",
#'            "Sp ~ bio_12 + I(bio_1^2) + I(bio_12^2)")
#'
#' # Fitting models
#' fits <- fit_glms(forms, data = enm_data)
#'
#' # Variable importance for single models
#' vi_1 <- var_importance(fits$ModelID_1)
#' plot_importance(x = vi_1)
#'
#' vi_2 <- var_importance(fits$ModelID_2)
#' plot_importance(x = vi_2)
#'
#' # Variable importance for multiple models
#' vi_c <- var_importance(fits)
#' plot_importance(x = vi_c)


plot_importance <- function(x, xlab = NULL, ylab = "Relative contribution",
                            main = "Variable importance", extra_info = TRUE,
                            ...) {

  if (missing(x)) {
    stop("Argument 'x' must be defined.")
  }

  # Check if single or multiple models
  if ("Models" %in% colnames(x)) {

    if (length(unique(x[,"Models"])) > 1){
      # sort predictors by importance
      sort_p <- with(x, reorder(predictor, contribution, median,
                                decreasing = TRUE))

      # Create a list of arguments to pass to boxplot
      boxplot_args <- list(formula = x$contribution ~ sort_p,
                           main = main, xlab= xlab, ylab = ylab,
                           ylim = c(0, max(x$contribution) + 0.2), ...)

      # plot a boxplot using do.call()
      do.call(boxplot, boxplot_args)


      if (extra_info) {
        #Calculate means and counts per group
        means  <- tapply(x$contribution, x$predictor, mean)[levels(sort_p)]
        counts <- tapply(x$contribution, x$predictor, length)[levels(sort_p)]

        # Common y coordinate for text labels in the upper part of the plot
        text_y <- max(x$contribution) + 0.15

        # Add means and counts to the plot (same level)
        for (i in 1:length(unique(x$predictor))) {
          text(i, text_y, paste("Mean =", round(means[i], 2)), pos = 3)
          text(i, text_y , paste("N =", counts[i]), pos = 1)
        }
      }
    } else {
      # Create a list of arguments to pass to barplot
      barplot_args <- list(height = x$contribution, names.arg = x$predictor,
                           main = main, xlab = xlab, ylab = ylab, ...)

      # plot a barplot using do.call()
      do.call(barplot, barplot_args)
    }

  } else {
    barplot_args <- list(height = x$contribution, names.arg = x$predictor,
                         main = main, xlab = xlab, ylab = ylab, ...)
    do.call(barplot, barplot_args)
  }
}

# ------------------------------------------------------------------------------
# Aux function to evaluate the Variable Contribution of the predictors
#
# get variable contribution for an individual model
var_importance_ind <- function(model, data = NULL, weights = NULL){

  # initial tests
  if (missing(model)) {
    stop("Argument 'model' must be defined.")
  }

  # deviance of the full model
  dev_full <- deviance(model)

  # preds names
  pnames <- names(coef(model))[-1]

  # deviance of the reduced models
  dev_reduction <- sapply(pnames, function(x) {
    dev_full - get_red_dev(full_model = model, reduce_var = x, data = data,
                           weights = weights)
  })

  deviance_importance <- dev_reduction / sum(dev_reduction)

  # preparing results
  tab_contr <- data.frame(predictor = names(deviance_importance),
                          stringsAsFactors = FALSE)
  tab_contr$contribution <- deviance_importance

  ord <- order(tab_contr$contribution, decreasing = TRUE)
  tab_contr <- tab_contr[ord,]
  tab_contr$cum_contribution <- cumsum(tab_contr$contribution)

  # returning results
  return(tab_contr)
}

#to get deviance of a model after excluding predictors
get_red_dev <- function(full_model, reduce_var, data, weights = NULL) {
  # initial tests
  if (missing(full_model)) {
    stop("Argument 'full_model' must be defined.")
  }
  if (missing(reduce_var)) {
    stop("Argument 'reduce_var' must be defined.")
  }
  if (missing(data)) {
    stop("Argument 'data' must be defined.")
  }

  # Ensure reduce_vars is a character vector
  reduce_var <- as.character(reduce_var)

  # Construct the reduced formula
  reduced_formula_str <- paste("~ . -", paste(reduce_var, collapse = " - "))
  reduced_formula <- as.formula(reduced_formula_str)

  # Attempt to update the model with the new formula and data
  reduce_model <- suppressWarnings(
    update(object = full_model, formula = reduced_formula,
           data = data, weights = weights)
  )

  return(deviance(reduce_model))
}

# Function to standardize interaction term names
standardize_interaction_names <- function(names) {
  sapply(names, function(name) {
    # Split the name by ':', sort, and re-join
    parts <- strsplit(name, ":")[[1]]
    if(length(parts) > 1) {
      return(paste(sort(parts), collapse = ":"))
    } else {
      return(name)
    }
  })
}

# ------------------------------------------------------------------------------
# Aux function to calculate the Response Curves
#


# aux function to get the individual response of a variable keeping constant
# the others predictors
response <- function(model, variable, data = NULL, n = 100, new_data = NULL,
                     extrapolate = FALSE) {

  # initial tests
  if (missing(model) | missing(variable)) {
    stop("Argument 'model' or 'variable' must be defined.")
  }

  if (class(model)[1] == "glm" && is.null(try(model$data)) && is.null(data)) {
    stop(paste0("The argument 'data' must be defined in case the model entered",
                " does not explicitly include a data component."))
  }

  if (class(model)[1] == "glm" && !is.null(try(model$data))) {
    data <- model$data
  }

  if (!is.null(new_data)) {
    if (!class(new_data)[1] %in% c("matrix", "data.frame", "SpatRaster")) {
      stop("'new_data' must be of class 'matrix', 'data.frame', 'SpatRaster'")
    }
  }

  # It gets only the variable names used in the fitted model
  vnames <- colSums(
    sapply(colnames(data), grepl, names(coef(model)[-1]))
  ) > 0

  if (any(!variable %in% names(vnames))) {
    stop("The name of the 'variable' was not defined correctly.")
  }

  # Extract calibration data from the model object
  cal_data <- data[, vnames]

  # Extract the limits of the calibration data
  cal_maxs <-  apply(cal_data, 2, FUN = max)
  cal_mins <-  apply(cal_data, 2, FUN = min)

  # Get the average of all variables
  means <- apply(cal_data, 2, FUN = mean)


  if (is.null(new_data)) {

    if (extrapolate){

      rr <- range(cal_data[, variable]) # range of the calibration data
      extension <- 0.11 * diff(rr)

      l_limit <- rr[1] - extension
      u_limit <- rr[2] + extension

      rangev <- c(l_limit, u_limit)

    } else {
      rangev <- range(cal_data[, variable])
    }

  } else {

    if (class(new_data)[1] == "SpatRaster") {
      rangev <- terra::minmax(new_data[[variable]])
    } else {
      rangev <- range(new_data[, variable])
    }

  }

  newvar <- seq(rangev[1], rangev[2], length = n)


  m <- data.frame(matrix(means, n, length(means), byrow = T))
  colnames(m) <- names(means)

  m[, variable] <- newvar

  # Response of the variable
  m$predicted <- stats::predict(model, m, type = "response")

  return(m[, c(variable, "predicted")])

}


# Response curve for a single model
response_curve_ind <- function(model, variable, data = NULL, n = 100,
                               new_data = NULL, extrapolate = FALSE,
                               xlab = NULL, ylab = NULL, col = NULL, ...) {

  # initial tests
  if (missing(model) | missing(variable)) {
    stop("Argument 'model' or 'variable' must be defined.")
  }

  if (class(model)[1] == "glm" && is.null(try(model$data)) && is.null(data)) {
    stop(paste0("The argument 'data' must be defined in case the model entered",
                " does not explicitly include a data component."))
  }

  if (class(model)[1] == "glm" && !is.null(try(model$data))) {
    data <- model$data
  }

  if (!is.null(new_data)) {
    if (!class(new_data)[1] %in% c("matrix", "data.frame", "SpatRaster")) {
      stop("'new_data' must be of class 'matrix', 'data.frame', 'SpatRaster'")
    }
  }


  response_out <- response(model = model, variable = variable, data = data,
                           n = n, extrapolate = extrapolate,
                           new_data = new_data)

  limits <- range(data[, variable])

  ## Plotting curve
  # Create a list of arguments to pass to the plot function
  if (is.null(xlab)) {xlab <- variable}

  plotcurve_args <- list(x = response_out[, variable],
                         y = response_out$predicted,
                         type = "l",
                         xlab= xlab,
                         ylab = ylab,
                         col = col,
                         ...)

  # plot using do.call()
  do.call(plot, plotcurve_args)

  abline(v = limits,
         col = c("black", "black"),
         lty = c(2, 2),
         lwd = c(1, 1)
  )
}

# Consensus response curve
response_curve_cons <- function(model, variable, data = NULL, n = 100,
                                new_data = NULL, extrapolate = FALSE,
                                show_lines = TRUE, xlab = NULL, ylab = NULL,
                                col = NULL, ...) {

  # initial tests
  if (missing(model) | missing(variable)) {
    stop("Argument 'model' or 'variable' must be defined.")
  }

  if (!is.null(new_data)) {
    if (!class(new_data)[1] %in% c("matrix", "data.frame", "SpatRaster")) {
      stop("'new_data' must be of class 'matrix', 'data.frame', 'SpatRaster'")
    }
  }


  # extract the response of the variable for each models
  response_out <- lapply(model, function(x){

    # It gets only the variable names used in the fitted model
    vnames <- colSums(sapply(colnames(data), grepl, names(coef(x)[-1]))) > 0

    if (any(!variable %in% names(vnames))) {
      stop("The name of the 'variable' was not defined correctly.")
    }

    coefs <- names(coef(x)[-1])
    c1 <- any(c(variable, paste0("I(", variable, "^2)")) %in% coefs)
    c2 <- any(grepl(paste0("^", variable, ":"), coefs))
    c3 <- any(grepl(paste0(":", variable, "$"), coefs))

    if (any(c1, c2, c3)){
      x <- response(x, variable, data = data, new_data = new_data,
                    extrapolate = extrapolate)
      return(x)

    } else {
      return(NULL)
    }
  })

  if (show_lines) {
    response_out0 <- response_out
  }

  # summary of responses
  response_out <- do.call(rbind, response_out)
  limits <- range(data[, variable])


  x <- response_out[, variable]
  y <- response_out$predicted

  ## Plotting curve
  # Create a list of arguments to pass to the plot function
  if (is.null(xlab)) {xlab <- variable}
  plotcurve_args <- list(x = x,
                         y = y,
                         type = "n",
                         xlab= xlab,
                         ylab = ylab,
                         ...)

  if (show_lines) {
    # plot using do.call()
    do.call(plot, plotcurve_args)

    # adding lines
    col <- grDevices::adjustcolor(col, alpha.f = 0.5)
    for (i in response_out0) {
      lines(i[, variable], i[, "predicted"], col = col)
    }

  } else {
    # Fit GAM model
    fitting <- mgcv::gam(y ~ s(x, bs = "cs"))

    # Generate predicted values and standard error.
    x_seq <- seq(min(x), max(x), length.out = 100)
    pred <- predict(fitting, newdata = data.frame(x = x_seq), se = T)

    # Extract predicted values, confidence intervals (95%), and standard errors
    y_pred <- pred$fit
    lower_ci <- y_pred - 1.96 * pred$se.fit
    upper_ci <- y_pred + 1.96 * pred$se.fit

    # plot using do.call()
    do.call(plot, plotcurve_args)

    # Create shading interval using polygon
    x_polygon <- c(x_seq, rev(x_seq))
    y_polygon <- c(lower_ci, rev(upper_ci))
    polygon(x_polygon, y_polygon, col = "lightgrey", border = NA)

    # Add the regression curve
    lines(x_seq, y_pred, col = col)
  }

  # It adds the calibration limits
  abline(v = limits, col = c("black", "black"), lty = c(2, 2), lwd = c(1, 1))
}

# ------------------------------------------------------------------------------
# Aux function to calculate consensus predictions (or "ensemble").
#

# Consensus forecasts are obtained by combining the forecasts from a collection
# (or “ensemble”) of model predictions

consensus_p <- function(predictions, weights = NULL){


  if (!is.null(weights)) {

    if (sum(weights) != 1){
      stop("'weights' must sum to 1 to calculate the weighted average.")

      } else {
        # Mean
        c_mean <- terra::app(predictions, mean)

        # Median
        c_media <- terra::app(predictions, median)

        # Variance
        c_var <- terra::app(predictions, var)

        # Weighted average
        c_wmean <- terra::app(predictions*weights, sum)

        cons <- c(c_mean, c_media, c_var, c_wmean)
        names(cons) <- c("Mean", "Median", "Variance", "Weighted_average")
    }
  } else {

    # Mean
    c_mean <- terra::app(predictions, mean)

    # Median
    c_media <- terra::app(predictions, median)

    # Variance between the consensus predictions
    c_var <- terra::app(c(c_mean, c_media), var)

    cons <- c(c_mean, c_media, c_var)
    names(cons) <- c("Mean", "Median", "Consensus_variance")
    }

  return(cons)

}

# ------------------------------------------------------------------------------
# Aux to check if an argument is a list of glm objects
check_if_glm_list <- function(arg) {
  if (methods::is(arg, "list") && all(sapply(arg, function(x) "glm" %in%  class(x)))) {
    return(TRUE)
  } else {
    return(FALSE)
  }
}

# ------------------------------------------------------------------------------
# Aux to save calibration result tables
save_cal <- function(x, out_dir) {

  if (missing(out_dir)) {
    stop("Argument 'out_dir' must be defined.")
  }

  if (!dir.exists(out_dir)) {
    dir.create(out_dir)
  }

  write.table(x$calibration_results, row.names = FALSE, sep = ",",
              file = paste0(out_dir, "/1_Full_report.csv"))

  write.table(x$summary, row.names = FALSE, sep = ",",
              file = paste0(out_dir, "/2_Summary.csv"))

  write.table(x$selected, row.names = FALSE, sep = ";",
              file = paste0(out_dir, "/3_Selected_models.csv"))

  write.table(x$data, row.names = FALSE, sep = ",",
              file = paste0(out_dir, "/4_Data_splitted.csv"))


}

#-------------------------------------------------------------------------------
# Aux functions to show executing time as hours, minutes and seconds.
format_time <- function(x){
  # formatting time
  hs <- as.numeric(x) %/% 3600
  ms <- (as.numeric(x) %% 3600) %/% 60
  ss<- as.numeric(x) %% 60

  # Time running message
  m <- sprintf("\nExecution time: %1.f hours, %1.f minutes and %.2f seconds",
               hs, ms, ss)
  message(m)
}

#' Plot Niche Signal results
#'
#' @description
#' Plots to interpret results from niche_signal tests
#' (Cobos & Peterson (2022) <doi:10.17161/bi.v17i.15985>).
#'
#' @usage
#' plot_niche_signal(niche_signal_list, statistic = "mean",
#'                   variables = NULL, ellipses = FALSE, level = 0.99,
#'                   breaks = "Sturges", main = "", xlab = NULL, ylab = NULL,
#'                   h_col = "lightgray", h_cex = 0.8, lty = 2, lwd = 1,
#'                   l_col = c("blue", "black"), e_col = c("black", "red"),
#'                   pch = 19, pt_cex = c(1.3, 0.8), pt_col = c("black", "red"),
#'                   ...)
#'
#' @param niche_signal_list list of results from niche_signal.
#' @param statistic (character) name of the statistic for which results will be
#' explored when results come for univariate analysis. Default = "mean". Options
#' are: "mean", "median", "SD", and "range".
#' @param variables (character) name of variables to used in plots when
#' results come from analysis using the `permanova` method. The default, NULL,
#' uses the first two variables.
#' @param ellipses (logical) whether to use ellipses to represent all and
#' positive data when results come from PERMANOVA. The default, FALSE, plots
#' points instead.
#' @param level (numeric) value from 0 to 1 representing the limit of
#' the ellipse to be plotted. Default = 0.99.
#' @param main (character) title for plot. Default = "".
#' @param xlab (character) x axis label. Default = NULL. For results from
#' PERMANOVA, appropriate variable names are used.
#' @param ylab (character) y axis label. Default = NULL. For univariate
#' results, the default turn into "Frequency". For results from PERMANOVA,
#' appropriate variable names are used.
#' @param breaks breaks in the histogram as in \code{\link[graphics]{hist}}.
#' Default = "Sturges".
#' @param h_col a color to be used to fill the bars of histograms.
#' Default = "lightgray".
#' @param h_cex (numeric) value by which plotting text and symbols should be
#' magnified relative to the default in histograms. Default = 0.8.
#' @param lty (numeric) line type. See options in \code{\link[graphics]{par}}.
#' Default = 2. A vector of length = 2 can be used, in order, all and positive.
#' @param lwd (numeric) line width. See options in \code{\link[graphics]{par}}.
#' Default = 1. A vector of length = 2 can be used, in order, all and positive.
#' @param l_col line color for observed value of positives and confidence
#' intervals. Default = c("blue", "black").
#' @param e_col color of ellipse lines for all and positive data.
#' Default = c("black", "red").
#' @param pch point type. See options in \code{\link[graphics]{points}}.
#' Default = 19. A vector of length = 2 can be used, in order, all and positive.
#' @param pt_cex (numeric) value by which points will be magnified. Values
#' for all and positive points are recommended. Default = c(1.3, 0.8).
#' @param pt_col color for points. Values for all and positive points are
#' recommended. Default = c("black", "red").
#' @param ... other plotting arguments to be used.
#'
#' @return
#' A plot.
#'
#' @export
#'
#' @importFrom ellipse ellipse
#' @importFrom stats cov anova ecdf quantile
#' @importFrom graphics lines box hist points
#'
#' @rdname plot_niche_signal
#'
#' @examples
#' # Load species occurrences and environmental data.
#' data("enm_data", package = "enmpa")
#' head(enm_data)
#'
#' # Detection of niche signal using an univariate non-parametric test
#' sn_bio1 <- niche_signal(data = enm_data,
#'                         variables = "bio_1",
#'                         condition = "Sp",
#'                         method = "univariate")
#'
#' plot_niche_signal(sn_bio1, variables = "bio_1")
#'
#' sn_bio12 <- niche_signal(data = enm_data,
#'                          variables = "bio_12",
#'                          condition = "Sp",
#'                          method = "univariate")
#'
#' plot_niche_signal(sn_bio12, variables = "bio_12")


plot_niche_signal <- function(niche_signal_list, statistic = "mean",
                              variables = NULL, ellipses = FALSE,
                              level = 0.99, breaks = "Sturges", main = "",
                              xlab = NULL, ylab = NULL, h_col = "lightgray",
                              h_cex = 0.8, lty = 2, lwd = 1,
                              l_col = c("blue", "black"),
                              e_col = c("black", "red"),
                              pch = 19, pt_cex = c(1.3, 0.8),
                              pt_col = c("black", "red"), ...) {

  # method using to produce niche_signal_list
  method <- niche_signal_list$summary$method

  if (!is.null(method)) {
    # plotting results from general analysis
    if (method == "univariate") {
      rlist <- list(
        data = niche_signal_list$data,
        summary = niche_signal_list$summary,
        analysis_results = niche_signal_list$univariate_results
      )

      plot_niche_signal_univariate(rlist, statistic, breaks, main, xlab,
                                   ylab, h_col, h_cex, lty, lwd, l_col, ...)

    } else {
      rlist <- list(
        modified_data = niche_signal_list$modified_data,
        summary = niche_signal_list$summary,
        permanova_results = niche_signal_list$permanova_results
      )

      plot_niche_signal_permanova(rlist, variables,
                                  ellipses, level, main, xlab, ylab, e_col,
                                  lty, lwd, pch, pt_cex, pt_col, ...)
    }
  } else {
    # plotting results from univariate analysis
    if (!is.null(niche_signal_list$analysis_results)) {
      plot_niche_signal_univariate(niche_signal_list, statistic,
                                   breaks, main, xlab, ylab, h_col,
                                   h_cex, lty, lwd, l_col, ...)
    } else {
      # plotting results from permanova results
      if (!is.null(niche_signal_list$permanova_results)) {
        plot_niche_signal_permanova(niche_signal_list, variables,
                                    ellipses, level, main, xlab, ylab, e_col,
                                    lty, lwd, pch, pt_cex, pt_col, ...)
      }
    }
  }
}



#' @rdname plot_niche_signal
#' @param niche_signal_univariate_list list of results from niche_signal_univariate.
#' @export
#' @usage
#' plot_niche_signal_univariate(niche_signal_univariate_list, statistic = "mean",
#'                              breaks = "Sturges", main = "", xlab = NULL,
#'                              ylab = "Frequency", h_col = "lightgray",
#'                              h_cex = 0.8, lty = 2, lwd = 1,
#'                              l_col = c("blue", "black"), ...)

plot_niche_signal_univariate <- function(niche_signal_univariate_list, statistic = "mean",
                                         breaks = "Sturges", main = "", xlab = NULL,
                                         ylab = "Frequency", h_col = "lightgray",
                                         h_cex = 0.8, lty = 2, lwd = 1,
                                         l_col = c("blue", "black"), ...) {
  # test
  if (missing(niche_signal_univariate_list)) {
    stop("Argument 'niche_signal_univariate_list' is missing")
  }

  # other graphical settings
  if (length(l_col) == 1) {
    l_col <- rep(l_col, 2)
  } else {
    l_col <- l_col[1:2]
  }

  if (length(lty) == 1) {
    lty <- rep(lty, 2)
  } else {
    lty <- lty[1:2]
  }

  if (length(lwd) == 1) {
    lwd <- rep(lwd, 2)
  } else {
    lwd <- lwd[1:2]
  }

  # preparing data
  if (statistic == "mean") {
    all <- niche_signal_univariate_list$analysis_results$null_results$mean
    pos <- niche_signal_univariate_list$analysis_results$positive_niche["mean"]
    ci <- quantile(niche_signal_univariate_list$analysis_results$null_results$mean,
                   c(0.025, 0.975))
  }

  if (statistic == "median") {
    all <- niche_signal_univariate_list$analysis_results$null_results$median
    pos <- niche_signal_univariate_list$analysis_results$positive_niche["median"]
    ci <- quantile(niche_signal_univariate_list$analysis_results$null_results$median,
                   c(0.025, 0.975))
  }

  if (statistic == "SD") {
    all <- niche_signal_univariate_list$analysis_results$null_results$SD
    pos <- niche_signal_univariate_list$analysis_results$positive_niche["SD"]
    ci <- quantile(niche_signal_univariate_list$analysis_results$null_results$SD,
                   c(0.025, 0.975))
  }

  if (statistic == "range") {
    all <- niche_signal_univariate_list$analysis_results$null_results$range
    pos <- niche_signal_univariate_list$analysis_results$positive_niche["range"]
    ci <- quantile(niche_signal_univariate_list$analysis_results$null_results$range,
                   c(0.025, 0.975))
  }

  var <- niche_signal_univariate_list$summary$variable

  # other things for plots
  ylab <- ifelse(is.null(ylab), "Frequency", ylab)
  xlab <- ifelse(is.null(xlab), paste0(var, " (", statistic, ")"), xlab)
  xrang <- range(c(all, pos))

  # plot
  hist(all, breaks = breaks, xlim = xrang, main = main, xlab = xlab,
       ylab = ylab, col = h_col, cex = h_cex, ...)
  abline(v = pos, col = l_col[1], lty = lty[1], lwd = lwd[1])
  abline(v = ci, col = l_col[2], lty = lty[2], lwd = lwd[2])
  box(bty = "l")
}



#' @rdname plot_niche_signal
#' @param niche_signal_permanova_list list of results from niche_signal_permanova.
#' @export
#' @usage
#' plot_niche_signal_permanova(niche_signal_permanova_list, variables = NULL,
#'                            ellipses = FALSE, level = 0.99, main = "",
#'                            xlab = NULL, ylab = NULL,
#'                            e_col = c("black", "red"), lty = 2, lwd = 1,
#'                            pch = 19, pt_cex = c(1.3, 0.8),
#'                            pt_col = c("black", "red"), ...)

plot_niche_signal_permanova <- function(niche_signal_permanova_list, variables = NULL,
                                        ellipses = FALSE, level = 0.99,
                                        main = "", xlab = NULL, ylab = NULL,
                                        e_col = c("black", "red"), lty = 2, lwd = 1,
                                        pch = 19, pt_cex = c(1.3, 0.8),
                                        pt_col = c("black", "red"), ...) {

  # test
  if (missing(niche_signal_permanova_list)) {
    stop("Argument 'niche_signal_permanova_list' is missing")
  }

  # selecting variables to use
  if (is.null(variables)) {
    variables <- niche_signal_permanova_list$summary$variables[1:2]
  } else {
    variables <- variables[1:2]
  }

  # relevant data to plot
  condition <- niche_signal_permanova_list$summary$condition
  condition <- niche_signal_permanova_list$modified_data[, condition]

  all <- condition == 0
  posi <- condition == 1

  # type of points
  if (ellipses == FALSE) {
    if (length(pch) == 1) {
      pch <- c(pch, pch)
    } else {
      pch <- pch[1:2]
    }

    if (length(pt_col) == 1) {
      pt_col <- c(pt_col, pt_col)
    } else {
      pt_col <- pt_col[1:2]
    }

    # labels
    xlab <- ifelse(is.null(xlab), variables[1], xlab)
    ylab <- ifelse(is.null(ylab), variables[2], ylab)

    # plot
    allp <- niche_signal_permanova_list$modified_data[all, variables]
    plot(allp, main = main, xlab = xlab, ylab = ylab, pch = pch[1],
         cex = pt_cex[1], col = pt_col[1], ...)

    allp <- niche_signal_permanova_list$modified_data[posi, variables]
    points(allp,
           pch = pch[2], cex = pt_cex[2], col = pt_col[2])

  } else {
    # type of lines
    if (length(e_col) == 1) {
      e_col <- rep(e_col, 2)
    } else {
      e_col <- e_col[1:2]
    }

    if (length(lty) == 1) {
      lty <- rep(lty, 2)
    } else {
      lty <- lty[1:2]
    }

    if (length(lwd) == 1) {
      lwd <- rep(lwd, 2)
    } else {
      lwd <- lwd[1:2]
    }

    # all ellipse
    hell <- ellipse::ellipse(
      x = cov(niche_signal_permanova_list$modified_data[all, variables]),
      centre = colMeans(niche_signal_permanova_list$modified_data[all, variables]),
      level = level
    )

    # positive ellipse
    pell <- ellipse::ellipse(
      x = cov(niche_signal_permanova_list$modified_data[posi, variables]),
      centre = colMeans(niche_signal_permanova_list$modified_data[posi, variables]),
      level = level
    )

    # plot limits
    rans <- apply(rbind(hell, pell), 2, range)

    # plot
    plot(rans, type = "n", main = main, xlab = xlab, ylab = ylab, ...)
    lines(hell, col = e_col[1], lwd = lwd[1], lty = lty[1])
    lines(pell, col = e_col[2], lwd = lwd[2], lty = lty[2])
  }
}

#' Example of results obtained from GLM calibration using enmpa
#'
#' An object of the class enmpa_calibration storing the results from GLM
#' calibration.
#'
#' @format An object of class enmpa_calibration with results from the function
#' `\link{calibration_glm}`.
#'
#' @examples
#' data("cal_res", package = "enmpa")
#'
#' str(cal_res)
"cal_res"



#' Example data used to run model calibration exercises
#'
#' @description A dataset containing information on presence and absence, and
#' independent variables used to fit GLM models.
#'
#' @format A data frame with 5627 rows and 3 columns.
#' \describe{
#'   \item{Sp}{numeric, values of 0 = absence and 1 = presence.}
#'   \item{bio_1}{numeric, temperature values.}
#'   \item{bio_12}{numeric, precipitation values.}
#' }
#'
#' @examples
#' data("enm_data", package = "enmpa")
#' head(enm_data)
"enm_data"



#' Example data used to test models
#'
#' @description A dataset containing information on presence and absence, and
#' independent variables used to fit GLM models.
#'
#' @format A data frame with 100 rows and 3 columns.
#' \describe{
#'   \item{Sp}{numeric, values of 0 = absence and 1 = presence.}
#'   \item{lon}{numeric, longitude values.}
#'   \item{lat}{numeric, latitude values.}
#' }
#'
#' @examples
#' data("test", package = "enmpa")
#' head(test)
"test"



#' Example of selected models fitted
#'
#' @description An object of the class enmpa_fitted_models containing
#' fitted selected model(s) and the information from model evaluation for such
#' model(s).
#'
#' @format A list with four elements.
#' \describe{
#'   \item{glms_fitted}{a fitted glm (ModelID_7).}
#'   \item{selected}{a data.frame with results from evaluation of ModelID_7}
#'   \item{data}{a data.frame containing information on presence and absence, and
#'   independent variables used to fit GLM models.}
#'   \item{weights}{a vector with the weights for observations.}
#' }
#'
#' @examples
#' data("sel_fit", package = "enmpa")
"sel_fit"

# detection of response curves that are concave (non-unimodal)
detect_concave <- function(glm_coefficents) {
  if (missing(glm_coefficents)) {
    stop("Argumet 'glm_coefficents' must be defined.")
  }
  coef <- glm_coefficents
  varname <- names(coef)

  ccoef <- grep("\\^2", varname, value = TRUE)

  if (length(ccoef) == 0) {
    return("")
  } else {
    cccoef <- ccoef[coef[ccoef] >= 0]

    return(paste(cccoef, collapse = ", "))
  }
}



# contribution_formulas <- function(data, dependent, independent, weights = NULL,
#                                   type = "lqp", range = c(0.0001, 0.2),
#                                   by = 0.0001) {
#
#   f <- get_formulas(dependent = dependent, independent = independent,
#                     type = type, all_combinations = FALSE)
#
#   allfit <- glm(as.formula(f), data = data, family = binomial(link = "logit"),
#                 weights = weights)
#
#   av <- anova(allfit, test = "Chisq")
#   preds <- rownames(av)
#   devs <- av$Deviance[-1]
#
#   sequ <- seq(range[1], range[2], by = by)
#
#   norm_dev <- seq(0, max(devs), length.out = 100)
#
#   forms <- lapply(sequ, function(x) {
#     thres <- quantile(norm_dev, x)
#
#     pick <- devs > thres
#
#     pred <- preds[-1][pick]
#
#     paste(dependent, "~", paste(pred, collapse = " + "))
#   })
#
#   forms <- unique(unlist(forms))
#   forms <- forms[length(forms):1]
#
#   return(forms)
# }

#' Find threshold values to produce three optimal metrics
#'
#' @description
#' The metrics true skill statistic (TSS), sensitivity, specificity are explored
#' by comparing actual vs predicted values to find threshold values that produce
#' sensitivity = specificity, maximum TSS, and a sensitivity value of 0.9.
#'
#' @usage
#' optimize_metrics(actual, predicted, n_threshold = 100)
#'
#' @param actual (numeric) vector of actual values (0, 1) to be compared to
#' `predicted` values after applying a threshold.
#' @param predicted (numeric) vector of predicted probability values to be
#' thresholded and compared to `actual`.
#' @param n_threshold (numeric) number of threshold values to be used.
#' Default = 100.
#'
#' @return
#' A list containing a data.frame with the resulting metrics for all threshold
#' values tested, and a second data.frame with the results for the threshold
#' values that produce sensitivity = specificity (ESS), maximum TSS (maxTSS),
#' and a sensitivity value of 0.9 (SEN90).
#'
#' @export
#'
#' @examples
#' # example data
#' act <- c(rep(1, 20), rep(0, 80))
#' pred <- c(runif(20, min = 0.4, max = 0.7), runif(80, min = 0, max = 0.5))
#'
#' # run example
#' om <- optimize_metrics(actual = act, predicted = pred)
#' om$optimized

optimize_metrics <- function(actual, predicted, n_threshold = 100) {

  if (missing(actual) | missing(predicted)) {
    stop("Arguments 'actual' and 'predicted' must be defined.")
  }

  # Calculate TPR and FPR for different threshold values
  thresholds <- seq(min(predicted), max(predicted), length.out = n_threshold)

  tpr <- numeric(length(thresholds))
  fpr <- numeric(length(thresholds))
  spe <- numeric(length(thresholds))
  tss <- numeric(length(thresholds))
  acc <- numeric(length(thresholds))

  for (i in 1:length(thresholds)) {
    tp <- sum(actual & (predicted >= thresholds[i]))
    fn <- sum(actual & (predicted < thresholds[i]))
    tn <- sum(!actual & (predicted < thresholds[i]))
    fp <- sum(!actual & (predicted >= thresholds[i]))

    tpr[i] <- tp / (tp + fn)
    fpr[i] <- fp / (fp + tn)
    spe[i] <- tn / (tn + fp)
    acc[i] <- (tn + tp) / (tp + tn + fp + fn)
    tss[i] <- spe[i] + tpr[i] - 1
  }

  # final stats
  roc_auc <- abs(sum(diff(fpr) * (tpr[-1] + tpr[-length(tpr)]) / 2))

  metrics <- data.frame(Threshold = thresholds, ROC_AUC = roc_auc,
                        False_positive_rate = fpr, Accuracy = acc,
                        Sensitivity = tpr, Specificity = spe, TSS = tss)

  t09 <- tpr == 0.9
  if (sum(t09) == 0) {
    t09 <- tpr == min(tpr[tpr >= 0.9])
  }

  optimized <- rbind(metrics[which.min(abs(tpr - spe)), ][1, ],
                     metrics[which.max(tss), ][1, ], metrics[t09, ][1, ])

  criteria <- c("ESS", "maxTSS", "SEN90")

  optimized <- data.frame(Threshold_criteria = criteria, optimized)

  return(list(optimized = optimized, metrics = metrics))
}

#' Constructor of S3 objects of class enmpa_calibration
#'
#' @name enmpa_calibration
#' @aliases enmpa_calibration new_enmpa_calibration
#'
#' @param selected date.frame with information about selected models.
#' @param summary data.frame a summary of statistics for all models.
#' @param calibration_results data.frame with results obtained from
#' cross-validation for all models.
#' @param data data.frame or matrix with the input data used for calibration.
#' @param partitioned_data a list of partition indices.
#' @param weights (numeric) a vector with the weights for observations.
#' Default = NULL.
#'
#' @return
#' An S3 object of class \code{enmpa_calibration}.
#'
#' @export
#'
#' @usage
#' new_enmpa_calibration(selected, summary, calibration_results, data,
#'                       partitioned_data, weights = NULL)

new_enmpa_calibration <- function(selected, summary, calibration_results, data,
                              partitioned_data, weights = NULL) {
  structure(
    list(
      selected = selected,
      summary = summary,
      calibration_results = calibration_results,
      data = data,
      partitioned_data = partitioned_data,
      weights = weights
    ),
    class = "enmpa_calibration"
  )
}

#' Constructor of S3 objects of class enmpa_fitted_models
#'
#' @name enmpa_fitted_models
#' @aliases enmpa_fitted_models new_enmpa_fitted_models
#' @param glms_fitted a list of fitted GLMs.
#' @param selected date.frame with information about selected models.
#' @param data data.frame or matrix with the input data used for calibration.
#' @param weights (numeric) a vector with the weights for observations.
#' Default = NULL.
#'
#' @return
#' An S3 object of class \code{enmpa_fitted_models}.
#'
#' @export
#'
#' @usage
#' new_enmpa_fitted_models(glms_fitted, selected, data, weights = NULL)
#'

new_enmpa_fitted_models <- function(glms_fitted, selected, data, weights = NULL) {
  structure(
    list(
      glms_fitted = glms_fitted,
      selected = selected,
      data = data,
      weights = weights
    ),
    class = "enmpa_fitted_models"
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_importance.R
\name{var_importance}
\alias{var_importance}
\title{Variable importance for GLMs}
\usage{
var_importance(fitted, modelID = NULL, data = NULL)
}
\arguments{
\item{fitted}{an object of class \code{glm}, a list of GLMs obtained using the
function \code{\link{fit_glms}}, or an object \code{enmpa_fitted_models} from
\code{\link{fit_selected}} .}

\item{modelID}{(character) vector of ModelID(s) to be considered when the
\code{fitted} input is of the class \code{enmpa_fitted_models}. By default all models
are included. Default = NULL.}

\item{data}{data.frame or matrix of data used in the model calibration step.
It must be defined in case the model entered does not explicitly include a
data component. Default = NULL.}
}
\value{
A data.frame containing the relative contribution of each variable. An
identification for distinct models is added if \code{fitted} contains multiple
models.
}
\description{
Calculates the relative importance of predictor variables based on the
concept of explained deviance. This is achieved by fitting a GLMs multiple
times, each time leaving out a different predictor variable to observe its
impact on the model's performance.
}
\details{
The process begins by fitting the full GLM model, which includes all predictor
variables. Subsequently, separate GLM models are fitted, excluding one
variable at a time to assess the influence of its absence on the model's
performance. By systematically evaluating the effect of removing each
predictor variable, the function provides valuable insights into their
individual contributions to the model's overall performance and explanatory
power.
}
\examples{
# Load a fitted selected model
data(sel_fit, package = "enmpa")

# Variable importance for single models
var_importance(sel_fit, modelID = "ModelID_7")

# Variable importance for multiple models (only one model in this list)
var_importance(sel_fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_importance.R
\name{plot_importance}
\alias{plot_importance}
\title{Plot variable importance}
\usage{
plot_importance(x, xlab = NULL, ylab = "Relative contribution",
                main = "Variable importance", extra_info = TRUE, ...)
}
\arguments{
\item{x}{data.frame output from \code{\link{var_importance}}.}

\item{xlab}{(character) a label for the x axis.}

\item{ylab}{(character) a label for the y axis.}

\item{main}{(character) main title for the plot.}

\item{extra_info}{(logical) when results are from more than one model, it
adds information about the number of models using each predictor and the mean
contribution found.}

\item{...}{additional arguments passed to \code{\link[graphics]{barplot}} or
\code{\link[graphics]{boxplot}}.}
}
\value{
A plot
}
\description{
Visualization of the results obtained with the function
\code{\link{var_importance}}.
}
\examples{

# Load species occurrences and environmental data.
data("enm_data", package = "enmpa")

# Custom formulas
forms <- c("Sp ~ bio_1 + I(bio_1^2) + I(bio_12^2)",
           "Sp ~ bio_12 + I(bio_1^2) + I(bio_12^2)")

# Fitting models
fits <- fit_glms(forms, data = enm_data)

# Variable importance for single models
vi_1 <- var_importance(fits$ModelID_1)
plot_importance(x = vi_1)

vi_2 <- var_importance(fits$ModelID_2)
plot_importance(x = vi_2)

# Variable importance for multiple models
vi_c <- var_importance(fits)
plot_importance(x = vi_c)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evaluation_stats.R
\name{evaluation_stats}
\alias{evaluation_stats}
\title{Summary of evaluation statistics for candidate models}
\usage{
evaluation_stats(evaluation_results, bimodal_toexclude = FALSE)
}
\arguments{
\item{evaluation_results}{data.frame model evaluation results. These results
are the output of the function \code{\link{model_validation}}.}

\item{bimodal_toexclude}{(logical) whether models in which binomial variable
response curves were detected will be excluded during selection processes.}
}
\value{
A data.frame with the mean and standard deviation for all metrics considering
cross-validation kfolds.
}
\description{
Calculate median and standard deviation of evaluation results for all
candidate models considering cross-validation kfolds.
}
\examples{
# data
data("cal_res", package = "enmpa")
all_res <- cal_res$calibration_results[, -1]

# statistics for all evaluation results
evaluation_stats(all_res, bimodal_toexclude = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_selection.R
\name{model_selection}
\alias{model_selection}
\title{Selection of best candidate models considering various criteria}
\usage{
model_selection(evaluation_stats, criterion = "TSS", exclude_bimodal = FALSE,
                tolerance = 0.01)
}
\arguments{
\item{evaluation_stats}{data.frame with the statistics of model evaluation
results. These results are the output of the function
\code{\link{evaluation_stats}}.}

\item{criterion}{(character) metric used as the predictive criterion for
model selection.}

\item{exclude_bimodal}{(logical) whether to exclude models in which binomial
variable response curves were detected.}

\item{tolerance}{(numeric)}
}
\value{
A data.frame with one or more selected models.
}
\description{
Applies a series of criteria to select best candidate models.
}
\examples{
# data
data("cal_res", package = "enmpa")
eval_stats <- cal_res$summary[, -1]

# selecting best model
selected_mod <- model_selection(eval_stats, exclude_bimodal = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kfold_partition.R
\name{kfold_partition}
\alias{kfold_partition}
\title{K-fold data partitioning}
\usage{
kfold_partition(data, dependent, k = 2, seed = 1)
}
\arguments{
\item{data}{data.frame or matrix containing at least two columns.}

\item{dependent}{(character) name of column that contains the
presence-absence records (1-0).}

\item{k}{(numeric) the number of groups that the given data is to be split
into.}

\item{seed}{(numeric) integer value to specify an initial seed. Default = 1.}
}
\value{
A list of vectors with the indices of rows corresponding to each fold.
}
\description{
Creates indices to partition available data into k equal-sized
subsets or folds, maintaining the global proportion of presence-absences
in each fold.
}
\examples{
# example data
data <- data.frame(species = c(rep(0, 80), rep (1,20)),
                   variable1 = rnorm(100),
                   variable2 = rpois(100, 2))

# create partition indices
kfolds <- kfold_partition(data, dependent = "species", k = 2)

# data for partition 1
data[kfolds$Fold_1, ]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_glm.R
\name{fit_selected}
\alias{fit_selected}
\alias{fit_glms}
\title{Fitting selected GLMs models}
\usage{
fit_selected(glm_calibration)

fit_glms(formulas, data, weights = NULL, id = NULL)
}
\arguments{
\item{glm_calibration}{a list resulting from \code{\link{calibration_glm}}.
Models fitted are those in the slot "selected".}

\item{formulas}{(character) a vector containing the formula(s) for GLM(s).}

\item{data}{data.frame with the dependent and independent variables.}

\item{weights}{(numeric) a vector with the weights for observations.
Default = NULL.}

\item{id}{(character) id code for models fitted. Default = NULL.}
}
\value{
A list of fitted GLMs.

For \code{fit_selected}, an enmpa \verb{fitted models} object.
}
\description{
Functions to facilitate fitting multiple GLMs.
}
\examples{
# GLM calibration results
data(cal_res, package = "enmpa")

# Fitting selected models
sel_fit <- fit_selected(cal_res)

sel_fit

# Custom formulas
forms <- c("Sp ~ bio_1 + I(bio_1^2) + I(bio_12^2)",
           "Sp ~ bio_12 + I(bio_1^2) + I(bio_12^2)")

# Fitting models
fits <- fit_glms(forms, data = cal_res$data)

fits$ModelID_1
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proc.R
\name{proc_enm}
\alias{proc_enm}
\title{Partial ROC calculation}
\usage{
proc_enm(test_prediction, prediction, threshold = 5, sample_percentage = 50,
         iterations = 500)
}
\arguments{
\item{test_prediction}{(numeric) vector of model predictions for testing
data.}

\item{prediction}{\code{SpatRaster} or numeric vector of model predictions to be
evaluated.}

\item{threshold}{(numeric) value from 0 to 100 to represent the percentage of
potential error (E) that the data could have due to any source of uncertainty.
Default = 5.}

\item{sample_percentage}{(numeric) percentage of testing data to be used in each
bootstrapped process for calculating the partial ROC. Default = 50.}

\item{iterations}{(numeric) number of bootstrap iterations to be performed;
default = 500.}
}
\value{
A list with the summary of the results and a data.frame containing
the AUC values and AUC ratios calculated for all iterations.
}
\description{
proc applies partial ROC tests to model predictions.
}
\details{
Partial ROC is calculated following Peterson et al. (2008)
\url{doi:10.1016/j.ecolmodel.2007.11.008}.
}
\examples{
# Loading a model prediction
pred <- terra::rast(system.file("extdata", "proj_out_wmean.tif",
                                package = "enmpa"))

# Simulated data
test <- runif(100, min = 0.3, max = 0.8)

# partial ROC calculation
pr <- proc_enm(test, pred, threshold = 5, sample_percentage = 50,
               iterations = 500)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimize_metrics.R
\name{optimize_metrics}
\alias{optimize_metrics}
\title{Find threshold values to produce three optimal metrics}
\usage{
optimize_metrics(actual, predicted, n_threshold = 100)
}
\arguments{
\item{actual}{(numeric) vector of actual values (0, 1) to be compared to
\code{predicted} values after applying a threshold.}

\item{predicted}{(numeric) vector of predicted probability values to be
thresholded and compared to \code{actual}.}

\item{n_threshold}{(numeric) number of threshold values to be used.
Default = 100.}
}
\value{
A list containing a data.frame with the resulting metrics for all threshold
values tested, and a second data.frame with the results for the threshold
values that produce sensitivity = specificity (ESS), maximum TSS (maxTSS),
and a sensitivity value of 0.9 (SEN90).
}
\description{
The metrics true skill statistic (TSS), sensitivity, specificity are explored
by comparing actual vs predicted values to find threshold values that produce
sensitivity = specificity, maximum TSS, and a sensitivity value of 0.9.
}
\examples{
# example data
act <- c(rep(1, 20), rep(0, 80))
pred <- c(runif(20, min = 0.4, max = 0.7), runif(80, min = 0, max = 0.5))

# run example
om <- optimize_metrics(actual = act, predicted = pred)
om$optimized
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/response_curve.R
\name{response_curve}
\alias{response_curve}
\title{Variable response curves for GLMs}
\usage{
response_curve(fitted, variable, data = NULL, modelID = NULL,  n = 100,
               new_data = NULL,  extrapolate = TRUE, show_lines = TRUE,
               xlab = NULL, ylab = "Probability", col = "red", ...)
}
\arguments{
\item{fitted}{an object of class \code{glm}, a list of GLMs obtained using the
function \code{\link{fit_glms}},  or an object \code{enmpa_fitted_models} from
\code{\link{fit_selected}} .}

\item{variable}{(character) name of the variables to be plotted.}

\item{data}{data.frame or matrix of data used in the model calibration step.
Default = NULL.}

\item{modelID}{(character) vector of ModelID(s) to be considered when the
fitted models is an \code{enmpa_fitted_object}. By default all models are included.
Default = NULL.}

\item{n}{(numeric) an integer guiding the number of breaks. Default = 100}

\item{new_data}{a \code{SpatRaster}, data.frame, or  matrix of variables
representing the range of variable values in an area of interest.
Default = NULL. It must be defined in case the model entered does not
explicitly include a data component.}

\item{extrapolate}{(logical) whether to allow extrapolation to study the
behavior of the response outside the calibration limits. Ignored if
\code{new_data} is defined. Default = TRUE.}

\item{show_lines}{(logical) whether to show variable responses of distinct
models as different lines. Default = TRUE. If}

\item{xlab}{(character) a label for the x axis. The default, NULL, uses the
name defined in \code{variable}.}

\item{ylab}{(character) a label for the y axis. Default = "Probability".}

\item{col}{(character) color for lines. Default = "red".}

\item{...}{additional arguments passed to \code{\link[graphics]{plot}}.}
}
\value{
A plot with the response curve for a \code{variable}.
}
\description{
A view of variable responses in models. Responses based on single or multiple
models can be provided.
}
\details{
The function calculates these probabilities by focusing on a single
environmental variable while keeping all other variables constant at their
mean values.

When responses for multiple models are to be plotted, and \code{show_lines} =
FALSE, the mean and confidence intervals for the set of responses are
calculated using a GAM.
}
\examples{
# Load a fitted selected model
data(sel_fit, package = "enmpa")

# Response curve for single models
response_curve(sel_fit, modelID = "ModelID_7", variable = "bio_1")

# Response curve when model(s) are in a list (only one model in this one)
response_curve(sel_fit, variable = "bio_12")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_formulas.R
\name{get_formulas}
\alias{get_formulas}
\alias{get_formulas_main}
\alias{aux_var_comb}
\alias{aux_string_comb}
\title{Get GLM formulas according to defined response types}
\usage{
get_formulas(dependent, independent, type = "l", mode = "moderate",
             minvar = 1, maxvar = NULL)

get_formulas_main(dependent, independent, type = "l",
                  complex = FALSE, minvar = 1, maxvar = NULL)

aux_var_comb(var_names, minvar = 2, maxvar = NULL)

aux_string_comb(string)
}
\arguments{
\item{dependent}{(character) name of dependent variable.}

\item{independent}{(character) vector of name(s) of independent variable(s).}

\item{type}{(character) a character string that must contain "l",
"p", "q" or a combination of them. l = lineal, q = quadratic,
p = interaction between two variables. Default = "l".}

\item{mode}{(character) (character) a character string to indicate the strategy to
create the formulas for candidate models. Options are: "light", "moderate",
"intensive", or "complex". Default = "moderate".}

\item{minvar}{(numeric) minimum number of independent variables in formulas.}

\item{maxvar}{(numeric) maximum number of independent variables in formulas.}

\item{complex}{(logical) whether to return the most complex formula.}

\item{var_names}{sames as \code{independent}.}

\item{string}{same as \code{type}.}
}
\value{
A character vector containing the resulting formula(s).
}
\description{
Generate GLM formulas for independent variables predicting a dependent
variable, taking into account response types required. All possible
combinations of variables can be created using arguments of the function.
}
\details{
\code{mode} options determine what strategy to iterate the predictors
defined in \code{type} for creating models:
\itemize{
\item \strong{light}.-- returns simple iterations of complex formulas.
\item \strong{moderate}.-- returns a comprehensive number of iterations.
\item \strong{intensive}.-- returns all possible combination. Very time-consuming for
6 or more independent variables.
\item \strong{complex}.-- returns only the most complex formula.
}
}
\examples{
# example variables
dep <- "sp"
ind <- c("temp", "rain", "slope")

# The most complex formula according to "type"
get_formulas(dep, ind, type = "lqp", mode = "complex")

# mode = 'light', combinations according to type
get_formulas(dep, ind, type = "lqp", mode = "light")

# mode = 'light', combinations according to type
get_formulas(dep, ind, type = "lqp", mode = "intensive")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resp2var.R
\name{resp2var}
\alias{resp2var}
\title{Two-Way interaction response plot}
\usage{
resp2var(model, variable1 , variable2, modelID = NULL, data = NULL, n = 1000,
         new_data = NULL, extrapolate = FALSE, add_bar = TRUE,
         add_limits = FALSE, color.palette	= NULL, xlab = NULL, ylab = NULL,
         ...)
}
\arguments{
\item{model}{an object of class \code{glm} or \code{enmpa_fitted_models}.}

\item{variable1}{(character) name of the variable to be plotted in x axis.}

\item{variable2}{(character) name of the variable to be plotted in y axis.}

\item{modelID}{(character) name of the ModelID if inputed \code{model} is  in the
\code{enmpa_fitted_models} object.
Default = NULL.}

\item{data}{data.frame or matrix of data to be used in model calibration.
Default = NULL.}

\item{n}{(numeric) an integer guiding the number of breaks. Default = 100}

\item{new_data}{a \code{SpatRaster}, data.frame, or  matrix of variables
representing the range of variable values in an area of interest.
Default = NULL.}

\item{extrapolate}{(logical) whether to allow extrapolation to study the
behavior of the response outside the calibration limits. Ignored if
\code{new_data} is defined. Default = TRUE.}

\item{add_bar}{(logical) whether to add bar legend. Default = TRUE.}

\item{add_limits}{(logical) whether to add calibration limits if
\code{extrapolate = TRUE}. Default = FALSE.}

\item{color.palette}{(function) a color palette function to be used to assign
colors in the plot. Default = function(n) rev(hcl.colors(n, "terrain")).}

\item{xlab}{(character) a label for the x axis. The default, NULL, uses the
name defined in \code{variable1}.}

\item{ylab}{(character) a label for the y axis. The default, NULL, uses the
name defined in \code{variable2}.}

\item{...}{additional arguments passed to
\code{\link[graphics]{image}}.}
}
\value{
A plot with the response interaction of two environmental dimensions for
\code{variable1} and \code{variable2}, and don't return anything.
}
\description{
A view of the species probability into a two-dimensional
environmental space.
}
\details{
The function calculates probabilities by focusing on each combination of the
two supplied environmental variable while keeping all other variables constant
at their mean values.
}
\examples{
# Load a fitted selected model
data(sel_fit, package = "enmpa")

# Two-Way interaction response plot in the calibration limits

resp2var(sel_fit, variable1 = "bio_1", variable2 = "bio_12", xlab = "BIO-1",
ylab = "BIO-12", modelID = "ModelID_7")

# Two-Way interaction response plot allowing extrapolation
resp2var(sel_fit, variable1 = "bio_1", variable2 = "bio_12", xlab = "BIO-1",
ylab = "BIO-12", modelID = "ModelID_7", extrapolate = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Methods.R
\name{summary}
\alias{summary}
\alias{summary.enmpa_calibration}
\alias{summary,enmpa_calibration-method}
\alias{summary,enmpa_fitted_models-method}
\alias{summary.enmpa_fitted_models}
\title{Summary of 'calibration' and 'fitted models'}
\usage{
\method{summary}{enmpa_calibration}(object, ...)

\method{summary}{enmpa_fitted_models}(object, ...)
}
\arguments{
\item{object}{of class enmpa_calibration or enmpa_fitted_models}

\item{...}{additional arguments affecting the summary produced. Ignored in
these functions.}
}
\value{
A printed summary.
}
\description{
Summary of 'calibration' and 'fitted models'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/independent_eval.R
\name{independent_eval1}
\alias{independent_eval1}
\alias{independent_eval01}
\title{Evaluate final models using independent data}
\usage{
independent_eval1(prediction, threshold, test_prediction = NULL,
                  lon_lat = NULL)

independent_eval01(prediction, observation, lon_lat = NULL)
}
\arguments{
\item{prediction}{(numeric) vector or \code{SpatRaster} object. If numeric,
predicted values in independent data (for \code{\link{independent_eval01}}),
or the entire area of prediction (for \code{\link{independent_eval1}}).
If \code{SpatRaster} prediction over the area of interest.}

\item{threshold}{(numeric) the lowest predicted probability value for an
occurrence point. This value must be defined for presences-only data.
Default = NULL.}

\item{test_prediction}{(numeric) vector of predictions for independent data.
Default = NULL.}

\item{lon_lat}{matrix or data.frame of coordinates (longitude and latitude,
in that order) of independent data. Points must be located within the valid
area of \code{prediction}. For \code{\link{independent_eval01}} they
must correspond with values in observation.}

\item{observation}{(numeric) vector of observed (known) values of presence
or absence to test against \code{prediction} (if numeric) or values of prediction
in \code{lon_lat}.}
}
\value{
A data.frame or list containing evaluation results.
}
\description{
Final evaluation steps for model predictions using an independent dataset
(not used in model calibration).
}
\examples{
# Independent test data based on coordinates (lon/lat WGS 84) from presence
# and absences records
data("test", package = "enmpa")
head(test)

# Loading a model prediction
pred <- terra::rast(system.file("extdata", "proj_out_wmean.tif",
                                package = "enmpa"))
terra::plot(pred)

# Evaluation using presence-absence data
independent_eval01(prediction = pred, observation = test$Sp,
                   lon_lat = test[, 2:3])

# Evaluation using presence-only data
test_p_only <- test[test$Sp == 1, ]
th_maxTSS   <- 0.1274123            # threshold based on the maxTSS

independent_eval1(prediction = pred, threshold = th_maxTSS,
                  lon_lat = test_p_only[, 2:3])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.R
\docType{data}
\name{test}
\alias{test}
\title{Example data used to test models}
\format{
A data frame with 100 rows and 3 columns.
\describe{
\item{Sp}{numeric, values of 0 = absence and 1 = presence.}
\item{lon}{numeric, longitude values.}
\item{lat}{numeric, latitude values.}
}
}
\usage{
test
}
\description{
A dataset containing information on presence and absence, and
independent variables used to fit GLM models.
}
\examples{
data("test", package = "enmpa")
head(test)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.R
\docType{data}
\name{sel_fit}
\alias{sel_fit}
\title{Example of selected models fitted}
\format{
A list with four elements.
\describe{
\item{glms_fitted}{a fitted glm (ModelID_7).}
\item{selected}{a data.frame with results from evaluation of ModelID_7}
\item{data}{a data.frame containing information on presence and absence, and
independent variables used to fit GLM models.}
\item{weights}{a vector with the weights for observations.}
}
}
\usage{
sel_fit
}
\description{
An object of the class enmpa_fitted_models containing
fitted selected model(s) and the information from model evaluation for such
model(s).
}
\examples{
data("sel_fit", package = "enmpa")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_glm.R
\name{predict_glm}
\alias{predict_glm}
\title{Extension of glm predict to generate predictions of different types}
\usage{
predict_glm(
  model,
  newdata,
  data = NULL,
  extrapolation_type = "E",
  restricted_vars = NULL,
  type = "response"
)
}
\arguments{
\item{model}{a \code{glm} object.}

\item{newdata}{a data.frame or matrix with the new data to project the
predictions.}

\item{data}{data.frame or matrix of data used in the model calibration step.
Default = NULL.}

\item{extrapolation_type}{(character) to indicate extrapolation type of model.
Models can be transferred with three options: free extrapolation ('E'),
extrapolation with clamping ('EC'), and no extrapolation ('NE').
Default = 'E'.}

\item{restricted_vars}{(character) a vector containing the names of the
variables that will undergo clamping or no extrapolation. For clamping,
these variables are set to minimum and maximum values established for the
max and min values within calibration values. For no extrapolation, the
variables outside calibration limits became NA. If no specific names are
provided, the value is set to NULL by default, indicating that clamping (EC)
or no extrapolation (NE) will be applied to all variables. Ignore if
extrapolation_type = 'E'.}

\item{type}{(character) the type of prediction required. For a default
binomial model the default predictions are of log-odds (probabilities on
logit scale). The default, "response", returns predicted probabilities.}
}
\value{
A \code{SpatRaster} object or a vector with predictions.
}
\description{
Obtains predictions from a fitted generalized linear model objects. It also
allows the clamping option to restrict extrapolation in areas outside the
calibration area.
}
\examples{
# Load fitted model
data("sel_fit", package = "enmpa")

# Load raster layers to be projected
env_vars <- terra::rast(system.file("extdata", "vars.tif", package = "enmpa"))

# Prediction
pred <- predict_glm(sel_fit$glms_fitted$ModelID_7, newdata = env_vars,
                    data = sel_fit$data)
terra::plot(pred)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Classes.R
\name{enmpa_calibration}
\alias{enmpa_calibration}
\alias{new_enmpa_calibration}
\title{Constructor of S3 objects of class enmpa_calibration}
\usage{
new_enmpa_calibration(selected, summary, calibration_results, data,
                      partitioned_data, weights = NULL)
}
\arguments{
\item{selected}{date.frame with information about selected models.}

\item{summary}{data.frame a summary of statistics for all models.}

\item{calibration_results}{data.frame with results obtained from
cross-validation for all models.}

\item{data}{data.frame or matrix with the input data used for calibration.}

\item{partitioned_data}{a list of partition indices.}

\item{weights}{(numeric) a vector with the weights for observations.
Default = NULL.}
}
\value{
An S3 object of class \code{enmpa_calibration}.
}
\description{
Constructor of S3 objects of class enmpa_calibration
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enmpa.R
\docType{package}
\name{enmpa}
\alias{enmpa-package}
\alias{enmpa}
\title{enmpa: Ecological Niche Modeling using Presence-Absence Data}
\description{
\code{enmpa} contains a set of tools to perform detailed Ecological Niche Modeling
using presence-absence data.
}
\details{
It includes algorithms for data partitioning, model fitting, calibration,
evaluation, selection, and prediction. Other functions help to explore model
features as such variable response curves and variable importance.
}
\section{Main functions in \code{enmpa}}{

\code{\link{calibration_glm}}, \code{\link{evaluation_stats}},
\code{\link{fit_glms}}, \code{\link{fit_selected}},
\code{\link{get_formulas}}, \code{\link{independent_eval1}},
\code{\link{kfold_partition}}, \code{\link{model_selection}}
\code{\link{model_validation}}, \code{\link{niche_signal}},
\code{\link{optimize_metrics}}, \code{\link{predict_glm}},
\code{\link{predict_selected}}, \code{\link{response_curve}},
\code{\link{resp2var}}, \code{\link{var_importance}},
\code{\link{jackknife}}, \code{\link{plot_jk}}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/Luisagi/enmpa}
  \item Report bugs at \url{https://github.com/Luisagi/enmpa/issues}
}

}
\author{
\strong{Maintainer}: Luis F. Arias-Giraldo \email{lfarias.giraldo@gmail.com} (\href{https://orcid.org/0000-0003-4861-8064}{ORCID})

Authors:
\itemize{
  \item Luis F. Arias-Giraldo \email{lfarias.giraldo@gmail.com} (\href{https://orcid.org/0000-0003-4861-8064}{ORCID})
  \item Marlon E. Cobos \email{manubio13@gmail.com} (\href{https://orcid.org/0000-0002-2611-1767}{ORCID})
}

Other contributors:
\itemize{
  \item A. Townsend Peterson \email{town@ku.edu} (\href{https://orcid.org/0000-0003-0243-2379}{ORCID}) [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/niche_signal.R
\name{niche_signal}
\alias{niche_signal}
\alias{niche_signal_univariate}
\alias{niche_signal_permanova}
\title{Niche Signal detection using one or multiple variables}
\usage{
niche_signal(data, condition, variables, method = "univariate",
             permanova_method = "mahalanobis", iterations = 1000,
             set_seed = 1, verbose = TRUE, ...)

niche_signal_univariate(data, condition, variable, iterations = 1000,
                        set_seed = 1, verbose = TRUE)

niche_signal_permanova(data, condition, variables, permutations = 999,
                       permanova_method = "mahalanobis", verbose = TRUE, ...)
}
\arguments{
\item{data}{matrix or data.frame containing at least the following
information: a column representing \code{condition} (positive = 1 or
negative = 0), and one or more columns representing environmental variables.}

\item{condition}{(character) name of the column with numeric information
about detection (positive = 1 or negative = 0).}

\item{variables}{(character) vector of one or more names of columns to be
used as environmental variables. If \code{method} = "univariate", only one
variable is used; for \code{method} = "permanova", multiple variables can
be used.}

\item{method}{(character) name of the method to be used for niche comparison.
Default = "univariate".}

\item{permanova_method}{(character) name of the dissimilarity index to be
used as \code{method} in \code{\link[vegan]{adonis2}}. See all options in
\code{\link[vegan]{vegdist}}. Default = "mahalanobis".}

\item{iterations}{(numeric) number of iterations to be used in analysis.
Default = 1000. If \code{method} = "permanova", permutations = iterations - 1.}

\item{set_seed}{(numeric) integer value to specify a initial seed.
Default = 1.}

\item{verbose}{(logical) whether or not to print messages about the process.
Default = TRUE.}

\item{...}{other arguments to be passed to \code{\link[vegan]{adonis2}}.}

\item{variable}{(character) name of the column containing data to be used
as environmental variable.}

\item{permutations}{number of permutations to be performed.}
}
\value{
A list with results from analysis depending on \code{method}.
}
\description{
Identifies whether a signal of niche can be detected using one or multiple
variables. This is an implementation of the methods developed by
Cobos & Peterson (2022) \url{doi:10.17161/bi.v17i.15985} that
focuses on identifying niche signals in presence-absence data.
}
\examples{
# Load species occurrences and environmental data.
data("enm_data", package = "enmpa")
head(enm_data)

# Detection of niche signal using an univariate non-parametric test
sn_bio1 <- niche_signal(data = enm_data, variables = "bio_1",
                        condition = "Sp", method = "univariate")
sn_bio1

sn_bio12 <- niche_signal(data = enm_data, variables = "bio_12",
                         condition = "Sp", method = "univariate")
sn_bio12
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_validation.R
\name{model_validation}
\alias{model_validation}
\title{Model validation options}
\usage{
model_validation(formula, data, family = binomial(link = "logit"),
                 weights = NULL, cv = FALSE, partition_index = NULL,
                 k = NULL, dependent = NULL, n_threshold = 100,
                 keep_coefficients = FALSE, seed = 1)
}
\arguments{
\item{formula}{(character) \code{expression} to be used as a glm \code{formula}.}

\item{data}{data.frame with dependent and independent variables.}

\item{family}{a \code{family} object for models used by functions such as \code{glm}.
Default = binomial(link = "logit").}

\item{weights}{(numeric) vector with weights for observations. Default = NULL.}

\item{cv}{(logical) whether to use a k-fold cross validation for evaluation.
Default = FALSE.}

\item{partition_index}{list of indices for cross validation in k-fold.
Obtained with the function \code{\link{kfold_partition}}. Default = NULL.}

\item{k}{(numeric) number of folds for a new k-fold index preparation.
Ignored if \code{partition_index} is defined or if \code{cv} = FALSE. Default = NULL.}

\item{dependent}{(character) name of dependent variable. Ignore if
\code{cv} = FALSE. Default = NULL.}

\item{n_threshold}{(numeric) number of threshold values to be used for ROC.
Default = 100.}

\item{keep_coefficients}{(logical) whether to keep model coefficients.
Default = FALSE.}

\item{seed}{(numeric) a seed number. Default = 1.}
}
\value{
A data.frame with results from evaluation.
}
\description{
Model evaluation using entire set of data and a k-fold cross validation
approach. Models are assessed based on discrimination power (ROC-AUC),
classification ability (accuracy, sensitivity, specificity, TSS, etc.),
and the balance between fitting and complexity (AIC).
}
\examples{
# Load species occurrences and environmental data.
data("enm_data", package = "enmpa")
head(enm_data)

# Custom formula
form <- c("Sp ~ bio_1 + I(bio_1^2) + I(bio_12^2)")

# Model evaluation using the entire set of records
model_validation(form, data = enm_data)

# Model evaluation using a k-fold cross-validation (k = 3)
model_validation(form, data = enm_data, cv = TRUE, k = 3, dependent = "Sp")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_niche_signal.R
\name{plot_niche_signal}
\alias{plot_niche_signal}
\alias{plot_niche_signal_univariate}
\alias{plot_niche_signal_permanova}
\title{Plot Niche Signal results}
\usage{
plot_niche_signal(niche_signal_list, statistic = "mean",
                  variables = NULL, ellipses = FALSE, level = 0.99,
                  breaks = "Sturges", main = "", xlab = NULL, ylab = NULL,
                  h_col = "lightgray", h_cex = 0.8, lty = 2, lwd = 1,
                  l_col = c("blue", "black"), e_col = c("black", "red"),
                  pch = 19, pt_cex = c(1.3, 0.8), pt_col = c("black", "red"),
                  ...)

plot_niche_signal_univariate(niche_signal_univariate_list, statistic = "mean",
                             breaks = "Sturges", main = "", xlab = NULL,
                             ylab = "Frequency", h_col = "lightgray",
                             h_cex = 0.8, lty = 2, lwd = 1,
                             l_col = c("blue", "black"), ...)

plot_niche_signal_permanova(niche_signal_permanova_list, variables = NULL,
                           ellipses = FALSE, level = 0.99, main = "",
                           xlab = NULL, ylab = NULL,
                           e_col = c("black", "red"), lty = 2, lwd = 1,
                           pch = 19, pt_cex = c(1.3, 0.8),
                           pt_col = c("black", "red"), ...)
}
\arguments{
\item{niche_signal_list}{list of results from niche_signal.}

\item{statistic}{(character) name of the statistic for which results will be
explored when results come for univariate analysis. Default = "mean". Options
are: "mean", "median", "SD", and "range".}

\item{variables}{(character) name of variables to used in plots when
results come from analysis using the \code{permanova} method. The default, NULL,
uses the first two variables.}

\item{ellipses}{(logical) whether to use ellipses to represent all and
positive data when results come from PERMANOVA. The default, FALSE, plots
points instead.}

\item{level}{(numeric) value from 0 to 1 representing the limit of
the ellipse to be plotted. Default = 0.99.}

\item{breaks}{breaks in the histogram as in \code{\link[graphics]{hist}}.
Default = "Sturges".}

\item{main}{(character) title for plot. Default = "".}

\item{xlab}{(character) x axis label. Default = NULL. For results from
PERMANOVA, appropriate variable names are used.}

\item{ylab}{(character) y axis label. Default = NULL. For univariate
results, the default turn into "Frequency". For results from PERMANOVA,
appropriate variable names are used.}

\item{h_col}{a color to be used to fill the bars of histograms.
Default = "lightgray".}

\item{h_cex}{(numeric) value by which plotting text and symbols should be
magnified relative to the default in histograms. Default = 0.8.}

\item{lty}{(numeric) line type. See options in \code{\link[graphics]{par}}.
Default = 2. A vector of length = 2 can be used, in order, all and positive.}

\item{lwd}{(numeric) line width. See options in \code{\link[graphics]{par}}.
Default = 1. A vector of length = 2 can be used, in order, all and positive.}

\item{l_col}{line color for observed value of positives and confidence
intervals. Default = c("blue", "black").}

\item{e_col}{color of ellipse lines for all and positive data.
Default = c("black", "red").}

\item{pch}{point type. See options in \code{\link[graphics]{points}}.
Default = 19. A vector of length = 2 can be used, in order, all and positive.}

\item{pt_cex}{(numeric) value by which points will be magnified. Values
for all and positive points are recommended. Default = c(1.3, 0.8).}

\item{pt_col}{color for points. Values for all and positive points are
recommended. Default = c("black", "red").}

\item{...}{other plotting arguments to be used.}

\item{niche_signal_univariate_list}{list of results from niche_signal_univariate.}

\item{niche_signal_permanova_list}{list of results from niche_signal_permanova.}
}
\value{
A plot.
}
\description{
Plots to interpret results from niche_signal tests
(Cobos & Peterson (2022) \url{doi:10.17161/bi.v17i.15985}).
}
\examples{
# Load species occurrences and environmental data.
data("enm_data", package = "enmpa")
head(enm_data)

# Detection of niche signal using an univariate non-parametric test
sn_bio1 <- niche_signal(data = enm_data,
                        variables = "bio_1",
                        condition = "Sp",
                        method = "univariate")

plot_niche_signal(sn_bio1, variables = "bio_1")

sn_bio12 <- niche_signal(data = enm_data,
                         variables = "bio_12",
                         condition = "Sp",
                         method = "univariate")

plot_niche_signal(sn_bio12, variables = "bio_12")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calibration_glm.R
\name{calibration_glm}
\alias{calibration_glm}
\title{GLM calibration with presence-absence data}
\usage{
calibration_glm(data, dependent, independent, weights = NULL,
                response_type = "l", formula_mode = "moderate",
                minvar = 1, maxvar = NULL, user_formulas = NULL,
                cv_kfolds = 5, partition_index = NULL, seed = 1,
                n_threshold = 100, selection_criterion = "TSS",
                exclude_bimodal = FALSE, tolerance = 0.01,
                out_dir = NULL, parallel = FALSE,
                n_cores = NULL, verbose = TRUE)
}
\arguments{
\item{data}{data.frame or matrix of data to be used in model calibration.
Columns represent dependent and independent variables.}

\item{dependent}{(character) name of dependent variable.}

\item{independent}{(character) vector of name(s) of independent variable(s).}

\item{weights}{(numeric) a vector with the weights for observations.}

\item{response_type}{(character) a character string that must contain "l",
"p", "q" or a combination of them. l = lineal, q = quadratic,
p = interaction between two variables. Default = "l".}

\item{formula_mode}{(character) a character string to indicate the strategy to
create the formulas for candidate models. Options are: "light", "moderate",
"intensive", or "complex". Default = "moderate". "complex" returns only the
most complex formula defined in \code{response_type}.}

\item{minvar}{(numeric) minimum number of independent variables in formulas.}

\item{maxvar}{(numeric) maximum number of independent variables in formulas.}

\item{user_formulas}{(character) vector  with formula(s) to test.
Default = NULL.}

\item{cv_kfolds}{(numeric) number of folds to use for k-fold
cross-validation exercises. Default = 5. Ignored if \code{partition_index}
is defined.}

\item{partition_index}{list of indices for cross-validation in k-fold. The
default, NULL, uses the function \code{\link{kfold_partition}}.}

\item{seed}{(numeric) a seed for k-fold partitioning.}

\item{n_threshold}{(logical) number of threshold values to produce
evaluation metrics. Default = 100.}

\item{selection_criterion}{(character) criterion used to select best models,
options are "TSS" and "ESS". Default = "TSS".}

\item{exclude_bimodal}{(logical) whether to filter out models with one or
more variables presenting concave responses. Default = FALSE.}

\item{tolerance}{(numeric) value to modify the limit value of the metric
used to filter models during model selection if none of the models meet
initial considerations. Default = 0.01}

\item{out_dir}{(character) output directory name to save the main calibration
results. Default = NULL.}

\item{parallel}{(logical) whether to run on parallel or sequential.
Default = FALSE.}

\item{n_cores}{(numeric) number of cores to use. Default = number of free
processors - 1.}

\item{verbose}{(logical) whether to print messages and show progress bar.
Default = TRUE}
}
\value{
An object of the class enmpa_calibration containing: selected models,
a summary of statistics for all models, results obtained in cross-validation
for all models, original data used, weights, and data-partition indices used.
}
\description{
Creates candidate models based on distinct parameter settings, evaluates
models, and selects the ones that perform the best.
}
\details{
Model evaluation is done considering the ability to predict presences and
absences,as well as model fitting and complexity. Model selection consists
of three steps: 1) a first filter to keep the models with ROC AUC >= 0.5
(statistically significant models), 2) a second filter to maintain only
models that meet the \code{selection_criterion} ("TSS": TSS >= 0.4; or "ESS":
maximum Accuracy - \code{tolerance}), and 3) from those, pick the ones with
delta AIC <= 2.

\code{formula_mode} options determine what strategy to iterate the predictors
defined in \code{type} for creating models:
\itemize{
\item \strong{light}.-- returns simple iterations of complex formulas.
\item \strong{moderate}.-- returns a comprehensive number of iterations.
\item \strong{intensive}.-- returns all possible combination. Very time-consuming for
6 or more independent variables.
\item \strong{complex}.-- returns only the most complex formula.
}
}
\examples{
# Load species occurrences and environmental data.
data("enm_data", package = "enmpa")
head(enm_data)

# Calibration using linear (l), quadratic (q), products(p) responses.
cal_res <- calibration_glm(data = enm_data, dependent = "Sp",
                           independent = c("bio_1", "bio_12"),
                           response_type = "lpq", formula_mode = "moderate",
                           selection_criterion = "TSS", cv_kfolds = 3,
                           exclude_bimodal = TRUE, verbose = FALSE)
print(cal_res)
summary(cal_res)

head(cal_res$calibration_results)
head(cal_res$summary)
head(cal_res$selected)
head(cal_res$data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Methods.R
\name{print}
\alias{print}
\alias{print.enmpa_calibration}
\alias{print,enmpa_calibration-method}
\alias{print,enmpa_fitted_models-method}
\alias{print.enmpa_fitted_models}
\title{Print a short version of elements in 'calibration' and 'fitted models' objects}
\usage{
\method{print}{enmpa_calibration}(x, ...)

\method{print}{enmpa_fitted_models}(x, ...)
}
\arguments{
\item{x}{object of enmpa_fitted_models or enmpa_calibration}

\item{...}{additional arguments affecting the summary produced. Ignored in
these functions.}
}
\description{
Print a short version of elements in 'calibration' and 'fitted models' objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jackknife.R
\name{jackknife}
\alias{jackknife}
\title{Jackkniffe test for variable contribution}
\usage{
jackknife(data, dependent, independent, user_formula = NULL, cv = 3,
          response_type = "l", weights = NULL)
}
\arguments{
\item{data}{data.frame or matrix of data to be used in model calibration.
Columns represent dependent and independent variables.}

\item{dependent}{(character) name of dependent variable.}

\item{independent}{(character) vector of name(s) of independent variable(s).}

\item{user_formula}{(character) custom formula to test. Default = NULL.}

\item{cv}{(numeric) number of folds to use for k-fold
cross-validation exercises. Default = 3.}

\item{response_type}{(character) a character string that must contain "l",
"p", "q" or a combination of them. l = lineal, q = quadratic,
p = interaction between two variables. Default = "l".}

\item{weights}{(numeric) a vector with the weights for observations.}
}
\value{
list including model performance metrics (ROC-AUC, TSS, AICc, and
deviance) for the complete model, model performance when excluding a specific
predictor, and the independent contribution of that predictor to the model.
}
\description{
The Jackknife function providing a detailed reflection of the  impact of each
variable on the overall model, considering four difference measures:
ROC-AUC, TSS, AICc, and Deviance.
}
\examples{

# Load data
data("enm_data", package = "enmpa")

jk <- jackknife(data = enm_data,
                dependent = "Sp",
                independent = c("bio_1", "bio_12"),
                user_formula = NULL,
                cv = 3, response_type = "lpq")
jk

# plot JK's results
plot_jk(jk, metric = "TSS")
plot_jk(jk, metric = "ROC_AUC")
plot_jk(jk, metric = "AIC")
plot_jk(jk, metric = "Residual_deviance")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_selected.R
\name{predict_selected}
\alias{predict_selected}
\title{Predictions for the models selected after calibration}
\usage{
predict_selected(fitted, newdata, extrapolation_type = "E",
                 restricted_vars = NULL, type = "response", consensus = TRUE)
}
\arguments{
\item{fitted}{an enmpa-class\verb{fitted models} object obtained using the
functions \code{\link{fit_selected}}.}

\item{newdata}{a \code{SpatRaster}, data.frame, or matrix with the new data on
which to predict.}

\item{extrapolation_type}{(character) to indicate extrapolation type of model.
Models can be transferred with three options: free extrapolation ('E'),
extrapolation with clamping ('EC'), and no extrapolation ('NE').
Default = 'E'.}

\item{restricted_vars}{(character) a vector containing the names of the
variables that will undergo clamping or no extrapolation. For clamping,
these variables are set to minimum and maximum values established for the
max and min values within calibration values. For no extrapolation, the
variables outside calibration limits became NA. If no specific names are
provided, the value is set to NULL by default, indicating that clamping (EC)
or no extrapolation (NE) will be applied to all variables. Ignore if
extrapolation_type = 'E'.}

\item{type}{(character) the type of prediction required. For a default
binomial model the default predictions are of log-odds (probabilities on
logit scale). The default, "response", returns predicted probabilities.}

\item{consensus}{(logical) valid if \code{newdata} is a \code{SpatRaster}, whether to
produce consensus results obtained by combining the predictions from the
collection of selected models. By default consensuses are calculated using
the mean, median, variance, and weighted average using the AIC weights.
Default = TRUE.}
}
\value{
A list with predictions of selected models on the \code{newdata} and fitted
selected model(s). Consensus predictions are added if multiple selected
models exits and if \code{newdata} is a \code{SpatRaster} object.
}
\description{
Wrapper function that facilitates the prediction of those models
selected as the most robust. In addition, it allows the calculation of
consensus models, when more than one model are selected.
}
\examples{
# Load a fitted selected model
data(sel_fit, package = "enmpa")

# Load raster layers to be projected
env_vars <- terra::rast(system.file("extdata", "vars.tif", package = "enmpa"))

# Predictions (only one selected mode, no consensus required)
preds <- predict_selected(sel_fit, newdata = env_vars, consensus = FALSE)

# Plot prediction
terra::plot(preds$predictions)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_jk.R
\name{plot_jk}
\alias{plot_jk}
\title{Jackkniffe plot for variable contribution}
\usage{
plot_jk(x, metric = "ROC_AUC", legend = TRUE,
        colors = c("cyan", "blue", "red"),
        xlab = NULL, main = NULL)
}
\arguments{
\item{x}{list output from \code{\link{jackknife}}.}

\item{metric}{(character) model metric to plot. Default = "ROC_AUC".}

\item{legend}{(logical) whether to add legend. Default = TRUE.}

\item{colors}{(character) vector of colors.
Default = c("cyan", "blue", "red").}

\item{xlab}{(character) a label for the x axis.}

\item{main}{(character) main title for the plot.}
}
\description{
The jackknife figure shows the impact of each variable on the full model,
providing detailed information about the function and significance of each
variable. Light blue indicates the impact on the model if the variable is not
included, while dark blue indicates the independent contribution of the
variable to the model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.R
\docType{data}
\name{cal_res}
\alias{cal_res}
\title{Example of results obtained from GLM calibration using enmpa}
\format{
An object of class enmpa_calibration with results from the function
\verb{\link{calibration_glm}}.
}
\usage{
cal_res
}
\description{
An object of the class enmpa_calibration storing the results from GLM
calibration.
}
\examples{
data("cal_res", package = "enmpa")

str(cal_res)
}
\keyword{datasets}
